.TH CAMLC 1 "Tue Aug 13"  

.SH NAME
camlc \- The CAML Light compiler


.SH SYNOPSIS
.B camlc
[
.B \-cvg
]
[
.BI \-o \ exec-file
]
[
.BI \-I \ lib-dir
]
[
.BI \-O \ module-set
]
[
.BI \-@ \ response-file
]
.I filename ...


.SH DESCRIPTION

.B camlc
is the standalone compiler for CAML Light, an implementation of the ML
language. (For interactive use of CAML Light, see camllight(1).)
.B camlc
has a command-line interface similar to the C compiler cc(1).
It accepts several types of arguments: source programs, module
interfaces, compiled object programs.

Arguments ending in `.mli' are taken to be the sources of module
interfaces, declaring public global identifiers, concrete types,...
From the file 
.I xxx.mli,
camlc produces a compiled interface, in the file
.I xxx.zi.

Arguments ending in `.ml' are taken to be the sources of module
implementations, defining global identifiers, concrete types, and
giving expressions to be evaluated. From the file
.I xxx.ml,
camlc produces a compiled object code, in the file
.I xxx.zo,
and a compiled interface
.I xxx.zi.
The compiled interface is not produced if
the interface of the module is explicitly given, that is, if file
.I xxx.mli
exists; then, the module implementation
.I xxx.ml
is checked against the compiled interface
.I xxx.zi,
which is assumed to exist.
However, if no source interface exists, then camlc produces a compiled
interface
.I xxx.zi,
that exports everything that is defined in the implementation
.I xxx.ml.

Arguments ending in `.zo' are taken to be compiled object code. These
files are linked together, along with the object code files obtained
by compiling `.ml' arguments (if any), and parts of the CAML Light library,
to produce a standalone executable program. The order in which `.zo'
and `.ml' arguments are presented on the command line is relevant:
global identifiers are initialized in that order at run-time,
and it is a link-time error to use a global identifier before having
initialized it.



The output of the linking phase is a file containing bytecode
executable by the CAML Light bytecode interpreter. On most systems,
the bytecode file can be run directly: it has the executable bit set,
and it manages to launch the bytecode interpreter by itself. If this
fails, the bytecode file must be given as first argument to the
bytecode interpreter (/usr/local/lib/caml-light/run, with the default
installation).



.SH OPTIONS

The following options are interpreted by camlc.

.TP
.B \-c
Compile only. Suppresses the linking phase of the compilation.
.TP
.BI \-o \ exec-file
Specifies the name of the output executable file. The default is
`a.out'.
.TP
.BI \-I \ lib-dir
Adds lib-dir to the path of directories searched for compiled
interface files `.zi' and compiled object files `.zo'. The default is
to search in the CAML Light library first (usually
/usr/local/lib/caml-light), then in the current directory. A
directory added by -I is searched first. This option takes effect for
the compilation of subsequent `.ml' and `.mli' files, as well as for
the final linking.
.TP
.BI \-O \ module-set
Specifies which set of standard library modules are to be
implicitly "opened" at the beginning of a compilation, and linked with
the user's object code. (See below the section on modules for an
explanation of "opened" modules). There are three module sets
currently available: "cautious" and "fast", and "none".
"Cautious" and "fast" both provide the same standard operations on
integers, floating-point numbers, characters and character
strings, polymorphic lists and vectors, ..., as well as basic
input/output, exception handling, ... Modules from the "cautious"
set perform range and bound checking on string and vector
operations, and various sanity checks on their arguments as well,
while those from the "fast" set don't check anything. The default
is "cautious". The "-O none" option suppresses all default module
opening; compilation starts in an (almost) empty environment, and
no library code is linked. It is not of general use, except to
compile the standard library itself.
.TP
.B \-g
Add debugging information at the end of the executable bytecode file
produced by the linking phase.
.TP
.B \-v
Prints the compiler version number.

.TP
.BI \-@ \ response-file
Process the files whose names are listed in file
.I response-file,
just as if these names appeared on the command line. File names in
.I response-file
are separated by spaces and/or newlines. This option serves to
overcome silly limitations on the length of the command line.

.SH OVERVIEW OF THE MODULE SYSTEM

CAML Light uses a Modula-2-like module system to guarantee type safety in
the presence of separate compilation. Each CAML source file (an
"implementation", in the following) is associated an interface file,
describing the global identifiers and concrete types it publicizes.
Here is a sample interface source file, stack.mli:

    type 'a optional = None | Some of 'a;;
        (* public type *)
    type 'a stack;;
        (* abstract type *)
    exception Stack_error of string;;
    value empty : 'a stack
      and push : 'a -> 'a stack -> 'a stack
      and pop  : 'a stack -> 'a * 'a stack
      and top  : 'a stack -> 'a optional;;

Here is an implementation (stack.ml) that meets the interface above:

    type 'a stack = Stack of 'a list;;
    let stack_error s = raise (Stack_error s);;
    let empty = Stack [];;
    let push x (Stack s) = Stack (x::s);;
    let pop = function
        Stack [] -> raise (Stack_error "empty")
      | Stack (a::s) -> (a, Stack s);;
    let top = function
        Stack [] -> None
      | Stack (a::s) -> Some a;;

The global "stack_error", and the constructor "Stack", are not
exported, hence remains private to the implementation. The
implementation can freely make use of globals that are defined in the
interface, e.g. the constructors None and Some , and the
exception Stack_error. However, globals that are only declared, such
as the type stack, and the push, pop and top functions, must be
defined by the implementation, and cannot be referenced before being defined.

It is also possible to let the CAML Light compiler generate a compiled
interface (stack.zi) from the implementation. Then, it assumes all
globals are to be exported. In the example above, we would get a
compiled interface corresponding to:

    type 'a optional = None | Some of 'a
     and 'a stack = Stack of 'a list;;
    exception Stack_error of string;;
    value stack_error : string -> 'a
      and empty : 'a stack
      and push : 'a -> 'a stack -> 'a stack
      and pop  : 'a stack -> 'a * 'a stack
      and top  : 'a stack -> 'a optional;;

Two mechanisms are provided to refer to a global defined in another
module. The first is to use qualified identifiers, of the format
`module__name':

    match stack__top(stack__push 1 stack__empty) with
        stack__None -> ...
      | stack__Some x -> ...

The other way is to rely on automatic qualification of identifiers.
The system maintains a list of "default" modules, called the currently
opened modules, and whenever it encounters an identifier without the
`module__' part, it searches through the opened modules, to find which
one defines this identifier. The identifier is then qualified with the
name of its defining module. Searched modules always include the
module being compiled (searched first), and some library modules of
general use (see the explanation for the -O flag above). In addition,
two directives are provided to add and to remove modules from the list
of opened modules:

    #open "module";;  (* to add "module", in front *)
    #close "module";; (* to remove "module" *)

So we can rewrite the example above as:

    #open "stack";;

    match top(push 1 empty) with
        None -> ...
      | Some x -> ...

If two modules, say "mod1" and "mod2", define both a global value, say
"f", then "f" in a client module "client" resolves to "mod1__f" if
"mod1" is opened but not "mod2", or if "mod1" has been opened more
recently than "mod2".


.SH THE STANDARD LIBRARY

A number of primitives and predefined functions are available to the
user. They are documented in the .mli files in the library directory
(usually /usr/local/lib/caml-light). Some of these libraries are
implicitly "opened" at the beginning of a compilation, and implicitly
linked with the user's object code. Without the "-O" option, or with
"-O cautious", the default modules are:

    exc  bool  cstring  cchar  cvect  list
    pair  ref  float  int  eq  io

With "-O fast", the default modules are:

    exc  bool  fstring  fchar  fvect  list
    pair  ref  float  int  eq  io


.SH FILES

.ta 3.5i


/usr/local/lib/caml-light	standard library
.br
/usr/local/lib/caml-light/run	the bytecode interpreter
.br
/usr/local/bin/camlc	the compiler
.br
/usr/local/bin/camllight	the toplevel
.br



.SH BUGS

There are very few debugging tools.

Polymorphic references, and more generally mutable concrete types, are
not safe: it is possible to create polymorphic references through a
functional encoding; and pattern-matching on mutable structures is not
compiled correctly.

Structured input/output is not type-safe; nothing prevents the user
from writing data of one type and reading it with another type.

Various size limitations are not checked by the compiler: the size of
a local environment should not exceed 256; the number of constructors
in a concrete type should not exceed 253; the number of global values
should be less than 65536; etc.

The module system is not foolproof, since it relies on the (base) names
of the files. Anything can happen if the user monkeys with the interface
path, or renames compiled interfaces.
