.TH CAMLYACC 1 _DATE _FOOTER _HEADER

.SH NAME
camlyacc \- a parser generator for Caml Light

.SH SYNOPSIS
.B camlyacc
[
.B \-vs
]
[
.BI \-b prefix
]
.I grammar.xxx

.SH DESCRIPTION

.B camlyacc
produces a parser from a context-free grammar specification
with attached semantic actions, in the style of yacc(1). Assuming the
input file is
.I grammar.xxx,
camlyacc outputs the two files
.I grammar.mli
and
.I grammar.ml.
The two output files must be compiled by Caml Light, and
linked with the standard library named ``parsing''.

The generated module defines one parsing function per entry point in the
grammar. These functions have the same names as the entry points.
Parsing functions take as arguments a lexical analyzer (a function
from lexer buffers to tokens) and a lexer buffer, and return the
semantic attribute of the corresponding entry point. Lexical analyzer
functions are usually generated from a lexer specification by the
camllex(1) program. Lexer buffers are an abstract data type
implemented in the standard library ``lexing''. Tokens are values from
the concrete datatype ``token'', defined in the interface file
grammar.mli produced by camlyacc.

.SH GRAMMAR FORMAT

Grammar specifications have the following format:

%{
.br
.I Header
.br
%}
.br
.I Declarations
.br
%%
.br
.I Rules
.br
%%
.br
.I Trailer

.SS Rules

The syntax for rules is as usual:

.I Nonterminal
:
.I symbol ... symbol
{
.I semantic action
}
.br
            |
.I ...
.br
            |
.I symbol ... symbol
{
.I semantic action
}
.br
            ;

Rules can also contain the `%prec symbol' directive in the right-hand
side part, to override the default precedence and
associativity of the rule with the precedence and associativity
of the given symbol.

Semantic actions are arbitrary Caml Light expressions, that are
evaluated to produce the semantic attribute attached to the defined
nonterminal. The semantic actions can access the semantic attributes
of the symbols in the right-hand side of the rule with the `$'
notation: $1 is the attribute for the first (leftmost) symbol, $2 is
the attribute for the second symbol, etc.

.SS Header and trailer

The header and the trailer sections are Caml Light code that is copied
as is into file
.I grammar.ml.
Both sections are optional. The header goes at the beginning of the
output file; it usually contains `#open' directives required by the
semantic actions of the rules. The trailer goes at the end of the
output file.

.SS Declarations

Declarations are given one per line, in arbitrary order. They all
start with a `%' sign.

.TP
.B %token symbol ... symbol
Declare the given symbols as tokens (terminal symbols). These symbols
are added as constant constructors for the `token' concrete datatype.
.TP
.B %token <type> symbol ... symbol
Declare the given symbols as tokens with an attached attribute of the
given type. These symbols are added as constructors with arguments
of the given type for the `token' concrete datatype.
.TP
.B %start symbol ... symbol
Declare the given symbols as entry points for the grammar. For each
entry point, a parsing function with the same name is defined in the
output module. Non-terminals that are not declared as entry points
have no such parsing function. Start symbols must be given a type with
the `%type' directive below.
.TP
.B %type <type> symbol ... symbol
Specify the type of the semantic attributes for the given symbols.
This is mandatory for start symbols only. Other nonterminal symbols need
not be given types by hand: these types will be inferred when running
the output files through the Caml Light compiler (unless the \-s
option is in effect).
.TP
.B %left symbol ... symbol
.TP
.B %right symbol ... symbol
.TP
.B %nonassoc symbol ... symbol
Associate precedences and associativities to the given symbols. All
symbols on the same line are given the same precedence. They have
higher precedence than symbols declared before in a `%left', `%right'
or `%nonassoc' line. They have lower precedence than symbols declared
after in a `%left', `%right' or `%nonassoc' line. The symbols
are declared to associate to the left (%left), to the right (%right),
or to be non-associative (%nonassoc). The symbols are usually tokens.
They can also be dummy nonterminals, for use with the `%prec'
directive inside the rules.

.SH OPTIONS

.TP
.B -v
Generate a description of the parsing tables and a report on conflicts
resulting from ambiguities in the grammar. The description is put in
file grammar.output.

.TP
.B -s
Generate a grammar.ml file with smaller phrases. Semantic actions are
presented in the grammar.ml output file as one large vector of functions. By
default, this vector is built by a single phrase. When the grammar is
large, or contains complicated semantic actions, the resulting phrase
may require large amounts of memory to be compiled by Caml Light. With
the \-s option, the vector of actions is constructed incrementally,
one phrase per action. This lowers the memory requirements for the
compiler, but it is not possible anymore to infer the types of
nonterminal symbols: typechecking is turned off on symbols that do not
have a type specified by a %type directive.

.TP
.BI -b prefix
Name the output files
.I prefix.ml,
.I prefix.mli,
.I prefix.output,
instead of the default naming convention.

.SH A COMPLETE EXAMPLE

The all-time favorite: a desk calculator.

File parser.mly (the grammar specification):

    %token <int> INT
    %token PLUS MINUS TIME DIV
    %token LPAREN RPAREN
    %token EOL

    %right PLUS MINUS
    %right TIMES DIV
    %nonassoc UMINUS

    %start Main
    %type <int> Main

    %%

    Main:
        Expr EOL                { $1 }
    ;
    Expr:
        INT                     { $1 }
      | LPAREN Expr RPAREN      { $1 }
      | Expr PLUS Expr          { $1 + $3 }
      | Expr MINUS Expr         { $1 - $3 }
      | Expr TIMES Expr         { $1 * $3 }
      | Expr DIV Expr           { $1 / $3 }
      | MINUS Expr %prec UMINUS { - $2 }
    ;

File lexer.mll (the lexer specification):

    {
    #open "parser";;        (* Defines the token type *)
    exception Eof;;
    }

    rule Token = parse
        [` ` `\\t`]     { Token lexbuf }
      | [`\\n` `\\r`]    { EOL }
      | [`0`-`9`]+     { INT(int_of_string
                              (get_lexeme lexbuf)) }
      | `+`            { PLUS }
      | `-`            { MINUS }
      | `*`            { TIMES }
      | `/`            { DIV }
      | `(`            { LPAREN }
      | `)`            { RPAREN }
      | eof            { raise Eof }
    ;;

File calc.ml (the main file):

    try
      let lexbuf = lexing__create_lexer_channel std_in in
      while true do
        let result = parser__Main lexer__Token lexbuf in
          print_int result; print_newline(); flush std_out
      done
    with Eof ->
      ()
    ;;

How to compile everything:

    camllex lexer.mll
    camlyacc parser.mly
    camlc -c parser.mli
    camlc -c lexer.ml
    camlc -c parser.ml
    camlc -c calc.ml
    camlc -o calc lexing.zo parsing.zo lexer.zo parser.zo calc.zo

.SH SEE ALSO
camllex(1)
.br
.I
YACC - Yet Another Compiler Compiler,
.P
by S. C. Johnson.
.br
.I
Compilers - Principles, Techniques, and Tools
.P
(chapter 4), by A. V. Aho, R. Sethi, and J. D. Ullman.

.SH DIAGNOSTICS

The number of reduce-reduce conflicts, shift-reduce conflicts, and
unused rules is reported on the standard output. A more detailed
report can be obtained with the \-v flag.

.SH BUGS

The %{ ... %} section of the grammar definition is copied to
.I grammar.ml
only, not to
.I grammar.mli.
As a consequence, type
expressions in the `%token' and `%type' declarations must contain
fully qualified type names (e.g. mymodule__mytype) for all types except
standard built-in types, even if the proper `#open' directives
(e.g. #open "mymodule";;) were given in the %{ ... %} section.

Error recovery is not implemented.

Type checking is performed when compiling the .ml output file, not
when compiling the grammar. This does not break type safety, since the
.ml output file is well-typed iff the input grammar is well-typed.
However, this results in type error messages that are somehow hard to
relate to the input grammar.
