(* The CAML Light runtime system *)

#open "mec";;
#open "rules_c";;

(*
 Define ANSI if your C compiler is ANSI-standard compliant.
 This turns function prototypes on.
 asflags_opti is needed for optimizing assemblers (Mips, Sparc).
 Older assemblers may choke on it; just remove its definition, then.
*)

(* Standard C compiler *)
(*
let cc = ref "cc";;
let cflags_user = ref "";;
let ansi = ref false;;

let cflags_opti = "-O -DPROFIL";;
let cflags_debug = "-g -DDEBUG";;
let cflags_trace = "-DTRACE";;
let asflags_opti = "-j";;
*)
(* The Gnu C compiler *)

let cc = ref "gcc";;
let cflags_user = ref "";;
let ansi = ref true;;

let cflags_opti = "-O -fstrength-reduce";; (* -DPROFIL *)
let cflags_debug = "-DDEBUG";;
let cflags_trace = "-DTRACE -DPROFIL";;
let cflags_profil = "-O -fstrength-reduce -DPROFIL";;
let asflags_opti = "-j";;

(* Parse options now.  We will use the value of [cc], [ansi], and
   [cflags_user]. *)
let targets =
  get_args [str_option "-cc" cc;
            str_option "-cflags" cflags_user;
            flag_option "-ansi" ansi]
;;
let cflags = if !ansi then "-DANSI " ^ !cflags_user else !cflags_user;;


let Prims = ["interp.c"; "equal.c"; "ints.c"; "floats.c"; "strings.c";
             "io.c"; "intext.c"; "hash.c"; "unix.c"; "meta.c"; "signals.c";
             "threads.c"; "parsing.c"]
;;
let Runtime = ["freelist.c"; "i_interp.c"; "minor_gc.c"; "memory.c";
               "mem_misc.c"; "fail.c"; "main.c"; "major_gc.c"; "semaph.c";
               "stacks.c"; "prims.c"; "roots.c"; "callback.c"; "profil.c"]
;;
let Debug = ["debugger.c"];;
let Objs = ["interp.o"; "i_interp.o"; "equal.o"; "ints.o"; "floats.o";
            "strings.o"; "io.o"; "roots.o";
            "intext.o"; "hash.o"; "unix.o"; "meta.o"; "signals.o";
            "freelist.o"; "minor_gc.o"; "memory.o"; "mem_misc.o";
            "fail.o"; "main.o"; "stacks.o"; "threads.o"; "major_gc.o";
            "callback.o"; "semaph.o"; "parsing.o"]
;;


let strip = strip_suffix [".c"; ".d.o"; ".t.o"; ".p.o"; ".o"];;
let c name = (strip name) ^ ".c";;
let obj name = (strip name) ^ ".o";;
let dobj name = (strip name) ^ ".d.o";;
let tobj name = (strip name) ^ ".t.o";;
let pobj name = (strip name) ^ ".p.o";;

let Debug_objs = ["debugger.d.o"] @ map dobj Objs ;;
let Trace_objs = ["debugger.t.o"; "profil.t.o"; "kern_timestamp.o"]
                 @ map tobj Objs ;;
let Profil_objs =  ["profil.p.o"; "kern_timestamp.o"] @ map pobj Objs;;

rule "all" ["zrun"; "zrun.d"; "zrun.t"; "zrun.p"] [];;

force "clean" []
  [command (["rm -f"] @ Objs @ Debug_objs @ Trace_objs @ Profil_objs);
   command ["rm -f interp.s interp.s2 prims.o prims.d.o zrun zrun.d zrun.t"]]
;;

force "scratch" ["clean"]
  [command ["rm -f primitives prims.c opnames.h"]]
;;

rule "zrun" ("prims.o" :: Objs)
  [command ([!cc; cflags; cflags_opti; "-o zrun"]
            @ Objs @ ["prims.o -lm -lthreads -lmach"])]
;;

rule "zrun.d" ("prims.d.o" :: Debug_objs)
  [command ([!cc; cflags; cflags_debug; "-o zrun.d"]
            @ Debug_objs @ ["prims.d.o -lm -lthreads -lmach"])]
;;

rule "zrun.t" ("prims.t.o" :: Trace_objs)
  [command ([!cc; cflags; cflags_debug; cflags_trace; "-o zrun.t"]
            @ Trace_objs @ ["prims.t.o -lm -lthreads -lmach"])]
;;

rule "zrun.p" ("prims.p.o" :: Profil_objs)
  [command ([!cc; cflags; cflags_profil; "-o zrun.p"]
            @ Profil_objs @ ["prims.p.o -lm -lthreads -lmach"])]
;;

rule1 "interp.o" (fun _ ->
  try rules "patch";
      (["interp.c"; "patch"],
       [command [!cc; cflags; cflags_opti; "-S interp.c"];
        command ["sed -f patch interp.s >interp.s2"];
        command ["as"; asflags_opti; "-o interp.o interp.s2"]])
  with Missing ->
      (["interp.c"],
       [command [!cc; cflags; cflags_opti; "-c interp.c"]]))
;;

rule "kern_timestamp.o" ["kern_timestamp.s"]
  [command ["as"; asflags_opti; "-o kern_timestamp.o kern_timestamp.s"]]
;;

rule1 "*.d.o" (fun name ->
  (c name :: (get_deps name),
   [command [!cc; cflags; cflags_debug; "-c"; c name; "-o"; dobj name]]))
;;

rule1 "*.t.o" (fun name ->
  (c name :: (get_deps name),
   [command [!cc; cflags; cflags_debug; cflags_trace; "-c"; c name;
                  "-o"; tobj name]]))
;;

rule1 "*.p.o" (fun name ->
  (c name :: (get_deps name),
   [command [!cc; cflags; cflags_profil; "-c"; c name; "-o"; pobj name]]))
;;

rules_c__default [!cc; cflags; cflags_opti];;

rule "primitives" Prims
  [command (["sed -n -e 's|value \([a-zA-Z0-9_]*\).*/\* ML \*/.*|\1|p'"]
            @ Prims @ ["> primitives2"]);
   command ["sh -c 'if cmp -s primitives primitives2;";
            "then rm primitives2;";
            "else mv primitives2 primitives;";
            "fi'"]]
;;

rule "prims.c" ["primitives"]
  [command ["(echo '#include \"mlvalues.h\"';";
             "echo '#include \"cprim.h\"';";
             "sed -e 's/.*/extern value &();/' primitives;";
             "echo 'c_primitive cprim[] = {';";
             "sed -e 's/.*/  &,/' primitives;";
             "echo '  0 };') > prims.c"]]
;;

rule "opnames.h" ["instruct.h"]
  [command ["sed -e '/^#/d' -e '/\\/\\*/d'";
                "-e 's/enum /char * names_of_/g'";
                "-e 's/{/[] = {/g'";
                "-e 's/\\([A-Z][A-Z_0-9]*\\)/\"\\1\"/g' instruct.h";
              ">opnames.h"]]
;;

force "depend" ["opnames.h"]
  [command ["mv Mecfile.ml Mecfile.ml.bak"];
   command (["(sed -n -e '1,/^(\\* DO NOT DELETE THIS LINE \\*)/p' Mecfile.ml.bak;";
              "cc"; cflags; cflags_opti; cflags_debug; cflags_trace; cflags_profil; "-M"]
            @ Prims @ Runtime @ Debug
            @ ["| sed -e 's,\\./,,'";
                     "-e '/\\.c$/d'";
                     "-e '/\\//d'";
                     "-e 's/^\\([^:]*\\)\\.o: \\([^:]*\\)\\.h$/deps [\"\\1.o\"; \"\\1.d.o\"; \"\\1.t.o\"; \"\\1.p.o\"] [\"\\2.h\"];;/';";
                "echo 'go \"all\" targets;;') > Mecfile.ml"])]
;;

(* Everything that goes beyond this line will be erased without warning. *)
(* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ *)
(* DO NOT DELETE THIS LINE *)
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["control.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["cprim.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["mlvalues.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["config.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["debugger.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["mlvalues.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["fail.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["mlvalues.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["instruct.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["memory.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["gc.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["mlvalues.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["major_gc.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["control.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["private.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["control.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["debugger.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["freelist.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["control.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["mlvalues.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["intext.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["major_gc.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["mlvalues.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["profil.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["semaph.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["control.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["mem_misc.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["mlvalues.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["private.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["mem_misc.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["mlvalues.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["private.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["signals.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["stacks.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["mlvalues.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["strings.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["unalignd.h"];;
deps ["interp.o"; "interp.d.o"; "interp.t.o"; "interp.p.o"] ["mem_misc.h"];;
deps ["equal.o"; "equal.d.o"; "equal.t.o"; "equal.p.o"] ["control.h"];;
deps ["equal.o"; "equal.d.o"; "equal.t.o"; "equal.p.o"] ["fail.h"];;
deps ["equal.o"; "equal.d.o"; "equal.t.o"; "equal.p.o"] ["mlvalues.h"];;
deps ["equal.o"; "equal.d.o"; "equal.t.o"; "equal.p.o"] ["config.h"];;
deps ["equal.o"; "equal.d.o"; "equal.t.o"; "equal.p.o"] ["memory.h"];;
deps ["equal.o"; "equal.d.o"; "equal.t.o"; "equal.p.o"] ["gc.h"];;
deps ["equal.o"; "equal.d.o"; "equal.t.o"; "equal.p.o"] ["mlvalues.h"];;
deps ["equal.o"; "equal.d.o"; "equal.t.o"; "equal.p.o"] ["major_gc.h"];;
deps ["equal.o"; "equal.d.o"; "equal.t.o"; "equal.p.o"] ["control.h"];;
deps ["equal.o"; "equal.d.o"; "equal.t.o"; "equal.p.o"] ["private.h"];;
deps ["equal.o"; "equal.d.o"; "equal.t.o"; "equal.p.o"] ["control.h"];;
deps ["equal.o"; "equal.d.o"; "equal.t.o"; "equal.p.o"] ["debugger.h"];;
deps ["equal.o"; "equal.d.o"; "equal.t.o"; "equal.p.o"] ["mlvalues.h"];;
deps ["equal.o"; "equal.d.o"; "equal.t.o"; "equal.p.o"] ["freelist.h"];;
deps ["equal.o"; "equal.d.o"; "equal.t.o"; "equal.p.o"] ["control.h"];;
deps ["equal.o"; "equal.d.o"; "equal.t.o"; "equal.p.o"] ["mlvalues.h"];;
deps ["equal.o"; "equal.d.o"; "equal.t.o"; "equal.p.o"] ["intext.h"];;
deps ["equal.o"; "equal.d.o"; "equal.t.o"; "equal.p.o"] ["major_gc.h"];;
deps ["equal.o"; "equal.d.o"; "equal.t.o"; "equal.p.o"] ["mlvalues.h"];;
deps ["equal.o"; "equal.d.o"; "equal.t.o"; "equal.p.o"] ["profil.h"];;
deps ["equal.o"; "equal.d.o"; "equal.t.o"; "equal.p.o"] ["semaph.h"];;
deps ["equal.o"; "equal.d.o"; "equal.t.o"; "equal.p.o"] ["control.h"];;
deps ["equal.o"; "equal.d.o"; "equal.t.o"; "equal.p.o"] ["mem_misc.h"];;
deps ["equal.o"; "equal.d.o"; "equal.t.o"; "equal.p.o"] ["mlvalues.h"];;
deps ["equal.o"; "equal.d.o"; "equal.t.o"; "equal.p.o"] ["private.h"];;
deps ["equal.o"; "equal.d.o"; "equal.t.o"; "equal.p.o"] ["mem_misc.h"];;
deps ["equal.o"; "equal.d.o"; "equal.t.o"; "equal.p.o"] ["mlvalues.h"];;
deps ["equal.o"; "equal.d.o"; "equal.t.o"; "equal.p.o"] ["strings.h"];;
deps ["ints.o"; "ints.d.o"; "ints.t.o"; "ints.p.o"] ["debugger.h"];;
deps ["ints.o"; "ints.d.o"; "ints.t.o"; "ints.p.o"] ["mlvalues.h"];;
deps ["ints.o"; "ints.d.o"; "ints.t.o"; "ints.p.o"] ["config.h"];;
deps ["ints.o"; "ints.d.o"; "ints.t.o"; "ints.p.o"] ["fail.h"];;
deps ["ints.o"; "ints.d.o"; "ints.t.o"; "ints.p.o"] ["mlvalues.h"];;
deps ["ints.o"; "ints.d.o"; "ints.t.o"; "ints.p.o"] ["mlvalues.h"];;
deps ["floats.o"; "floats.d.o"; "floats.t.o"; "floats.p.o"] ["mlvalues.h"];;
deps ["floats.o"; "floats.d.o"; "floats.t.o"; "floats.p.o"] ["config.h"];;
deps ["floats.o"; "floats.d.o"; "floats.t.o"; "floats.p.o"] ["fail.h"];;
deps ["floats.o"; "floats.d.o"; "floats.t.o"; "floats.p.o"] ["mlvalues.h"];;
deps ["strings.o"; "strings.d.o"; "strings.t.o"; "strings.p.o"] ["mlvalues.h"];;
deps ["strings.o"; "strings.d.o"; "strings.t.o"; "strings.p.o"] ["config.h"];;
deps ["io.o"; "io.d.o"; "io.t.o"; "io.p.o"] ["mlvalues.h"];;
deps ["io.o"; "io.d.o"; "io.t.o"; "io.p.o"] ["config.h"];;
deps ["io.o"; "io.d.o"; "io.t.o"; "io.p.o"] ["io.h"];;
deps ["io.o"; "io.d.o"; "io.t.o"; "io.p.o"] ["control.h"];;
deps ["io.o"; "io.d.o"; "io.t.o"; "io.p.o"] ["fail.h"];;
deps ["io.o"; "io.d.o"; "io.t.o"; "io.p.o"] ["mlvalues.h"];;
deps ["io.o"; "io.d.o"; "io.t.o"; "io.p.o"] ["signals.h"];;
deps ["io.o"; "io.d.o"; "io.t.o"; "io.p.o"] ["mem_misc.h"];;
deps ["io.o"; "io.d.o"; "io.t.o"; "io.p.o"] ["gc.h"];;
deps ["io.o"; "io.d.o"; "io.t.o"; "io.p.o"] ["mlvalues.h"];;
deps ["io.o"; "io.d.o"; "io.t.o"; "io.p.o"] ["control.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["control.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["debugger.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["mlvalues.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["config.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["fail.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["mlvalues.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["gc.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["mlvalues.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["intext.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["io.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["control.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["memory.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["gc.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["major_gc.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["control.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["private.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["control.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["debugger.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["freelist.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["control.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["mlvalues.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["intext.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["major_gc.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["mlvalues.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["profil.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["semaph.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["control.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["mem_misc.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["mlvalues.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["private.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["mlvalues.h"];;
deps ["intext.o"; "intext.d.o"; "intext.t.o"; "intext.p.o"] ["private.h"];;
deps ["hash.o"; "hash.d.o"; "hash.t.o"; "hash.p.o"] ["control.h"];;
deps ["hash.o"; "hash.d.o"; "hash.t.o"; "hash.p.o"] ["mlvalues.h"];;
deps ["hash.o"; "hash.d.o"; "hash.t.o"; "hash.p.o"] ["config.h"];;
deps ["hash.o"; "hash.d.o"; "hash.t.o"; "hash.p.o"] ["private.h"];;
deps ["hash.o"; "hash.d.o"; "hash.t.o"; "hash.p.o"] ["control.h"];;
deps ["hash.o"; "hash.d.o"; "hash.t.o"; "hash.p.o"] ["debugger.h"];;
deps ["hash.o"; "hash.d.o"; "hash.t.o"; "hash.p.o"] ["mlvalues.h"];;
deps ["hash.o"; "hash.d.o"; "hash.t.o"; "hash.p.o"] ["freelist.h"];;
deps ["hash.o"; "hash.d.o"; "hash.t.o"; "hash.p.o"] ["control.h"];;
deps ["hash.o"; "hash.d.o"; "hash.t.o"; "hash.p.o"] ["mlvalues.h"];;
deps ["hash.o"; "hash.d.o"; "hash.t.o"; "hash.p.o"] ["intext.h"];;
deps ["hash.o"; "hash.d.o"; "hash.t.o"; "hash.p.o"] ["major_gc.h"];;
deps ["hash.o"; "hash.d.o"; "hash.t.o"; "hash.p.o"] ["control.h"];;
deps ["hash.o"; "hash.d.o"; "hash.t.o"; "hash.p.o"] ["private.h"];;
deps ["hash.o"; "hash.d.o"; "hash.t.o"; "hash.p.o"] ["mlvalues.h"];;
deps ["hash.o"; "hash.d.o"; "hash.t.o"; "hash.p.o"] ["profil.h"];;
deps ["hash.o"; "hash.d.o"; "hash.t.o"; "hash.p.o"] ["semaph.h"];;
deps ["hash.o"; "hash.d.o"; "hash.t.o"; "hash.p.o"] ["control.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["config.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["control.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["debugger.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["mlvalues.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["config.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["globals.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["control.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["mlvalues.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["memory.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["gc.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["mlvalues.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["major_gc.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["control.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["private.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["control.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["debugger.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["freelist.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["control.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["mlvalues.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["intext.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["major_gc.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["mlvalues.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["profil.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["semaph.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["control.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["mem_misc.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["mlvalues.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["private.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["mlvalues.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["profil.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["stacks.h"];;
deps ["unix.o"; "unix.d.o"; "unix.t.o"; "unix.p.o"] ["mlvalues.h"];;
deps ["meta.o"; "meta.d.o"; "meta.t.o"; "meta.p.o"] ["mlvalues.h"];;
deps ["meta.o"; "meta.d.o"; "meta.t.o"; "meta.p.o"] ["config.h"];;
deps ["meta.o"; "meta.d.o"; "meta.t.o"; "meta.p.o"] ["memory.h"];;
deps ["meta.o"; "meta.d.o"; "meta.t.o"; "meta.p.o"] ["gc.h"];;
deps ["meta.o"; "meta.d.o"; "meta.t.o"; "meta.p.o"] ["mlvalues.h"];;
deps ["meta.o"; "meta.d.o"; "meta.t.o"; "meta.p.o"] ["major_gc.h"];;
deps ["meta.o"; "meta.d.o"; "meta.t.o"; "meta.p.o"] ["control.h"];;
deps ["meta.o"; "meta.d.o"; "meta.t.o"; "meta.p.o"] ["private.h"];;
deps ["meta.o"; "meta.d.o"; "meta.t.o"; "meta.p.o"] ["control.h"];;
deps ["meta.o"; "meta.d.o"; "meta.t.o"; "meta.p.o"] ["debugger.h"];;
deps ["meta.o"; "meta.d.o"; "meta.t.o"; "meta.p.o"] ["mlvalues.h"];;
deps ["meta.o"; "meta.d.o"; "meta.t.o"; "meta.p.o"] ["freelist.h"];;
deps ["meta.o"; "meta.d.o"; "meta.t.o"; "meta.p.o"] ["control.h"];;
deps ["meta.o"; "meta.d.o"; "meta.t.o"; "meta.p.o"] ["mlvalues.h"];;
deps ["meta.o"; "meta.d.o"; "meta.t.o"; "meta.p.o"] ["intext.h"];;
deps ["meta.o"; "meta.d.o"; "meta.t.o"; "meta.p.o"] ["major_gc.h"];;
deps ["meta.o"; "meta.d.o"; "meta.t.o"; "meta.p.o"] ["mlvalues.h"];;
deps ["meta.o"; "meta.d.o"; "meta.t.o"; "meta.p.o"] ["profil.h"];;
deps ["meta.o"; "meta.d.o"; "meta.t.o"; "meta.p.o"] ["semaph.h"];;
deps ["meta.o"; "meta.d.o"; "meta.t.o"; "meta.p.o"] ["control.h"];;
deps ["meta.o"; "meta.d.o"; "meta.t.o"; "meta.p.o"] ["mem_misc.h"];;
deps ["meta.o"; "meta.d.o"; "meta.t.o"; "meta.p.o"] ["mlvalues.h"];;
deps ["meta.o"; "meta.d.o"; "meta.t.o"; "meta.p.o"] ["private.h"];;
deps ["meta.o"; "meta.d.o"; "meta.t.o"; "meta.p.o"] ["globals.h"];;
deps ["meta.o"; "meta.d.o"; "meta.t.o"; "meta.p.o"] ["control.h"];;
deps ["meta.o"; "meta.d.o"; "meta.t.o"; "meta.p.o"] ["mlvalues.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["private.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["control.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["debugger.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["mlvalues.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["config.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["freelist.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["control.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["mlvalues.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["intext.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["major_gc.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["control.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["private.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["mlvalues.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["profil.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["semaph.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["control.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["control.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["fail.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["mlvalues.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["globals.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["control.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["mlvalues.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["memory.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["gc.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["mlvalues.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["major_gc.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["mem_misc.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["mlvalues.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["private.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["mlvalues.h"];;
deps ["signals.o"; "signals.d.o"; "signals.t.o"; "signals.p.o"] ["signals.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["callback.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["mlvalues.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["config.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["control.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["debugger.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["mlvalues.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["fail.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["mlvalues.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["i_interp.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["major_gc.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["control.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["private.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["control.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["debugger.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["freelist.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["control.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["mlvalues.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["intext.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["major_gc.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["mlvalues.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["profil.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["semaph.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["control.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["mlvalues.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["memory.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["gc.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["mlvalues.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["major_gc.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["mem_misc.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["mlvalues.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["private.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["private.h"];;
deps ["threads.o"; "threads.d.o"; "threads.t.o"; "threads.p.o"] ["semaph.h"];;
deps ["parsing.o"; "parsing.d.o"; "parsing.t.o"; "parsing.p.o"] ["config.h"];;
deps ["parsing.o"; "parsing.d.o"; "parsing.t.o"; "parsing.p.o"] ["mlvalues.h"];;
deps ["parsing.o"; "parsing.d.o"; "parsing.t.o"; "parsing.p.o"] ["config.h"];;
deps ["parsing.o"; "parsing.d.o"; "parsing.t.o"; "parsing.p.o"] ["memory.h"];;
deps ["parsing.o"; "parsing.d.o"; "parsing.t.o"; "parsing.p.o"] ["gc.h"];;
deps ["parsing.o"; "parsing.d.o"; "parsing.t.o"; "parsing.p.o"] ["mlvalues.h"];;
deps ["parsing.o"; "parsing.d.o"; "parsing.t.o"; "parsing.p.o"] ["major_gc.h"];;
deps ["parsing.o"; "parsing.d.o"; "parsing.t.o"; "parsing.p.o"] ["control.h"];;
deps ["parsing.o"; "parsing.d.o"; "parsing.t.o"; "parsing.p.o"] ["private.h"];;
deps ["parsing.o"; "parsing.d.o"; "parsing.t.o"; "parsing.p.o"] ["control.h"];;
deps ["parsing.o"; "parsing.d.o"; "parsing.t.o"; "parsing.p.o"] ["debugger.h"];;
deps ["parsing.o"; "parsing.d.o"; "parsing.t.o"; "parsing.p.o"] ["mlvalues.h"];;
deps ["parsing.o"; "parsing.d.o"; "parsing.t.o"; "parsing.p.o"] ["freelist.h"];;
deps ["parsing.o"; "parsing.d.o"; "parsing.t.o"; "parsing.p.o"] ["control.h"];;
deps ["parsing.o"; "parsing.d.o"; "parsing.t.o"; "parsing.p.o"] ["mlvalues.h"];;
deps ["parsing.o"; "parsing.d.o"; "parsing.t.o"; "parsing.p.o"] ["intext.h"];;
deps ["parsing.o"; "parsing.d.o"; "parsing.t.o"; "parsing.p.o"] ["major_gc.h"];;
deps ["parsing.o"; "parsing.d.o"; "parsing.t.o"; "parsing.p.o"] ["mlvalues.h"];;
deps ["parsing.o"; "parsing.d.o"; "parsing.t.o"; "parsing.p.o"] ["profil.h"];;
deps ["parsing.o"; "parsing.d.o"; "parsing.t.o"; "parsing.p.o"] ["semaph.h"];;
deps ["parsing.o"; "parsing.d.o"; "parsing.t.o"; "parsing.p.o"] ["control.h"];;
deps ["parsing.o"; "parsing.d.o"; "parsing.t.o"; "parsing.p.o"] ["mem_misc.h"];;
deps ["parsing.o"; "parsing.d.o"; "parsing.t.o"; "parsing.p.o"] ["mlvalues.h"];;
deps ["parsing.o"; "parsing.d.o"; "parsing.t.o"; "parsing.p.o"] ["private.h"];;
deps ["parsing.o"; "parsing.d.o"; "parsing.t.o"; "parsing.p.o"] ["private.h"];;
deps ["freelist.o"; "freelist.d.o"; "freelist.t.o"; "freelist.p.o"] ["config.h"];;
deps ["freelist.o"; "freelist.d.o"; "freelist.t.o"; "freelist.p.o"] ["control.h"];;
deps ["freelist.o"; "freelist.d.o"; "freelist.t.o"; "freelist.p.o"] ["debugger.h"];;
deps ["freelist.o"; "freelist.d.o"; "freelist.t.o"; "freelist.p.o"] ["mlvalues.h"];;
deps ["freelist.o"; "freelist.d.o"; "freelist.t.o"; "freelist.p.o"] ["config.h"];;
deps ["freelist.o"; "freelist.d.o"; "freelist.t.o"; "freelist.p.o"] ["freelist.h"];;
deps ["freelist.o"; "freelist.d.o"; "freelist.t.o"; "freelist.p.o"] ["control.h"];;
deps ["freelist.o"; "freelist.d.o"; "freelist.t.o"; "freelist.p.o"] ["mlvalues.h"];;
deps ["freelist.o"; "freelist.d.o"; "freelist.t.o"; "freelist.p.o"] ["gc.h"];;
deps ["freelist.o"; "freelist.d.o"; "freelist.t.o"; "freelist.p.o"] ["mlvalues.h"];;
deps ["freelist.o"; "freelist.d.o"; "freelist.t.o"; "freelist.p.o"] ["mem_misc.h"];;
deps ["freelist.o"; "freelist.d.o"; "freelist.t.o"; "freelist.p.o"] ["mlvalues.h"];;
deps ["i_interp.o"; "i_interp.d.o"; "i_interp.t.o"; "i_interp.p.o"] ["config.h"];;
deps ["i_interp.o"; "i_interp.d.o"; "i_interp.t.o"; "i_interp.p.o"] ["control.h"];;
deps ["i_interp.o"; "i_interp.d.o"; "i_interp.t.o"; "i_interp.p.o"] ["freelist.h"];;
deps ["i_interp.o"; "i_interp.d.o"; "i_interp.t.o"; "i_interp.p.o"] ["control.h"];;
deps ["i_interp.o"; "i_interp.d.o"; "i_interp.t.o"; "i_interp.p.o"] ["mlvalues.h"];;
deps ["i_interp.o"; "i_interp.d.o"; "i_interp.t.o"; "i_interp.p.o"] ["config.h"];;
deps ["i_interp.o"; "i_interp.d.o"; "i_interp.t.o"; "i_interp.p.o"] ["major_gc.h"];;
deps ["i_interp.o"; "i_interp.d.o"; "i_interp.t.o"; "i_interp.p.o"] ["control.h"];;
deps ["i_interp.o"; "i_interp.d.o"; "i_interp.t.o"; "i_interp.p.o"] ["private.h"];;
deps ["i_interp.o"; "i_interp.d.o"; "i_interp.t.o"; "i_interp.p.o"] ["control.h"];;
deps ["i_interp.o"; "i_interp.d.o"; "i_interp.t.o"; "i_interp.p.o"] ["debugger.h"];;
deps ["i_interp.o"; "i_interp.d.o"; "i_interp.t.o"; "i_interp.p.o"] ["mlvalues.h"];;
deps ["i_interp.o"; "i_interp.d.o"; "i_interp.t.o"; "i_interp.p.o"] ["freelist.h"];;
deps ["i_interp.o"; "i_interp.d.o"; "i_interp.t.o"; "i_interp.p.o"] ["intext.h"];;
deps ["i_interp.o"; "i_interp.d.o"; "i_interp.t.o"; "i_interp.p.o"] ["major_gc.h"];;
deps ["i_interp.o"; "i_interp.d.o"; "i_interp.t.o"; "i_interp.p.o"] ["mlvalues.h"];;
deps ["i_interp.o"; "i_interp.d.o"; "i_interp.t.o"; "i_interp.p.o"] ["profil.h"];;
deps ["i_interp.o"; "i_interp.d.o"; "i_interp.t.o"; "i_interp.p.o"] ["semaph.h"];;
deps ["i_interp.o"; "i_interp.d.o"; "i_interp.t.o"; "i_interp.p.o"] ["control.h"];;
deps ["i_interp.o"; "i_interp.d.o"; "i_interp.t.o"; "i_interp.p.o"] ["memory.h"];;
deps ["i_interp.o"; "i_interp.d.o"; "i_interp.t.o"; "i_interp.p.o"] ["gc.h"];;
deps ["i_interp.o"; "i_interp.d.o"; "i_interp.t.o"; "i_interp.p.o"] ["mlvalues.h"];;
deps ["i_interp.o"; "i_interp.d.o"; "i_interp.t.o"; "i_interp.p.o"] ["major_gc.h"];;
deps ["i_interp.o"; "i_interp.d.o"; "i_interp.t.o"; "i_interp.p.o"] ["mem_misc.h"];;
deps ["i_interp.o"; "i_interp.d.o"; "i_interp.t.o"; "i_interp.p.o"] ["mlvalues.h"];;
deps ["i_interp.o"; "i_interp.d.o"; "i_interp.t.o"; "i_interp.p.o"] ["private.h"];;
deps ["i_interp.o"; "i_interp.d.o"; "i_interp.t.o"; "i_interp.p.o"] ["mem_misc.h"];;
deps ["i_interp.o"; "i_interp.d.o"; "i_interp.t.o"; "i_interp.p.o"] ["private.h"];;
deps ["minor_gc.o"; "minor_gc.d.o"; "minor_gc.t.o"; "minor_gc.p.o"] ["debugger.h"];;
deps ["minor_gc.o"; "minor_gc.d.o"; "minor_gc.t.o"; "minor_gc.p.o"] ["mlvalues.h"];;
deps ["minor_gc.o"; "minor_gc.d.o"; "minor_gc.t.o"; "minor_gc.p.o"] ["config.h"];;
deps ["minor_gc.o"; "minor_gc.d.o"; "minor_gc.t.o"; "minor_gc.p.o"] ["gc.h"];;
deps ["minor_gc.o"; "minor_gc.d.o"; "minor_gc.t.o"; "minor_gc.p.o"] ["mlvalues.h"];;
deps ["minor_gc.o"; "minor_gc.d.o"; "minor_gc.t.o"; "minor_gc.p.o"] ["major_gc.h"];;
deps ["minor_gc.o"; "minor_gc.d.o"; "minor_gc.t.o"; "minor_gc.p.o"] ["control.h"];;
deps ["minor_gc.o"; "minor_gc.d.o"; "minor_gc.t.o"; "minor_gc.p.o"] ["private.h"];;
deps ["minor_gc.o"; "minor_gc.d.o"; "minor_gc.t.o"; "minor_gc.p.o"] ["control.h"];;
deps ["minor_gc.o"; "minor_gc.d.o"; "minor_gc.t.o"; "minor_gc.p.o"] ["debugger.h"];;
deps ["minor_gc.o"; "minor_gc.d.o"; "minor_gc.t.o"; "minor_gc.p.o"] ["freelist.h"];;
deps ["minor_gc.o"; "minor_gc.d.o"; "minor_gc.t.o"; "minor_gc.p.o"] ["control.h"];;
deps ["minor_gc.o"; "minor_gc.d.o"; "minor_gc.t.o"; "minor_gc.p.o"] ["mlvalues.h"];;
deps ["minor_gc.o"; "minor_gc.d.o"; "minor_gc.t.o"; "minor_gc.p.o"] ["intext.h"];;
deps ["minor_gc.o"; "minor_gc.d.o"; "minor_gc.t.o"; "minor_gc.p.o"] ["major_gc.h"];;
deps ["minor_gc.o"; "minor_gc.d.o"; "minor_gc.t.o"; "minor_gc.p.o"] ["mlvalues.h"];;
deps ["minor_gc.o"; "minor_gc.d.o"; "minor_gc.t.o"; "minor_gc.p.o"] ["profil.h"];;
deps ["minor_gc.o"; "minor_gc.d.o"; "minor_gc.t.o"; "minor_gc.p.o"] ["semaph.h"];;
deps ["minor_gc.o"; "minor_gc.d.o"; "minor_gc.t.o"; "minor_gc.p.o"] ["control.h"];;
deps ["minor_gc.o"; "minor_gc.d.o"; "minor_gc.t.o"; "minor_gc.p.o"] ["mlvalues.h"];;
deps ["minor_gc.o"; "minor_gc.d.o"; "minor_gc.t.o"; "minor_gc.p.o"] ["mem_misc.h"];;
deps ["minor_gc.o"; "minor_gc.d.o"; "minor_gc.t.o"; "minor_gc.p.o"] ["roots.h"];;
deps ["minor_gc.o"; "minor_gc.d.o"; "minor_gc.t.o"; "minor_gc.p.o"] ["private.h"];;
deps ["minor_gc.o"; "minor_gc.d.o"; "minor_gc.t.o"; "minor_gc.p.o"] ["profil.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["control.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["debugger.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["mlvalues.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["config.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["fail.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["mlvalues.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["freelist.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["control.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["mlvalues.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["gc.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["mlvalues.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["major_gc.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["control.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["private.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["control.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["debugger.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["freelist.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["intext.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["major_gc.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["mlvalues.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["profil.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["semaph.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["control.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["mem_misc.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["memory.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["gc.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["major_gc.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["mem_misc.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["mlvalues.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["private.h"];;
deps ["memory.o"; "memory.d.o"; "memory.t.o"; "memory.p.o"] ["mlvalues.h"];;
deps ["mem_misc.o"; "mem_misc.d.o"; "mem_misc.t.o"; "mem_misc.p.o"] ["config.h"];;
deps ["mem_misc.o"; "mem_misc.d.o"; "mem_misc.t.o"; "mem_misc.p.o"] ["debugger.h"];;
deps ["mem_misc.o"; "mem_misc.d.o"; "mem_misc.t.o"; "mem_misc.p.o"] ["mlvalues.h"];;
deps ["mem_misc.o"; "mem_misc.d.o"; "mem_misc.t.o"; "mem_misc.p.o"] ["config.h"];;
deps ["mem_misc.o"; "mem_misc.d.o"; "mem_misc.t.o"; "mem_misc.p.o"] ["mem_misc.h"];;
deps ["fail.o"; "fail.d.o"; "fail.t.o"; "fail.p.o"] ["control.h"];;
deps ["fail.o"; "fail.d.o"; "fail.t.o"; "fail.p.o"] ["fail.h"];;
deps ["fail.o"; "fail.d.o"; "fail.t.o"; "fail.p.o"] ["mlvalues.h"];;
deps ["fail.o"; "fail.d.o"; "fail.t.o"; "fail.p.o"] ["config.h"];;
deps ["fail.o"; "fail.d.o"; "fail.t.o"; "fail.p.o"] ["memory.h"];;
deps ["fail.o"; "fail.d.o"; "fail.t.o"; "fail.p.o"] ["gc.h"];;
deps ["fail.o"; "fail.d.o"; "fail.t.o"; "fail.p.o"] ["mlvalues.h"];;
deps ["fail.o"; "fail.d.o"; "fail.t.o"; "fail.p.o"] ["major_gc.h"];;
deps ["fail.o"; "fail.d.o"; "fail.t.o"; "fail.p.o"] ["control.h"];;
deps ["fail.o"; "fail.d.o"; "fail.t.o"; "fail.p.o"] ["private.h"];;
deps ["fail.o"; "fail.d.o"; "fail.t.o"; "fail.p.o"] ["control.h"];;
deps ["fail.o"; "fail.d.o"; "fail.t.o"; "fail.p.o"] ["debugger.h"];;
deps ["fail.o"; "fail.d.o"; "fail.t.o"; "fail.p.o"] ["mlvalues.h"];;
deps ["fail.o"; "fail.d.o"; "fail.t.o"; "fail.p.o"] ["freelist.h"];;
deps ["fail.o"; "fail.d.o"; "fail.t.o"; "fail.p.o"] ["control.h"];;
deps ["fail.o"; "fail.d.o"; "fail.t.o"; "fail.p.o"] ["mlvalues.h"];;
deps ["fail.o"; "fail.d.o"; "fail.t.o"; "fail.p.o"] ["intext.h"];;
deps ["fail.o"; "fail.d.o"; "fail.t.o"; "fail.p.o"] ["major_gc.h"];;
deps ["fail.o"; "fail.d.o"; "fail.t.o"; "fail.p.o"] ["mlvalues.h"];;
deps ["fail.o"; "fail.d.o"; "fail.t.o"; "fail.p.o"] ["profil.h"];;
deps ["fail.o"; "fail.d.o"; "fail.t.o"; "fail.p.o"] ["semaph.h"];;
deps ["fail.o"; "fail.d.o"; "fail.t.o"; "fail.p.o"] ["control.h"];;
deps ["fail.o"; "fail.d.o"; "fail.t.o"; "fail.p.o"] ["mem_misc.h"];;
deps ["fail.o"; "fail.d.o"; "fail.t.o"; "fail.p.o"] ["mlvalues.h"];;
deps ["fail.o"; "fail.d.o"; "fail.t.o"; "fail.p.o"] ["private.h"];;
deps ["fail.o"; "fail.d.o"; "fail.t.o"; "fail.p.o"] ["mlvalues.h"];;
deps ["fail.o"; "fail.d.o"; "fail.t.o"; "fail.p.o"] ["private.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["config.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["control.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["debugger.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["mlvalues.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["config.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["exec.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["fail.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["mlvalues.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["gc.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["mlvalues.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["globals.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["control.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["mlvalues.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["i_interp.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["major_gc.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["control.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["private.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["control.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["debugger.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["freelist.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["control.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["mlvalues.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["intext.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["major_gc.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["mlvalues.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["profil.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["semaph.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["control.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["major_gc.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["mem_misc.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["memory.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["gc.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["major_gc.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["mem_misc.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["mlvalues.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["private.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["mlvalues.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["private.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["profil.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["stacks.h"];;
deps ["main.o"; "main.d.o"; "main.t.o"; "main.p.o"] ["mlvalues.h"];;
deps ["major_gc.o"; "major_gc.d.o"; "major_gc.t.o"; "major_gc.p.o"] ["control.h"];;
deps ["major_gc.o"; "major_gc.d.o"; "major_gc.t.o"; "major_gc.p.o"] ["debugger.h"];;
deps ["major_gc.o"; "major_gc.d.o"; "major_gc.t.o"; "major_gc.p.o"] ["mlvalues.h"];;
deps ["major_gc.o"; "major_gc.d.o"; "major_gc.t.o"; "major_gc.p.o"] ["config.h"];;
deps ["major_gc.o"; "major_gc.d.o"; "major_gc.t.o"; "major_gc.p.o"] ["freelist.h"];;
deps ["major_gc.o"; "major_gc.d.o"; "major_gc.t.o"; "major_gc.p.o"] ["control.h"];;
deps ["major_gc.o"; "major_gc.d.o"; "major_gc.t.o"; "major_gc.p.o"] ["mlvalues.h"];;
deps ["major_gc.o"; "major_gc.d.o"; "major_gc.t.o"; "major_gc.p.o"] ["gc.h"];;
deps ["major_gc.o"; "major_gc.d.o"; "major_gc.t.o"; "major_gc.p.o"] ["mlvalues.h"];;
deps ["major_gc.o"; "major_gc.d.o"; "major_gc.t.o"; "major_gc.p.o"] ["globals.h"];;
deps ["major_gc.o"; "major_gc.d.o"; "major_gc.t.o"; "major_gc.p.o"] ["control.h"];;
deps ["major_gc.o"; "major_gc.d.o"; "major_gc.t.o"; "major_gc.p.o"] ["mlvalues.h"];;
deps ["major_gc.o"; "major_gc.d.o"; "major_gc.t.o"; "major_gc.p.o"] ["major_gc.h"];;
deps ["major_gc.o"; "major_gc.d.o"; "major_gc.t.o"; "major_gc.p.o"] ["control.h"];;
deps ["major_gc.o"; "major_gc.d.o"; "major_gc.t.o"; "major_gc.p.o"] ["private.h"];;
deps ["major_gc.o"; "major_gc.d.o"; "major_gc.t.o"; "major_gc.p.o"] ["control.h"];;
deps ["major_gc.o"; "major_gc.d.o"; "major_gc.t.o"; "major_gc.p.o"] ["debugger.h"];;
deps ["major_gc.o"; "major_gc.d.o"; "major_gc.t.o"; "major_gc.p.o"] ["freelist.h"];;
deps ["major_gc.o"; "major_gc.d.o"; "major_gc.t.o"; "major_gc.p.o"] ["intext.h"];;
deps ["major_gc.o"; "major_gc.d.o"; "major_gc.t.o"; "major_gc.p.o"] ["major_gc.h"];;
deps ["major_gc.o"; "major_gc.d.o"; "major_gc.t.o"; "major_gc.p.o"] ["mlvalues.h"];;
deps ["major_gc.o"; "major_gc.d.o"; "major_gc.t.o"; "major_gc.p.o"] ["profil.h"];;
deps ["major_gc.o"; "major_gc.d.o"; "major_gc.t.o"; "major_gc.p.o"] ["semaph.h"];;
deps ["major_gc.o"; "major_gc.d.o"; "major_gc.t.o"; "major_gc.p.o"] ["control.h"];;
deps ["major_gc.o"; "major_gc.d.o"; "major_gc.t.o"; "major_gc.p.o"] ["mem_misc.h"];;
deps ["major_gc.o"; "major_gc.d.o"; "major_gc.t.o"; "major_gc.p.o"] ["profil.h"];;
deps ["semaph.o"; "semaph.d.o"; "semaph.t.o"; "semaph.p.o"] ["control.h"];;
deps ["semaph.o"; "semaph.d.o"; "semaph.t.o"; "semaph.p.o"] ["semaph.h"];;
deps ["semaph.o"; "semaph.d.o"; "semaph.t.o"; "semaph.p.o"] ["control.h"];;
deps ["stacks.o"; "stacks.d.o"; "stacks.t.o"; "stacks.p.o"] ["config.h"];;
deps ["stacks.o"; "stacks.d.o"; "stacks.t.o"; "stacks.p.o"] ["debugger.h"];;
deps ["stacks.o"; "stacks.d.o"; "stacks.t.o"; "stacks.p.o"] ["mlvalues.h"];;
deps ["stacks.o"; "stacks.d.o"; "stacks.t.o"; "stacks.p.o"] ["config.h"];;
deps ["stacks.o"; "stacks.d.o"; "stacks.t.o"; "stacks.p.o"] ["fail.h"];;
deps ["stacks.o"; "stacks.d.o"; "stacks.t.o"; "stacks.p.o"] ["mlvalues.h"];;
deps ["stacks.o"; "stacks.d.o"; "stacks.t.o"; "stacks.p.o"] ["mem_misc.h"];;
deps ["stacks.o"; "stacks.d.o"; "stacks.t.o"; "stacks.p.o"] ["mlvalues.h"];;
deps ["stacks.o"; "stacks.d.o"; "stacks.t.o"; "stacks.p.o"] ["private.h"];;
deps ["stacks.o"; "stacks.d.o"; "stacks.t.o"; "stacks.p.o"] ["control.h"];;
deps ["stacks.o"; "stacks.d.o"; "stacks.t.o"; "stacks.p.o"] ["debugger.h"];;
deps ["stacks.o"; "stacks.d.o"; "stacks.t.o"; "stacks.p.o"] ["freelist.h"];;
deps ["stacks.o"; "stacks.d.o"; "stacks.t.o"; "stacks.p.o"] ["control.h"];;
deps ["stacks.o"; "stacks.d.o"; "stacks.t.o"; "stacks.p.o"] ["mlvalues.h"];;
deps ["stacks.o"; "stacks.d.o"; "stacks.t.o"; "stacks.p.o"] ["intext.h"];;
deps ["stacks.o"; "stacks.d.o"; "stacks.t.o"; "stacks.p.o"] ["major_gc.h"];;
deps ["stacks.o"; "stacks.d.o"; "stacks.t.o"; "stacks.p.o"] ["control.h"];;
deps ["stacks.o"; "stacks.d.o"; "stacks.t.o"; "stacks.p.o"] ["private.h"];;
deps ["stacks.o"; "stacks.d.o"; "stacks.t.o"; "stacks.p.o"] ["mlvalues.h"];;
deps ["stacks.o"; "stacks.d.o"; "stacks.t.o"; "stacks.p.o"] ["profil.h"];;
deps ["stacks.o"; "stacks.d.o"; "stacks.t.o"; "stacks.p.o"] ["semaph.h"];;
deps ["stacks.o"; "stacks.d.o"; "stacks.t.o"; "stacks.p.o"] ["control.h"];;
deps ["stacks.o"; "stacks.d.o"; "stacks.t.o"; "stacks.p.o"] ["stacks.h"];;
deps ["stacks.o"; "stacks.d.o"; "stacks.t.o"; "stacks.p.o"] ["mlvalues.h"];;
deps ["prims.o"; "prims.d.o"; "prims.t.o"; "prims.p.o"] ["mlvalues.h"];;
deps ["prims.o"; "prims.d.o"; "prims.t.o"; "prims.p.o"] ["config.h"];;
deps ["prims.o"; "prims.d.o"; "prims.t.o"; "prims.p.o"] ["cprim.h"];;
deps ["prims.o"; "prims.d.o"; "prims.t.o"; "prims.p.o"] ["mlvalues.h"];;
deps ["roots.o"; "roots.d.o"; "roots.t.o"; "roots.p.o"] ["stacks.h"];;
deps ["roots.o"; "roots.d.o"; "roots.t.o"; "roots.p.o"] ["mlvalues.h"];;
deps ["roots.o"; "roots.d.o"; "roots.t.o"; "roots.p.o"] ["config.h"];;
deps ["roots.o"; "roots.d.o"; "roots.t.o"; "roots.p.o"] ["private.h"];;
deps ["roots.o"; "roots.d.o"; "roots.t.o"; "roots.p.o"] ["control.h"];;
deps ["roots.o"; "roots.d.o"; "roots.t.o"; "roots.p.o"] ["debugger.h"];;
deps ["roots.o"; "roots.d.o"; "roots.t.o"; "roots.p.o"] ["mlvalues.h"];;
deps ["roots.o"; "roots.d.o"; "roots.t.o"; "roots.p.o"] ["freelist.h"];;
deps ["roots.o"; "roots.d.o"; "roots.t.o"; "roots.p.o"] ["control.h"];;
deps ["roots.o"; "roots.d.o"; "roots.t.o"; "roots.p.o"] ["mlvalues.h"];;
deps ["roots.o"; "roots.d.o"; "roots.t.o"; "roots.p.o"] ["intext.h"];;
deps ["roots.o"; "roots.d.o"; "roots.t.o"; "roots.p.o"] ["major_gc.h"];;
deps ["roots.o"; "roots.d.o"; "roots.t.o"; "roots.p.o"] ["control.h"];;
deps ["roots.o"; "roots.d.o"; "roots.t.o"; "roots.p.o"] ["private.h"];;
deps ["roots.o"; "roots.d.o"; "roots.t.o"; "roots.p.o"] ["mlvalues.h"];;
deps ["roots.o"; "roots.d.o"; "roots.t.o"; "roots.p.o"] ["profil.h"];;
deps ["roots.o"; "roots.d.o"; "roots.t.o"; "roots.p.o"] ["semaph.h"];;
deps ["roots.o"; "roots.d.o"; "roots.t.o"; "roots.p.o"] ["control.h"];;
deps ["roots.o"; "roots.d.o"; "roots.t.o"; "roots.p.o"] ["mem_misc.h"];;
deps ["callback.o"; "callback.d.o"; "callback.t.o"; "callback.p.o"] ["callback.h"];;
deps ["callback.o"; "callback.d.o"; "callback.t.o"; "callback.p.o"] ["mlvalues.h"];;
deps ["callback.o"; "callback.d.o"; "callback.t.o"; "callback.p.o"] ["config.h"];;
deps ["callback.o"; "callback.d.o"; "callback.t.o"; "callback.p.o"] ["instruct.h"];;
deps ["callback.o"; "callback.d.o"; "callback.t.o"; "callback.p.o"] ["mlvalues.h"];;
deps ["callback.o"; "callback.d.o"; "callback.t.o"; "callback.p.o"] ["private.h"];;
deps ["callback.o"; "callback.d.o"; "callback.t.o"; "callback.p.o"] ["control.h"];;
deps ["callback.o"; "callback.d.o"; "callback.t.o"; "callback.p.o"] ["debugger.h"];;
deps ["callback.o"; "callback.d.o"; "callback.t.o"; "callback.p.o"] ["mlvalues.h"];;
deps ["callback.o"; "callback.d.o"; "callback.t.o"; "callback.p.o"] ["freelist.h"];;
deps ["callback.o"; "callback.d.o"; "callback.t.o"; "callback.p.o"] ["control.h"];;
deps ["callback.o"; "callback.d.o"; "callback.t.o"; "callback.p.o"] ["mlvalues.h"];;
deps ["callback.o"; "callback.d.o"; "callback.t.o"; "callback.p.o"] ["intext.h"];;
deps ["callback.o"; "callback.d.o"; "callback.t.o"; "callback.p.o"] ["major_gc.h"];;
deps ["callback.o"; "callback.d.o"; "callback.t.o"; "callback.p.o"] ["control.h"];;
deps ["callback.o"; "callback.d.o"; "callback.t.o"; "callback.p.o"] ["private.h"];;
deps ["callback.o"; "callback.d.o"; "callback.t.o"; "callback.p.o"] ["mlvalues.h"];;
deps ["callback.o"; "callback.d.o"; "callback.t.o"; "callback.p.o"] ["profil.h"];;
deps ["callback.o"; "callback.d.o"; "callback.t.o"; "callback.p.o"] ["semaph.h"];;
deps ["callback.o"; "callback.d.o"; "callback.t.o"; "callback.p.o"] ["control.h"];;
deps ["callback.o"; "callback.d.o"; "callback.t.o"; "callback.p.o"] ["stacks.h"];;
deps ["callback.o"; "callback.d.o"; "callback.t.o"; "callback.p.o"] ["mlvalues.h"];;
deps ["profil.o"; "profil.d.o"; "profil.t.o"; "profil.p.o"] ["profil.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["control.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["debugger.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["mlvalues.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["config.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["instruct.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["memory.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["gc.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["mlvalues.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["major_gc.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["control.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["private.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["control.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["debugger.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["freelist.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["control.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["mlvalues.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["intext.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["major_gc.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["mlvalues.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["profil.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["semaph.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["control.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["mem_misc.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["mlvalues.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["private.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["mlvalues.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["opnames.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["private.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["stacks.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["mlvalues.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["unalignd.h"];;
deps ["debugger.o"; "debugger.d.o"; "debugger.t.o"; "debugger.p.o"] ["mem_misc.h"];;
go "all" targets;;
