/* io.c : entrees/sorties bufferisees */

#include <fcntl.h>
#include "mlvalues.h"
#include "io.h"
#include "fail.h"
#include "signals.h"
#include "mem_misc.h"
#include "gc.h"
#include "control.h"

/* Fonctions communes entree/sortie */

struct channel_with_hd {
  header_t hd;
  struct channel chan;
};

value open_descriptor(fd)       /* ML */
     value fd;
{
  struct channel_with_hd * channel_hd;
  struct channel * channel;

  channel_hd =
    (struct channel_with_hd *) stat_alloc(sizeof(struct channel_with_hd));
  channel_hd->hd = Make_header(0, Abstract_tag, Blue);
  channel = &channel_hd->chan;
  mutex_init(&channel->mutex);
  channel->fd = Long_val(fd);
  channel->offset = 0;
  channel->curr = channel->max = channel->buff;
  channel->end = channel->buff + Io_buffer_size;
  return (value) channel;
}

value channel_descriptor(channel)   /* ML */
     struct channel * channel;
{
  return Val_long(channel->fd);
}

static void os_error_unlock(channel)
     struct channel * channel;
{
  mutex_unlock(&channel->mutex);
  end_blocking_section ();
  os_error();
}

value channel_size(channel)      /* ML */
     struct channel * channel;
{
  long end;

  begin_blocking_section ();
  mutex_lock(&channel->mutex);
  end = lseek(channel->fd, 0, 2);
  if (end == -1)
    os_error_unlock(channel);
  if (lseek(channel->fd, channel->offset, 0) != channel->offset)
    os_error_unlock(channel);
  mutex_unlock(&channel->mutex);
  end_blocking_section ();
  return Val_long(end);
}

/******** Sorties *********/

static void really_write(channel, p, n)
     struct channel * channel;
     char * p;
     long n;
{
  int retcode;
  while (n > 0) {
    retcode = write(channel->fd, p, (int) n);
    if (retcode == -1)
      os_error_unlock(channel);
    p += retcode;
    n -= retcode;
  }
}   

static void flush(channel)
     struct channel * channel;
{
  int n;
  n = channel->max - channel->buff;
  if (n > 0) {
    really_write(channel, channel->buff, n);
    channel->offset += n;
    channel->curr = channel->buff;
    channel->max  = channel->buff;
  }
}

value channel_flush(channel)            /* ML */
     struct channel * channel;
{
  begin_blocking_section ();
  mutex_lock(&channel->mutex);
  flush(channel);
  mutex_unlock(&channel->mutex);
  end_blocking_section ();
  return Atom(0);
}

#define putch(channel, ch)                                                    \
  { if ((channel)->curr >= (channel)->end) flush(channel);                    \
    *((channel)->curr)++ = (ch);                                              \
    if ((channel)->curr > (channel)->max) (channel)->max = (channel)->curr; }

value output_char(channel, ch)  /* ML */
     struct channel * channel;
     value ch;
{
  begin_blocking_section ();
  mutex_lock(&channel->mutex);
  putch(channel, Long_val(ch));
  mutex_unlock(&channel->mutex);
  end_blocking_section ();
  return Atom(0);
}

void putword(channel, w)
     struct channel * channel;
     long w;
{
  putch(channel, w >> 24);
  putch(channel, w >> 16);
  putch(channel, w >> 8);
  putch(channel, w);
}

value output_int(channel, w)    /* ML */
     struct channel * channel;
     value w;
{
  begin_blocking_section ();
  mutex_lock(&channel->mutex);
  putword(channel, Long_val(w));
  mutex_unlock(&channel->mutex);
  end_blocking_section ();
  return Atom(0);
}

void putblock(channel, buff, start, n)
     struct channel * channel;
     value buff;
     long start, n;
{
  unsigned char * p;
  long m;

  p = &Byte(buff, start);
  m = channel->end - channel->curr;
  if (channel->curr == channel->buff && n >= m) {
    really_write(channel, p, n);
  } else if (n <= m) {
    bcopy(p, channel->curr, n);
    channel->curr += n;
    if (channel->curr > channel->max) channel->max = channel->curr;
  } else {
    bcopy(p, channel->curr, m);
    p += m;
    n -= m;
    m = channel->end - channel->buff;
    really_write(channel, channel->buff, m);
    channel->offset += m;
    if (n <= m) {
      bcopy(p, channel->buff, n);
      channel->curr = channel->max = channel->buff + n;
    } else {
      really_write(channel, p, n);
      channel->offset += n;
      channel->curr = channel->max = channel->buff;
    }
  }
}

value output(channel, buff, start, length) /* ML */
     struct channel * channel;
     value buff, start, length;
{
  begin_blocking_section ();
  mutex_lock(&channel->mutex);
  putblock(channel, buff, Long_val(start), Long_val(length));
  mutex_unlock(&channel->mutex);
  end_blocking_section ();
  return Atom(0);
}

value seek_out(channel, pos)    /* ML */
     struct channel * channel;
     value pos;
{
  long dest;

  begin_blocking_section ();
  mutex_lock(&channel->mutex);
  dest = Long_val(pos);
  if (dest >= channel->offset &&
      dest <= channel->offset + channel->max - channel->buff) {
    channel->curr = channel->buff + dest - channel->offset;
  } else {
    flush(channel);
    if (lseek(channel->fd, dest, 0) != dest) os_error_unlock(channel);
    channel->offset = dest;
  }
  mutex_unlock(&channel->mutex);
  end_blocking_section ();
  return Atom(0);
}

value pos_out(channel)          /* ML */
     struct channel * channel;
{
  long pos;

  begin_blocking_section ();
  mutex_lock(&channel->mutex);
  pos = channel->offset + channel->curr - channel->buff;
  mutex_unlock(&channel->mutex);
  end_blocking_section ();
  return Val_long(pos);
}

value close_out(channel)     /* ML */
     struct channel * channel;
{
  begin_blocking_section ();
  flush(channel);
  close(channel->fd);
  end_blocking_section ();
  mutex_clear(&channel->mutex);
  stat_free((char_ptr) channel);
  return Atom(0);
}

/* Entrees */

static int really_read(channel, p, n)
     struct channel * channel;
     char * p;
     int n;
{
  int retcode;

  raise_on_signal = 1;          /* Pour ne pas rester bloque dans le read */
  if (signal_is_pending) {      /* Si un signal s'est declenche entre */
    mutex_unlock(&channel->mutex);  /* le dernier check_signals */
    end_blocking_section ();
    raise_pending_signal();     /* et raise_on_signal=1 */
  }
  retcode = read(channel->fd, p, n);
  raise_on_signal = 0;
  if (retcode == -1) os_error_unlock(channel);
  return retcode;
}

unsigned char refill(channel)
     struct channel * channel;
{
  int n;

  n = really_read(channel, channel->buff, Io_buffer_size);
  if (n == 0) {
    mutex_unlock(&channel->mutex);
    mlraise(Atom(End_of_file_exn));
  }
  channel->offset += n;
  channel->max = channel->buff + n;
  channel->curr = channel->buff + 1;
  return channel->buff[0];
}

#define getch(channel) \
  ((channel)->curr >= (channel)->max ?  refill(channel) : *((channel))->curr++)

value input_char(channel)       /* ML */
     struct channel * channel;
{
  unsigned char c;

  begin_blocking_section ();
  mutex_lock(&channel->mutex);
  c = getch(channel);
  mutex_unlock(&channel->mutex);
  end_blocking_section ();
  return Val_long(c);
}

long getword(channel)
     struct channel * channel;
{
  int i;
  long res;

  res = 0;
  for(i = 0; i < 4; i++)
    res = (res << 8) + getch(channel);
  return res;
}

value input_int(channel)        /* ML */
     struct channel * channel;
{
  long res;

  begin_blocking_section ();
  mutex_lock(&channel->mutex);
  res = getword(channel);
  mutex_unlock(&channel->mutex);
  end_blocking_section ();
  return Val_long(res);
}

long getblock(channel, buff, start, n)
     struct channel * channel;
     value buff;
     long start, n;
{
  unsigned char * p;
  long m, l;

  p = &Byte(buff, start);
  m = channel->max - channel->curr;
  if (n <= m) {
    bcopy(channel->curr, p, n);
    channel->curr += n;
    return n;
  }
  bcopy(channel->curr, p, m);
  p += m;
  n -= m;
  if (n < Io_buffer_size) {
    l = really_read(channel, channel->buff, Io_buffer_size);
    channel->offset += l;
    channel->max = channel->buff + l;
    if (n > l) n = l;
    bcopy(channel->buff, p, n);
    channel->curr = channel->buff + n;
    return m + n;
  }
  l = really_read(channel, p, n);
  channel->offset += l;
  channel->max = channel->curr = channel->buff;
  return m + l;
}

value input(channel, buff, start, length) /* ML */
     struct channel * channel;
     value buff, start, length;
{
  long res;

  begin_blocking_section ();
  mutex_lock(&channel->mutex);
  res = getblock(channel, buff, Long_val(start), Long_val(length));
  mutex_unlock(&channel->mutex);
  end_blocking_section ();
  return Val_long(res);
}

value seek_in(channel, pos)     /* ML */
     struct channel * channel;
     value pos;
{
  long dest;

  begin_blocking_section ();
  mutex_lock(&channel->mutex);
  dest = Long_val(pos);
  if (dest >= channel->offset - (channel->max - channel->buff) &&
      dest <= channel->offset) {
    channel->curr = channel->max - (channel->offset - dest);
  } else {
    if (lseek(channel->fd, dest, 0) != dest)
      os_error_unlock(channel);
    channel->offset = dest;
    channel->curr = channel->max = channel->buff;
  }
  mutex_unlock(&channel->mutex);
  end_blocking_section ();
  return Atom(0);
}

value pos_in(channel)           /* ML */
     struct channel * channel;
{
  long res;

  begin_blocking_section ();
  mutex_lock(&channel->mutex);
  res = channel->offset - (channel->max - channel->curr);
  mutex_unlock(&channel->mutex);
  end_blocking_section ();
  return Val_long(res);
}

value close_in(channel)     /* ML */
     struct channel * channel;
{
  close(channel->fd);
  mutex_clear(&channel->mutex);
  stat_free((char_ptr) channel);
  return Atom(0);
}
