#include "control.h"
#include "debugger.h"
#include "freelist.h"
#include "gc.h"
#include "globals.h"
#include "major_gc.h"
#include "mem_misc.h"
#include "profil.h"

uchar *malloc ();

struct gc_cycle gc_cycles [2];
gc_cycle_t current_gc_cycle;	/* Not protected by any lock since it is
				   only written to by the major gc, and
				   reading a word is atomic. */
long current_cycle_index;

mutex_t master_fl_mutex;
condition_t memory_low;
mutex_t heap_chunk_mutex;

uchar *heap_start;
uchar * volatile heap_end;
uchar * last_chunk;
unsigned long total_heap_size;
uchar * volatile page_table;
volatile mlsize_t page_table_size;

uchar *master_fl;
volatile unsigned long last_free_size;
volatile unsigned long running_procs;

uchar * volatile gc_sweep_hp;

static value gray_vals [Gray_stack_size];
volatile int heap_is_pure;
 /* The heap is pure if the only (truly) gray objects are also in the stack. */

#ifdef PROFIL
static profil *p_major_collection;
#endif

#define Next(bp) (* (unsigned char **) (bp))

static unsigned char *temp_fl [4] = {NULL, NULL, NULL, NULL};
static unsigned char *temp_fl_tail [4] = {NULL, NULL, NULL, NULL};
static unsigned long temp_fl_size [4] = {0, 0, 0, 0};
static unsigned char *last_hp = NULL;
static unsigned char *next_hp = NULL;

void accumulate_aux ()
{
  mlsize_t i;

  i = Wosize_hp (last_hp);
  if (i == 0){
    Hd_hp (last_hp) = Make_header (0, 0, White);
  }else{
    i = i <= 3 ? i : 0;
    Next (Bp_hp (last_hp)) = temp_fl [i];
    temp_fl [i] = Bp_hp (last_hp);
    if (temp_fl_tail [i] == NULL) temp_fl_tail [i] = Bp_hp (last_hp);
    ++temp_fl_size [i];
    if (i > 0 && temp_fl_size [i] >= 200){
      mutex_lock (small_fl [i].mu);{
	small_fl [i].head = (small_fl [i].head + 1) % Num_small_fl;
	Next (temp_fl_tail [i]) = small_fl [i].lists [small_fl [i].head];
	small_fl [i].lists [small_fl [i].head] = temp_fl [i];
	temp_fl [i] = NULL;
	temp_fl_tail [i] = NULL;
	temp_fl_size [i] = 0;
      }mutex_unlock (small_fl [i].mu);
      small_fl [i].avail = 1;
    }
  }
}

void accumulate (hp)
     unsigned char *hp;
{
  mlsize_t i;

  if (hp == next_hp){		/* Collapse the blocks. */
    Hd_hp (last_hp) = Make_header (Wosize_hp (last_hp) + Whsize_hp (hp),
				   0, Blue);
    next_hp = last_hp + Bhsize_hp (last_hp);
    return;
  }
  Hd_hp (hp) = Bluehd_hd (Hd_hp (hp));
  if (last_hp != NULL) accumulate_aux ();
  last_hp = hp;
  next_hp = last_hp + Bhsize_hp (last_hp);
}

void end_accumulate ()
{
  if (last_hp != NULL) accumulate_aux ();
  last_hp = NULL;
  next_hp = NULL;
  if (temp_fl_tail [0] != NULL){
    mutex_lock (master_fl_mutex);{
      Next (temp_fl_tail [0]) = master_fl;
      master_fl = temp_fl [0];
    }mutex_unlock (master_fl_mutex);
    temp_fl_tail [0] = NULL;
    temp_fl [0] = NULL;
  }
}

/* Recursion was eliminated from this function, because we don't want the
   GC to allocate an arbitrary amount of stack space. */
void trace_from (v)
     value v;
{
  value child;
  mlsize_t i;
  value *gray_vals_cur;

  gray_vals_cur = &(gray_vals [0]);
  Assert (Is_gray_val (v));
  *gray_vals_cur++ = v;
  while (gray_vals_cur > &(gray_vals [0])){
    v = *--gray_vals_cur;
    Hd_val (v) = Blackhd_hd (Hd_val (v));
    if (Tag_val (v) < No_scan_tag){
      i = Wosize_val (v);
      while (i != 0){
	--i;
	child = Field (v, i);
	if (Is_block (child) && Is_in_heap (child) && Is_white_val (child)){
	  Hd_val (child) = Grayhd_hd (Hd_val (child));
	  if (gray_vals_cur < &(gray_vals [Gray_stack_size])){
	    *gray_vals_cur++ = child;
	  }else{
	    heap_is_pure = 0;
	  }
	}
	Assert (!Is_block (child) || !Is_in_heap (child)
		|| !Is_blue_val (child));
      }
    }
  }
}

void gc_major ()
{
  gc_cycle_t prev_cycle;

  /* A new cycle begins. */
  prev_cycle = current_gc_cycle;
  current_cycle_index = 1 - current_cycle_index;
  gc_cycles [current_cycle_index].phase = Phase_mark;
  Assert (gc_cycles [current_cycle_index].head == NULL);
  current_gc_cycle = &(gc_cycles [current_cycle_index]);

  mutex_lock (global_data_mutex);
  /* First phase : marking. */
  { struct private_data *proc;
    unsigned long new_running_procs = 0;

    /* Global variables. */
    if (Is_block (global_data) && Is_in_heap (global_data)){
      Hd_val (global_data) = Grayhd_hd (Hd_val (global_data));
      trace_from (global_data);
    }
    /* Local variables and minor heaps of the threads. */
    while (1){
      mutex_lock (&prev_cycle->lock);{
	if (prev_cycle->head == NULL){
	  mutex_unlock (&prev_cycle->lock);
          goto local_roots_done;
	}
	proc = prev_cycle->head;
	prev_cycle->head = proc->next;
      }mutex_unlock (&prev_cycle->lock);
      if (proc->pd_am_dead){
        gc_message_str ("$");
	clear_private_data (proc);
      }else{
	proc->pd_force_minor_gc = 1;
	if (mutex_try_lock (&proc->pd_running)){
	  if (sem_try_P (&proc->pd_new_gc_cycle)){
	    gc_message_str ("]");
	  }else{
	    proc->pd_force_minor_gc = 0;
	    minor_collection_blocked (proc);
	  }
	  mutex_unlock (&proc->pd_running);
	}else{
	  sem_P (&proc->pd_new_gc_cycle);
	  gc_message_str ("]");
	  ++new_running_procs;
	}
	mutex_lock (&current_gc_cycle->lock);{
	  proc->next = current_gc_cycle->head;
	  current_gc_cycle->head = proc;
	}mutex_unlock (&current_gc_cycle->lock);
      }
    }
  local_roots_done:
    running_procs = new_running_procs;
    Assert (prev_cycle->head == 0);
    /* Find all the gray objects. */
    heap_is_pure = 0;
    while (!heap_is_pure){
      uchar *hp;
      uchar *limit, *chunk;

      heap_is_pure = 1;
      chunk = heap_start;
      while (chunk != NULL){
	/* Sweep one chunk. */
	hp = chunk;
	limit = chunk + (((heap_chunk_head *) chunk) [-1]).size;
	while (hp < limit){
	  if (Is_gray_val (Val_hp (hp))){
	    trace_from (Val_hp (hp));
	  }
	  hp += Bhsize_hp (hp);
	}
	Assert (hp == limit);
	/* Find the next chunk */
	mutex_lock (heap_chunk_mutex);{
	  chunk = (((heap_chunk_head *) chunk) [-1]).next;
	}mutex_unlock (heap_chunk_mutex);
      }
      Assert (hp == heap_end);
    }
  }
  mutex_unlock (global_data_mutex);

  /* A note about the sweeping code.
     A new heap chunk is inserted in the chunk list before it is marked
     "in heap" in the page table.  It can therefore happen that we sweep
     a block that is not in heap.  In that case it is always a blue block. */

  /* Unsynchronized accesses : writing a word is atomic. */
  gc_sweep_hp = heap_start;	/* Before changing the phase !!! */
  current_gc_cycle->phase = Phase_sweep;

  gc_message_str ("-GC-");
  /* Second phase : sweeping. */
  { 
    uchar *limit, *chunk;

    chunk = heap_start;
    Assert (gc_sweep_hp == chunk);	/* Assigned above. */
    while (chunk != NULL){
      /* Sweep one chunk. */
      limit = chunk + (((heap_chunk_head *) chunk) [-1]).size;
      while (gc_sweep_hp < limit){
	switch (Color_hp (gc_sweep_hp)){
	case White:
	  if (Tag_hp (gc_sweep_hp) == Final_tag){
	    Final_fun (Val_hp (gc_sweep_hp)) (Val_hp (gc_sweep_hp));
	  }
nTrace ("free", 0, Val_hp (gc_sweep_hp));
	  accumulate (gc_sweep_hp);
	  break;
	case Gray:
	case Black:
	  Hd_hp (gc_sweep_hp) = Whitehd_hd (Hd_hp (gc_sweep_hp));
	  break;
	case Blue:
	  break;
	}
	gc_sweep_hp += Bhsize_hp (gc_sweep_hp);
      }
      end_accumulate ();
      Assert (gc_sweep_hp == limit);
      /* Find the next chunk. */
      mutex_lock (heap_chunk_mutex);{
	chunk = (((heap_chunk_head *) chunk) [-1]).next;
      }mutex_unlock (heap_chunk_mutex);
      gc_sweep_hp = chunk;
    }
    gc_sweep_hp = Max_pointer;
  }
}

any_t major_collection (dummy_arg)
     any_t dummy_arg;
{
  mutex_t dummy_mutex;

  dummy_mutex = mutex_alloc ();
  mutex_lock (dummy_mutex);
  while (1){
    condition_wait (memory_low, dummy_mutex);
    profil_enter (p_major_collection);
    gc_message_str ("#GC-");
    gc_major ();
    gc_message_str ("-GC#");
    profil_exit (p_major_collection);
  }
}

gc_cycle_t init_heap (heap_size)
     unsigned long heap_size;
{
  unsigned long i, j;

#ifdef PROFIL
  p_major_collection = profil_new ("Major_gc");
#endif PROFIL
  master_fl_mutex = mutex_alloc ();
  heap_chunk_mutex = mutex_alloc ();
  mutex_lock (master_fl_mutex);{

    heap_size = (heap_size + Page_size) / Page_size * Page_size;
    Assert (heap_size % Page_size == 0);
    total_heap_size = heap_size;
    gc_message ("Initial heap size: ", heap_size / 1024, "k.");
    heap_start = aligned_malloc (heap_size + sizeof (heap_chunk_head),
				 -sizeof (heap_chunk_head));
    if (heap_start == NULL)
      fatal_unix_error ("cannot allocate the initial heap", "");
    heap_start += sizeof (heap_chunk_head);
    Assert ((unsigned long) heap_start % Page_size == 0);
nTrace ("heap_start", heap_start, 0);
    mutex_lock (heap_chunk_mutex);{
      (((heap_chunk_head *) heap_start) [-1]).size = heap_size;
      (((heap_chunk_head *) heap_start) [-1]).next = NULL;
      last_chunk = heap_start;
    }mutex_unlock (heap_chunk_mutex);
    heap_end = heap_start + heap_size;
    Assert ((unsigned long) heap_end % Page_size == 0);
    page_table_size = 4 * heap_size / Page_size;
    page_table = malloc (page_table_size);
    if (page_table == NULL)
      fatal_unix_error ("cannot allocate the initial heap", "");
    for (i = 0; i < page_table_size; i++) page_table [i] = Not_in_heap;
    for (i = 0; i < Page_hp (heap_end); i++) page_table [i] = In_heap;
    Hd_hp (heap_start) = Make_header (Wosize_bhsize (heap_size), 0, Blue);
    master_fl = Bp_hp (heap_start);
    last_free_size = heap_size;
    running_procs = 1;

    for (i = 0; i < 2; i++){
      mutex_init (&gc_cycles [i].lock);
      gc_cycles [i].phase = Phase_mark;
      gc_cycles [i].head = NULL;
    }
    current_cycle_index = 0;
    current_gc_cycle = &(gc_cycles [current_cycle_index]);
    current_gc_cycle->phase = Phase_sweep;
    gc_sweep_hp = Max_pointer;

    memory_low = condition_alloc ();

  }mutex_unlock (master_fl_mutex);

  for (i = 0; i < 4; i++){
    small_fl [i].avail = 0;
    small_fl [i].mu = mutex_alloc ();
    for (j = 0; j < Num_small_fl; j++) small_fl [i].lists [j] = NULL;
    small_fl [i].tail = 0;
    small_fl [i].head = 0;
  }

  return current_gc_cycle;
}

void start_major_gc ()
{
  cthread_detach (cthread_fork (major_collection, (any_t) 0));
}
