/* memory.c */

#include "control.h"
#include "debugger.h"
#include "fail.h"
#include "freelist.h"
#include "gc.h"
#include "major_gc.h"
#include "mem_misc.h"
#include "memory.h"
#include "mlvalues.h"
#include "profil.h"

uchar *malloc ();
uchar *realloc ();

unsigned long young_size;                       /* In bytes. */
unsigned long arg_stack_size, ret_stack_size;   /* In words. */

/* Allocate more memory from malloc for the heap.
   Return a block of at least the requested size (in words).
   Return NULL when out of memory.
   This function must be called with [master_fl_mutex] locked.
*/
uchar *expand_heap (request)
     mlsize_t request;
{
  uchar *mem, *raw_mem;
  uchar *new_page_table;
  unsigned long new_page_table_size;
  uchar *new_heap_end;
  mlsize_t malloc_request;
  unsigned long i;

  malloc_request = Bhsize_wosize (request);
  if (malloc_request < Heap_grow_increment){
    malloc_request = Heap_grow_increment;
  }else{
    malloc_request = malloc_request + Heap_grow_increment;
    malloc_request = (malloc_request / Page_size) * Page_size;
  }
  Assert (malloc_request % Page_size == 0);
  total_heap_size += malloc_request;
  gc_message ("Expanding heap to ", total_heap_size / 1024, "k.\n");
  mem = aligned_malloc (malloc_request + sizeof (heap_chunk_head),
			-sizeof (heap_chunk_head));
  if (mem == NULL){
    gc_message_str ("No room.");
    return NULL;
  }
  mem += sizeof (heap_chunk_head);
  /* Make sure the new chunk is in a good format, so we can insert it in the
     chunk list used by sweeping code. */
  Hd_hp (mem) = Make_header (Wosize_bhsize (malloc_request), 0, Blue);
  /* The heap is made of chunks, chained by increasing addresses.
     The chaining is protected by [heap_chunk_mutex]. */
  mutex_lock (heap_chunk_mutex);{
    (((heap_chunk_head *) mem) [-1]).size = malloc_request;
    if (mem >= heap_end){
      (((heap_chunk_head *) mem) [-1]).next = NULL;
      (((heap_chunk_head *) last_chunk) [-1]).next = mem;
      last_chunk = mem;
      new_heap_end = mem + malloc_request;
    }else{
      uchar **last = &heap_start;
      uchar *cur = *last;
      while (cur < mem){
	last = &((((heap_chunk_head *) cur) [-1]).next);
	cur = *last;
	Assert (cur != NULL);
      }
      (((heap_chunk_head *) mem) [-1]).next = cur;
      *last = mem;
      new_heap_end = heap_end;
    }
  }mutex_unlock (heap_chunk_mutex);
  Assert ((unsigned long) new_heap_end % Page_size == 0);;
  new_page_table_size = page_table_size;
  while (new_page_table_size < Page_hp (new_heap_end)){
    new_page_table_size *= 2;
  }
  if (new_page_table_size > page_table_size){
    /* Reallocate the page table.  Reading a value, then [heap_end] and then
       [page_table] must yield a triple that will remain valid at all times.
       Thus we do not deallocate the old page table. */
    new_page_table = malloc (new_page_table_size);
    if (new_page_table == NULL){
      free (raw_mem);
      return NULL;
    }
    bcopy (page_table, new_page_table, page_table_size);
    for (i = page_table_size; i < new_page_table_size; i++){
      new_page_table [i] = Not_in_heap;
    }
    page_table = new_page_table;
    page_table_size = new_page_table_size;
  }
  for (i = Page_hp (mem); i < Page_hp (new_heap_end); i++){
    page_table [i] = In_heap;
  }
  heap_end = new_heap_end;
  return Bp_hp (mem);
}

/* General allocation function.
   Usage: call [alloc], which returns a value.
   Fill the fields of this value; then call [end_alloc].
*/
value alloc (wosize)
     mlsize_t wosize;
{
#define Setup_for_gc {}
#define Restore_after_gc {}
  value result;

  if (wosize < Max_young_size){
    Alloc_small_unshr (result, wosize, 0);
  }else{
    result = alloc_shr (wosize);
  }
  return result;
#undef Setup_for_gc
#undef Restore_after_gc
}

/* Allocate a big, shared, fixed object.
   Usage: see [alloc].
   This function can raise.  You must not hold any lock when calling it.
*/
#define Next(bp) (* (unsigned char **) (bp))

value alloc_shr (wosize)
     mlsize_t wosize;
{
  uchar *new_block;
  value result;

  /* First try the private freelist. */
  result = fl_allocate (private_fl, wosize);
  if (result == NULL){
    /* The object cannot be allocated from the private free-list.  Allocate
       it from the master free-list. */
    mutex_lock (master_fl_mutex);{
nTrace ("enter_alloc_shr_in_master", cthread_self (), 0);
      while (1){
	result = list_allocate (&master_fl, wosize);
	if (result != NULL) break;
	/* The object cannot be allocated from the master free-list.
	   Expand the heap. */
	new_block = expand_heap (wosize);
	if (new_block == NULL){
	  mutex_unlock (master_fl_mutex);
	  raise_out_of_memory ();
	}
	Next (new_block) = master_fl;
	master_fl = new_block;
      }
nTrace ("exit_alloc_shr_in_master", cthread_self (), 0);
    }mutex_unlock (master_fl_mutex);
  }
  Assert (Is_in_heap (result));
  return result;
}

/* Finish allocation.
   Usage: see [alloc].
*/
#ifdef ANSI
void end_alloc (value v, mlsize_t wosize, tag_t tag)
#else
void end_alloc (v, wosize, tag)
     value v;
     mlsize_t wosize;
     tag_t tag;
#endif
{
  /* mark phase: Black (or Gray).
     sweep phase: Gray in front of the sweep pointer;
                  White (or Gray) behind the sweep pointer.
     If the major GC changes phase from mark to sweep while we are allocating,
     we must change the color to gray.
  */
  if (local_current_cycle->phase == Phase_mark){
    Hd_val (v) = Make_header ((wosize), (tag), Black);
    if (local_current_cycle->phase != Phase_mark){
      /* The mark -> sweep transition took place. */
      Hd_val (v) = Make_header ((wosize), (tag), Gray);
    }
  }else{
    if (Hp_val (v) < gc_sweep_hp){
      Hd_val (v) = Make_header ((wosize), (tag), White);
    }else{
      Hd_val (v) = Make_header ((wosize), (tag), Gray);
    }
  }
}

#ifdef ANSI
value alloc_small_unshr (mlsize_t wosize, tag_t tag)
#else
value alloc_small_unshr (wosize, tag)
     mlsize_t wosize;
     tag_t tag;
#endif
{
#define Setup_for_gc
#define Restore_after_gc
  value result;

  Alloc_small_unshr (result, wosize, tag);
  return result;
#undef Setup_for_gc
#undef Restore_after_gc
}

value alloc_string (str)
     char *str;
{
#define Setup_for_gc
#define Restore_after_gc
  value result;
  unsigned long len = strlen (str);

  Alloc_string (result, len);
  bcopy ((uchar *) str, (uchar *) String_val (result), len);
  return result;
#undef Setup_for_gc
#undef Restore_after_gc
}

value alloc_double (dbl)
     double dbl;
{
#define Setup_for_gc
#define Restore_after_gc
  value result;

  Alloc_small_unshr (result, Double_size, Double_tag);
  Double_val (result) = dbl;
  return result;
#undef Setup_for_gc
#undef Restore_after_gc
}

uchar *stat_alloc (sz)
     mlsize_t sz;
{
  uchar *result = malloc (sz);

  if (result == NULL) raise_out_of_memory ();
  return result;
}

void stat_free (blk)
     uchar *blk;
{
  free (blk);
}

uchar *stat_resize (blk, sz)
     uchar *blk;
     mlsize_t sz;
{
  return realloc (blk, sz);
}

void init_memory (generation_size, arg_size, ret_size)
     unsigned long generation_size, arg_size, ret_size;
{
  young_size = generation_size;
  arg_stack_size = arg_size;
  ret_stack_size = ret_size;
  if (mem_low_percent < 1) mem_low_percent = 1;
  if (mem_low_percent > 99) mem_low_percent = 99;
}
