/* Allocation macros and functions */

#ifndef _memory_
#define _memory_


#include "gc.h"
#include "major_gc.h"
#include "mem_misc.h"
#include "mlvalues.h"
#include "private.h"

unsigned long young_size;
unsigned long arg_stack_size, ret_stack_size;
volatile int heap_is_pure;

#ifdef ANSI

extern void init_memory (unsigned long, unsigned long, unsigned long);
extern value alloc (mlsize_t);
extern value alloc_shr (mlsize_t);
extern void end_alloc (value, mlsize_t, tag_t);
extern value oldify (value);		/* Defined in [minor_gc.c]. */
#ifdef PROFIL
extern value timed_oldify (value);	/* Defined in [minor_gc.c]. */
#endif
extern value alloc_small_unshr (mlsize_t, tag_t);
extern value alloc_string (char *);
extern value alloc_double (double);
extern uchar *stat_alloc (mlsize_t);	/* Size in bytes. */
extern void stat_free (uchar *);
extern uchar *stat_resize (uchar *, mlsize_t); /* Size in bytes. */
extern void begin_blocking_section (void); /* Defined in [threads.c]. */
extern void end_blocking_section (void); /* Defined in [threads.c]. */

#else

void init_memory ();
value alloc ();
value alloc_shr ();
void end_alloc ();
value oldify ();		/* Defined in [minor_gc.c]. */
#ifdef PROFIL
value timed_oldify ();		/* Defined in [minor_gc.c]. */
#endif
value alloc_small_unshr ();
value alloc_string ();
value alloc_double ();
uchar *stat_alloc ();		/* Size in bytes. */
void stat_free ();
uchar *stat_resize ();		/* Size in bytes. */
void begin_blocking_section ();	/* Defined in [threads.c]. */
void end_blocking_section ();	/* Defined in [threads.c]. */

#endif /* ANSI */

#ifndef PROFIL
#define timed_oldify oldify
#endif

#define Alloc_small_unshr(result, wosize, tag) { \
  uchar *res = young_ptr; \
  young_ptr += Blsize_wosize (wosize); \
  if (young_ptr > young_end){ \
    Setup_for_gc; \
    minor_collection (); \
    Restore_after_gc; \
    res = young_ptr; \
    young_ptr += Blsize_wosize (wosize); \
  } \
  ((long_header_t *) res)->forward = 0; \
  ((long_header_t *) res)->hd = Make_header ((wosize), (tag), Black); \
  (result) = Val_lp (res); \
}

#define Alloc_string(result, len) { \
  mlsize_t wosize = ((len) + 1 + sizeof (value)) / sizeof (value); \
  if (wosize < Max_young_size) { \
    Alloc_small_unshr ((result), wosize, String_tag); \
  }else{ \
    (result) = alloc_shr (wosize); \
    end_alloc ((result), wosize, String_tag); \
  } \
  Field ((result), wosize - 1) = 0; \
  Byte ((result), (len)) = 0;\
  Byte ((result), (wosize*sizeof(value))-1) = (wosize*sizeof(value))-(len); \
}

/* [Initialize] is used for the first assignment to a field of a shared block.
   (Typically between [alloc_shr] and [end_alloc]. */
#define Initialize(op, val) { \
  value _new_ = Is_block (val) ? timed_oldify (val) : (val); \
  *(op) = _new_; \
  if (current_gc_cycle->phase == Phase_mark){ \
    if (Is_block (_new_) && Is_in_heap (_new_) && Is_white_val (_new_)){ \
      heap_is_pure = 0; \
      Hd_val (_new_) = Grayhd_hd (Hd_val (_new_)); \
    } \
  } \
}

/* [Modify] is used to change a field of an existing shared block. */
#define Modify(op, val) { \
  value _old_ = *(op); \
  value _new_ = Is_block (val) ? timed_oldify (val) : (val); \
  if (current_gc_cycle->phase == Phase_mark){ \
    if (Is_block (_old_) && Is_in_heap (_old_) && Is_white_val (_old_)){ \
      heap_is_pure = 0;  /* BEFORE erasing the old pointer. */ \
      Hd_val (_old_) = Grayhd_hd (Hd_val (_old_)); \
    } \
  } \
  *(op) = _new_; \
  if (current_gc_cycle->phase == Phase_mark){ \
    if (Is_block (_new_) && Is_in_heap (_new_) && Is_white_val (_new_)){ \
      heap_is_pure = 0; \
      Hd_val (_new_) = Grayhd_hd (Hd_val (_new_)); \
    } \
  } \
}

/* [Push_roots] and [Pop_roots] are used for C variables that are GC roots.
 * It must contain all values in C local variables at the time the minor GC is
 * called.
 * Usage:
 * At the end of the declarations of your C local variables, add
 * [ Push_roots (variable_name, size); ]
 * The size is the number of declared roots.  They are accessed as
 * [ variable_name [0] ... variable_name [size - 1] ].
 * The [variable_name] must not be [ _ ].
 * Just before the function return, add a call to [Pop_roots].
 */

#define Push_roots(name, size) \
   value name [(size) + 2]; \
   { long _; for (_ = 0; _ < (size); name [_++] = Val_long (0)); } \
   name [(size)] = (value) (size); \
   name [(size) + 1] = (value) c_roots_head; \
   c_roots_head = &(name [(size)]);

#define Pop_roots() {c_roots_head = (value *) c_roots_head [1]; }


#endif _memory_
