/* mlvalues.h */

#ifndef _mlvalues_
#define _mlvalues_


#include "config.h"

/* Definitions

  long: A C long integer.
  val: The ML representation of something.  A long or a block or a pointer
       outside the heap.  If it is a block, it is the (encoded) address
       of an object.  If it is a long, it is encoded as well.
  object: Something allocated.  It always has a header and some
          fields or some bytes.  If it is in a minor heap, it also has a
          pre-header.  The pre-header an header together form a long header.
  field: A word-sized val which is part of an object.
  bp: Pointer to the first byte of an object.  (a char *)
  op: Pointer to the first field of an object.  (a value *)
  hp: Pointer to the header of an object.  (a char *)
  lp: Pointer to the long header of an object.  Valid only if the object is
        in a minor heap.  (a char *)
  Remark: An object size is always a multiple of the word size.
  bosize: Size (in bytes) of the "bytes" part.
  wosize: Size (in words) of the "fields" part.
  bhsize: Size (in bytes) of the object with its header.
  whsize: Size (in words) of the object with its header.
  blsize: Size (in bytes) of the object with a long header.
  wlsize: Size (in words) of the object with a long header.
  hd: A header.
  lhd: A long header.
  tag: The value of the tag field of the header.
  color: The value of the color field of the header.
         This is for use only by the GC.
*/

typedef long value;
typedef unsigned long header_t;
typedef unsigned long mlsize_t;
typedef unsigned char tag_t;
typedef unsigned long color_t;
typedef unsigned long mark_t;
typedef struct {
  value forward;
  header_t hd;
} long_header_t;

/* Longs vs blocks. */
#define Is_long(x)   (((x) & 1) == 1)
#define Is_block(x)  (((x) & 1) == 0)

/* Conversion macro names are always of the form  "to_from". */
/* Example: Val_long as in "Val from long" or "Val of long". */
#define Val_long(x)     (((long)(x) << 1) + 1)
#define Long_val(x)     ((x) >> 1)
#define Max_long (0x3fffffff)
#define Min_long (-0x40000000)

/* Structure of the header:
     +--------+-------+-----+
     | wosize | color | tag |
     +--------+-------+-----+
bits  31    10 9     8 7   0
*/

#define Tag_hd(hd) ((tag_t) ((hd) & 0xFF))
#define Wosize_hd(hd) ((mlsize_t) ((hd) >> 10))

#define Hd_val(val) (((header_t *) (val)) [-1])
#define Hd_op(op) Hd_val (op)
#define Hd_bp(bp) Hd_val (bp)
#define Hd_hp(hp) (* ((header_t *) (hp)))
#define Lhd_val(val) (((long_header_t *) (val)) [-1])
#define Hp_val(val) ((unsigned char *) (((header_t *) (val)) - 1))
#define Hp_op(op) Hp_val (op)
#define Hp_bp(bp) Hp_val (bp)
#define Val_hp(hp) ((value) (((header_t *) (hp)) + 1))
#define Op_hp(hp) ((value *) Val_hp (hp))
#define Bp_hp(hp) ((unsigned char *) Val_hp (hp))
#define Val_lp(lp) ((value) (((long_header_t *) (lp)) + 1))
#define Val_bp(bp) ((value) (bp))

#define Num_tags (1 << 8)
#define Max_wosize ((1 << 22) - 1)

#define Wosize_val(val) (Wosize_hd (Hd_val (val)))
#define Wosize_op(op) Wosize_val(op)
#define Wosize_bp(bp) Wosize_val(bp)
#define Wosize_hp(hp) Wosize_hd (Hd_hp (hp))
#define Whsize_wosize(sz) ((sz) + 1)
#define Wosize_whsize(sz) ((sz) - 1)
#define Wosize_bhsize(sz) ((sz) / sizeof (value) - 1)
#define Bsize_wsize(sz) ((sz) * sizeof (value))
#define Wsize_bsize(sz) ((sz) / sizeof (value))
#define Bhsize_wosize(sz) Bsize_wsize (Whsize_wosize (sz))
#define Blsize_wosize(sz) (sizeof (long_header_t) + Bsize_wsize (sz))
#define Bhsize_bosize(sz) ((sz) + sizeof (header_t))
#define Bosize_val(val) Bsize_wsize (Wosize_val (val))
#define Bosize_op(op) Bosize_val (op)
#define Bhsize_hp(hp) Bsize_wsize (Whsize_wosize (Wosize_hp (hp)))
#define Whsize_hp(hp) Whsize_wosize (Wosize_hp (hp))

#ifdef Big_endian
#define Tag_val(val) (((unsigned char *) (val)) [-1])
#define Tag_hp(hp) (((unsigned char *) (hp)) [3])
#else
#define Tag_val(val) (((unsigned char *) (val)) [-4])
#define Tag_hp(hp) (((unsigned char *) (hp)) [0])
#endif

/* The Lowest tag for blocks containing no value. */
#define No_scan_tag (Num_tags - 4)


/* 1- If tag < No_scan_tag : a tuple of fields.  */

  /* Pointer to the first field. */
#define Op_val(x) ((value *) (x))
  /* Field number i (i = 0 => first field). */
#define Field(x, i) (((value *)(x)) [i])

typedef unsigned char *code_t;

#define Closure_wosize 2
#define Closure_tag (No_scan_tag - 1)
#define Code_val(val) (((code_t *) (val)) [0])
#define Env_val(val) (Field(val, 1))


/* 2- If tag >= No_scan_tag : a sequence of bytes. */

  /* Pointer to the first byte */
#define Bp_val(x) ((unsigned char *) (x))
  /* Byte number i. */
#define Byte(x, i) (((unsigned char *) (x)) [i])

/* Abstract things.  Their contents is not traced by the GC; therefore they
   must not contain any [value].
*/
#define Abstract_tag No_scan_tag

/* Strings. */
#define String_tag (No_scan_tag + 1)
#define String_val(x) ((char *) Bp_val(x))

/* Floating-point numbers. */
#define Double_tag (No_scan_tag + 2)
#define Double_size ((sizeof(double) / sizeof(value)))
#define Double_val(x) (* (double *) (x))

/* Finalized things.  Just like abstract things, but the GC will call the
   [Final_fun] before deallocation.
*/
#define Final_tag (No_scan_tag + 3)
#ifdef Ansi
typedef void (*final_fun) (value);
#else
typedef void (*final_fun) ();
#endif
#define Final_fun(val) (((final_fun *) (val)) [0])


/* 3- Atoms are 0-tuples.  They are statically allocated once and for all. */
extern header_t first_atoms[];
#define Atom(tag) (Val_hp (&(first_atoms [tag])))


#endif /* _mlvalues_ */
