/* threads.c */

#include "callback.h"
#include "control.h"
#include "debugger.h"
#include "fail.h"
#include "i_interp.h"
#include "mlvalues.h"
#include "memory.h"
#include "private.h"
#include "semaph.h"

typedef struct {
  struct semaphore init_done;
  gc_cycle_t current_cycle;
  value closure;
} new_thread_arg;

/* Any computation that might take an arbitrary amount of time without calling
   STARTFUN must be enclosed between calls to [begin_blocking_section] and
   [end_blocking_section].  The roots and allocation pointers must not change
   during this time.  Only static allocation is allowed. */
void begin_blocking_section ()
{
  mutex_unlock (&running);
  if (force_minor_gc){
    mutex_lock (&running);{
      force_minor_gc = 0;
      minor_collection ();
    }mutex_unlock (&running);
  }
}

void end_blocking_section ()
{
  mutex_lock (&running);
}

any_t new_thread (arg)
     new_thread_arg *arg;
{
  jmp_buf buf;
  value clos;
  
  clos = arg->closure;
  init_interp (arg->current_cycle);
  sem_V (&arg->init_done);
  if (setjmp (buf) == 0) {
    external_raise = (jmp_buf *) buf;
    call_back (clos, Atom (0));
  }
  final_interp ();
  return (any_t) 0;
}

value thread_fork (fct)		/* ML */
     value fct;
{
  cthread_t pid;
  value result;
  new_thread_arg *arg;

  arg = (new_thread_arg *) stat_alloc (sizeof (new_thread_arg));
  sem_init (&arg->init_done);
  arg->closure = oldify(fct);
  arg->current_cycle = local_current_cycle;
  Assert (sizeof (cthread_t) == sizeof (value));
  pid = cthread_fork (new_thread, arg);
  if (pid == NO_CTHREAD) {
    sem_clear (&arg->init_done);
    stat_free ((unsigned char *) arg);
    failwith("fork");
  }
  /* Wait for the child to initialize, or else the major GC might switch
     phases and leave the child behind. */
  sem_P (&arg->init_done);
  sem_clear (&arg->init_done);
  stat_free ((unsigned char *) arg);
  result = alloc_shr (1);
  Field (result, 0) = (value) pid;
  end_alloc (result, 1, Abstract_tag);
#ifdef PROFILING
  register_thread(pid);
#endif
  return result;
}

value thread_exit (arg)		/* ML */
     value arg;
{
  final_interp ();
  cthread_exit((any_t) 0);
  return Val_long(0);		/* Not reached. */
}

value thread_join (arg)		/* ML */
     value arg;
{
  begin_blocking_section ();
  cthread_join((cthread_t) Field(arg, 0));
  end_blocking_section ();
  return Atom(0);
}

value thread_detach (arg)	/* ML */
     value arg;
{
  cthread_detach((cthread_t) Field(arg, 0));
  return Atom(0);
}

value thread_yield (arg)	/* ML */
     value arg;
{
  cthread_yield();
  return Atom(0);
}

value mlthread_self (arg)	/* ML */
     value arg;
{
  value result;

  result = alloc_small_unshr (1, Abstract_tag);
  Field (result, 0) = (value) cthread_self();
  return result;
}

void finalization_mutex (mut)
     value mut;
{
  mutex_clear ((mutex_t) &Field(mut, 1));
}

value thread_new_mutex (arg)	/* ML */
     value arg;
{
  value result;

  result = alloc_shr (sizeof(struct mutex) / sizeof(value) + 1);
  end_alloc (result, sizeof(struct mutex) / sizeof(value) + 1, Final_tag);
  Final_fun (result) = finalization_mutex;
  mutex_init ((mutex_t) &Field(result, 1));
  return result;
}

void finalization_condition (cond)
     value cond;
{
  condition_clear ((condition_t) &Field(cond, 1));
}

value thread_new_condition (arg)	/* ML */
     value arg;
{
  value result;

  result = alloc_shr (sizeof(struct condition) / sizeof(value) + 1);
  end_alloc (result, sizeof(struct condition) / sizeof(value) + 1, Final_tag);
  Final_fun (result) = finalization_condition;
  condition_init ((condition_t) &Field(result, 1));
  return result;
}

value thread_lock (lock)	/* ML */
     value lock;
{
  begin_blocking_section ();
  mutex_lock ((mutex_t) &Field(lock, 1));
  end_blocking_section ();
  return Atom(0);
}

value thread_unlock (lock)	/* ML */
     value lock;
{
  mutex_unlock ((mutex_t) &Field(lock, 1));
  return Atom(0);
}

value thread_try_lock (lock)	/* ML */
     value lock;
{
  if (mutex_try_lock ((mutex_t) &Field(lock, 1)))
    return Atom(1);
  else
    return Atom(0);
}

value thread_signal (cond)	/* ML */
     value cond;
{
  condition_signal ((condition_t) &Field(cond, 1));
  return Atom(0);
}

value thread_broadcast (cond)	/* ML */
     value cond;
{
  condition_broadcast ((condition_t) &Field(cond, 1));
  return Atom(0);
}

value thread_wait (cond, lock)	/* ML */
     value cond, lock;
{
  begin_blocking_section ();
  condition_wait ((condition_t) &Field(cond, 1), (mutex_t) &Field(lock, 1));
  end_blocking_section ();
  return Atom(0);
}

value thread_get_private_data () /* ML */
{
  return per_thread_data;
}

value thread_set_private_data (new_val) /* ML */
     value new_val;
{
/*  Modify(&per_thread_data, new_val); */
  per_thread_data = new_val;
  return Atom(0);
}
