#open "unix";;
#open "obj";;
#open "meta";;
#open "signal";;
#open "misc";;
#open "config";;
#open "const";;
#open "syntax";;
#open "builtins";;
#open "modules";;
#open "location";;
#open "globals";;
#open "types";;
#open "typing";;
#open "ty_decl";;
#open "front";;
#open "back";;
#open "instruct";;
#open "opcodes";;
#open "buffcode";;
#open "labels";;
#open "symtable";;
#open "emitcode";;
#open "emitdata";;
#open "compiler";;
#open "link";;
#open "pr_value";;

(* Utilisation interactive *)

exception Break;;

(* Chargement en memoire et execution d'une phrase, d'un fichier objet *)

let check_suffix name suffix =
  if sub_string name (string_length name - string_length suffix)
                     (string_length suffix) = suffix
  then (sub_string name 0 (string_length name - string_length suffix), name)
  else (name, name ^ suffix)
;;

let load_phrase phr =
  init_out_code();
  reloc__reset();
  reset_label_table();
  literal_table := [];
  let entrypoint =
    if phr.kph_rec then begin
      emit phr.kph_init;
      out STOP;
      emit phr.kph_fcts;
      0
    end else begin
      emit phr.kph_fcts;
      let p = get_out_position() in
      emit phr.kph_init;
      out STOP;
      p
    end in
  let code =
    dump_output_to_heap() in
  patch_object code (reloc__get_info());
  if number_of_globals() >= vect_length global_data then
    realloc_global_data(number_of_globals());
  emit_data_to_heap ();
  try
    let res =
      interprete code entrypoint in
    begin match phr.kph_fcts with [] -> static_free code | _ -> () end;
    res
  with x ->
    begin match phr.kph_fcts with [] -> static_free code | _ -> () end;
    match x with
      Break ->
        raise Break
    | _ ->
        print_begline "Uncaught exception: ";
        print_value (Repr x) type_exn;
        print_endline "";
        raise Toplevel
;;

let load_object name =
  let (_, filename) = check_suffix name ".zo" in
  let truename = find_in_path filename !path_interfaces in
  let inchan = open_in truename in
    object_name := name;
    let n = input_int inchan in
    let buff = static_alloc (n+1) in
    input inchan buff 0 n;
    set_nth_char buff n (char_of_int STOP);
    patch_object buff (input_value inchan : (reloc__info * int) list);
    close_in inchan;
    if number_of_globals() >= vect_length global_data then
      realloc_global_data(number_of_globals());
    emit_data_to_heap ();
    interprete buff 0;
    ()
;;

let do_toplevel_phrase (Impl(desc,loc)) =
  reset_type_expression_vars [];
  begin match desc with
    Zexpr expr ->
      let ty =
        type_expression loc expr in
      let _ =
        flush std_out in
      let res =
        load_phrase(compile_lambda false (translate_expression expr)) in
      print_begline "- = ";
      print_value res ty;
      print_endline ""
  | Zletdef(rec_flag, pat_expr_list) ->
      let env = type_letdef loc rec_flag pat_expr_list in
      let _ = flush std_out in
        if rec_flag then
          load_phrase
            (compile_lambda true (translate_letdef_rec loc pat_expr_list))
        else
          load_phrase
            (compile_lambda false(translate_letdef loc pat_expr_list));
        reset_rollback ();
        do_list
          (fun (name, (typ, mut_flag)) ->
            print_begline name; print_string " = ";
            print_value
              global_data.(get_slot_for_variable
                           {qual=compiled_module_name(); id=name})
              typ;
            print_endline "")
          env
  | Ztypedef decl ->
      type_typedecl loc decl
  | Zexcdef decl ->
      type_excdecl loc decl
  | Zimpldirective dir ->
      do_directive dir
  end;
  flush std_out
;;

(* Chargement d'un fichier *)

let loadfile filename =
  let (_, truename) = check_suffix filename ".ml" in
  let ic = open_in truename in
  let lexbuf = lexing__create_lexer_channel ic in
  let saved_input_name = !input_name
  and saved_input_chan = !input_chan
  and saved_input_lexbuf = !input_lexbuf in
    input_name := truename;
    input_chan := ic;
    input_lexbuf := lexbuf;
    try
      while true do
        do_toplevel_phrase(parse_impl_phrase lexbuf)
      done
    with x ->
      close_in ic;
      input_name := saved_input_name;
      input_chan := saved_input_chan;
      input_lexbuf := saved_input_lexbuf;
      match x with
        End_of_file -> ()
      | Toplevel -> ()
      | _ -> raise x
;;

let include name =
  let (modname, filename) = check_suffix name ".ml" in
    loadfile filename
;;

(* Comme include, mais avec creation d'un nouveau module *)

let load name =
  let (modname, filename) = check_suffix name ".ml" in
  let saved_defined_module = !defined_module
  and saved_used_modules = !used_modules
  and saved_type_stamp = !type_stamp in
  try
    new_globalenv modname;
    loadfile filename;
    defined_module := saved_defined_module;
    used_modules := saved_used_modules;
    type_stamp := saved_type_stamp;
    ()
  with x ->
(*  kill_module modname; *)
    defined_module := saved_defined_module;
    used_modules := saved_used_modules;
    type_stamp := saved_type_stamp;
    raise x
;;

(* Pour sortir --- bien que "^D" marche aussi bien... *)

let quit x = exit 0
;;

(* Cette saloperie de trace *)

let (trace, untrace) =
  let trace_env = ref [] in
  let rec trace_instr name obj ty =
    match type_repr ty with
      Typ(Tarrow(t1,t2), _) ->
        Repr(fun arg ->
          print_begline name; print_string " <-- ";
          print_value arg t1; print_newline();
          try
            let res = (magic_obj obj : obj -> obj) arg in
              print_begline name; print_string " --> ";
              print_value res t2; print_newline();
              trace_instr (name ^ "*") res t2
          with exc ->
            print_begline name; print_string " raises exception ";
            print_value (Repr exc) type_exn; print_newline();
            raise exc)
    | _ -> obj in
  (* trace *)
  (fun name ->
    try
      let val_desc = find_value_desc (GRname name) in
      let pos = get_slot_for_variable val_desc.qualid in
        if mem_assoc pos !trace_env then begin
          prerr_begline "> "; prerr_string name;
          prerr_endline " is already traced."
        end else begin
          trace_env := (pos, global_data.(pos)) :: !trace_env;
          global_data.(pos) <-
            trace_instr name global_data.(pos) val_desc.info.val_typ;
          prerr_begline "> "; prerr_string name;
          prerr_endline " is now traced."
        end
    with failure _ ->
      prerr_begline ">> "; prerr_string name; prerr_endline " is undefined."),
  (* untrace *)
  (fun name ->
    try
      let val_desc = find_value_desc (GRname name) in
      let pos = get_slot_for_variable val_desc.qualid in
      let rec except = function
        [] -> prerr_begline "> "; prerr_string name;
              prerr_endline " was not traced.";
              []
      | (pos',obj as pair)::rest ->
          if pos == pos' then begin
            global_data.(pos) <- obj;
            prerr_begline "> "; prerr_string name;
            prerr_endline " is no more traced.";
            rest
          end else
            pair :: except rest
      in
        trace_env := except !trace_env;
        ()
    with failure _ ->
      prerr_begline ">> "; prerr_string name; prerr_endline " is undefined.")
;;

(* Pour definir ses propres fonctions d'impression *)

let new_printer name printer =
  try
    printers := (find_type_desc (GRname name), magic printer) :: !printers;
    ()
  with failure _ ->
    prerr_begline ">> Type "; prerr_string name;
    prerr_endline " is undefined."
;;

let default_printer name =
  try
    let ty_constr = find_type_desc (GRname name) in
    let rec remove = function
      [] -> []
    | (ty,_ as pair)::rest ->
        if same_type_constr ty ty_constr then rest else pair :: remove rest in
    printers := remove !printers;
    ()
  with failure _ ->
    prerr_begline ">> Type "; prerr_string name;
    prerr_endline " is undefined."
;;


(* La boucle principale *)

try
  toplevel := true;
  prerr_begline version__banner;
  prerr_endline2 "";
  path_interfaces := [config__path_library; misc__current_dir_name];
  default_used_modules := "toplvl" :: assoc "cautious" default_used_interfaces;
  let current_arg = ref 1 in
  let arg_of_option () =
    let l = string_length command_line.(!current_arg) in
      if l > 2 then
        sub_string command_line.(!current_arg) 2 (l - 2)
      else if incr current_arg >= vect_length command_line then
       (prerr_begline ">> Missing parameter to option ";
        prerr_endline command_line.(pred !current_arg);
        raise Toplevel)
      else command_line.(!current_arg) in
  while !current_arg < vect_length command_line do
    let s = command_line.(!current_arg) in
    if nth_char s 0 == `-` then
      begin match nth_char s 1 with
        `I` -> path_interfaces := arg_of_option() :: !path_interfaces; ()
      | `O` -> let set = arg_of_option() in
                 begin try
                   default_used_modules :=
                     assoc set default_used_interfaces; ()
                 with failure "find" ->
                   prerr_begline ">> Unknown module set: ";
                   prerr_endline2 set
                 end
      |  _  -> prerr_begline "> Unrecognized option ";
               prerr_endline2 s
      end;
    incr current_arg
  done;
  let ic = open_in command_line.(0) in
    while input_char ic != `\010` do () done;
    let size_code = input_int ic in
    let size_data = input_int ic in
    let size_symb = input_int ic in
    let size_debug = input_int ic in
      seek_in ic (pos_in ic + size_code + size_data);
      load_linker_tables ic;
      close_in ic;
      new_globalenv "top";
      let lexbuf = lexing__create_lexer_channel std_in in
      set_signal SIGINT (Signal_raise Break);
      input_lexbuf := lexbuf;
      while true do
        try
          print_string config__toplevel_input_prompt;
          flush std_out;
          reset_rollback();
          do_toplevel_phrase(parse_impl_phrase lexbuf)
        with End_of_file ->
               close_out std_out; close_out std_err; exit 0
           | Toplevel ->
               flush std_out;
               protect rollback ()
           | Break ->
               print_begline "Interrupted."; print_endline "";
               flush std_out;
               protect rollback ()
      done
with x ->
  prerr_begline ">> Uncaught internal exception.";
  prerr_endline "";
  close_out std_out; close_out std_err; exit 4
;;
