#open "misc";;
#open "const";;
#open "lambda";;
#open "prim";;
#open "instruct";;

(*  back.ml : traduction lambda-calcul -> liste de code  *)

let reset_label, new_label =
  let label_counter = ref 0 in
    (fun () -> label_counter := 0),
    (fun () -> incr label_counter)
;;

let is_return =
  function Kreturn :: _ -> true | _ -> false
;;

let label_code = function
    Kbranch lbl :: _ as C ->
      (lbl, C)
  | Klabel lbl :: _ as C ->
      (lbl, C)
  | C ->
      let lbl = new_label() in
        (lbl, Klabel lbl :: C)
;;

let make_branch = function
    Kreturn :: _ as C ->
      (Kreturn, C)
  | (Kbranch _ as branch) :: _ as C ->
      (branch, C)
  | C ->
      let lbl = new_label() in
        (Kbranch lbl, Klabel lbl :: C)
;;

(* Le critere de choix d'un saut a travers table de preference a une
   suite de tests.                                                        *)

let cond_through_jumptable size casel =
  let l = list_length casel in
    (size < l * 4)
;;

let switch_through_jumptable size casel =
  for_all (fun (ConstrRegular (_,_),_) -> true | _ -> false) casel &
  cond_through_jumptable size casel
;;

(* Inversion d'un test booleen ( < devient >= and so on) *)

let invert_bool_test =
  let invert_prim_test = function
      PTeq -> PTnoteq
    | PTnoteq -> PTeq
    | PTnoteqimm x -> fatal_error "invert_prim_test"
    | PTlt -> PTge
    | PTle -> PTgt
    | PTgt -> PTle
    | PTge -> PTlt in
  function
      Peq_test -> Pnoteq_test
    | Pnoteq_test -> Peq_test
    | Pint_test t -> Pint_test(invert_prim_test t)
    | Pfloat_test t -> Pfloat_test(invert_prim_test t)
    | Pstring_test t -> Pstring_test(invert_prim_test t)
    | Peqtag_test t -> Pnoteqtag_test t
    | Pnoteqtag_test t -> Peqtag_test t
;;

(* Production d'un test immediat *)

let test_for_atom = function
    ACint x -> Pint_test(PTnoteqimm x)
  | ACchar x -> Pint_test(PTnoteqimm (int_of_char x))
  | ACfloat x -> Pfloat_test(PTnoteqimm x)
  | ACstring x -> Pstring_test(PTnoteqimm x)
;;

(*************************** Le compilateur *****************************)

let reset_to_compile, add_to_compile, get_to_compile =
  let still_to_compile = ref ([] : (string * lambda * int) list) in
    (fun () -> still_to_compile := []; ()),
    (fun bc -> still_to_compile := bc :: !still_to_compile; ()),
    (fun () -> match !still_to_compile with
                    []     -> failwith "finished"
                | bc::rest -> still_to_compile := rest; bc)
;;

(* staticfail : le label ou sauter quand on rencontre un Lstaticfail.
   lambda : une lambda-expression a compiler.
   C : la continuation == le code a placer apres le code de "lambda".       *)

(* Les tests sur la continuation servent a eviter les sauts vers des sauts,
   des sauts vers Return, et autres horreurs du meme genre.                 *)

let rec compile_expr staticfail =
  let rec compexp =
    let rec compexplist = fun
        [] C -> C
      | [exp] C -> compexp exp C
      | (exp::rest) C -> compexplist rest (Kpush :: compexp exp C)

    and comp_test2 cond ifso ifnot C =
      let branch1, C1 = make_branch C
      and lbl2 = new_label() in
        compexp cond (Kbranchifnot lbl2 ::
                     compexp ifso (branch1 :: Klabel lbl2 :: compexp ifnot C1))

    and comp_tests test_of_tag casel C =
      let branch1, C1 = make_branch C in
      let rec comp = function
          [] ->
            fatal_error "comp_tests"
        | [tag,exp] ->
            Ktest(test_of_tag tag, staticfail) :: compexp exp C1
        | (tag,exp)::rest ->
            let lbl = new_label() in
              Ktest(test_of_tag tag, lbl) ::
              compexp exp (branch1 :: Klabel lbl :: comp rest)
      in
        comp casel

    and comp_switch int_of_tag size casel C =
      let branch1, C1 = make_branch C in
      let switchtable = make_vect size staticfail in
      let rec comp_case = function
          [] ->
            fatal_error "comp_switch"
        | [tag, exp] ->
            let lbl = new_label() in
              switchtable.(int_of_tag tag) <- lbl;
              Klabel lbl :: compexp exp C1
        | (tag, exp) :: rest ->
            let lbl = new_label() in
              switchtable.(int_of_tag tag) <- lbl;
              Klabel lbl :: compexp exp (branch1 :: comp_case rest)
      in
        Kswitch switchtable :: comp_case casel

  in fun
    (Lvar n) C ->
        Kaccess n :: C
  | (Lconst cst) C ->
       (match C with
          (Kquote _ | Kget_global _ | Kaccess _ | Kpushmark) :: _ -> C
        | _ -> Kquote cst :: C)
  | (Lapply(body, args)) C ->
       (match C with
          Kreturn :: C' ->
            compexplist args (Kpush :: compexp body (Ktermapply :: C'))
        | _ ->
            Kpushmark ::
            compexplist args (Kpush :: compexp body (Kapply :: C)))
  | (Lfunction(name, body)) C ->
        if is_return C then
          Kgrab :: compexp body C
        else
         (let lbl = new_label() in
            add_to_compile (name, body, lbl);
            Kclosure lbl :: C)
  | (Llet(args, body)) C ->
        let C1 = if is_return C then C else Kendlet(list_length args) :: C in
	let C2 = compexp body C1 in
        let rec comp_args = function
	       []     -> C2
	  | exp::rest -> compexp exp (Klet :: comp_args rest) in
	comp_args args
  | (Lletrec(args, body)) C ->
        let size = list_length args in
        let C1 = if is_return C then C else Kendlet size :: C in
	let C2 = compexp body C1 in
	let rec comp_args i = function
	    []        -> C2
	  | (exp, sz)::rest
            -> compexp exp (Kupdate i :: comp_args (i-1) rest) in
	(map (fun (e, sz) -> Kdummy sz) args) @ comp_args (size-1) args
  | (Lprim(Pget_global qualid, [])) C ->
        Kget_global qualid :: C
  | (Lprim(Pset_global qualid, [exp])) C ->
        compexp exp (Kset_global qualid :: C)
  | (Lprim(Pdummy_global (qualid, sz), [])) C ->
        Kdummy sz :: Kaccess 0 :: Kset_global qualid :: Kendlet 1 :: C
  | (Lprim(Pupdate_global qualid, [exp])) C ->
        Kget_global qualid :: Klet
        :: compexp exp (Kupdate 0 :: Kendlet 1 :: C)
  | (Lprim(Pmakeblock (tag, shared), explist)) C ->
        compexplist explist (Kmakeblock(tag, list_length explist, shared) :: C)
  | (Lprim(Pnot, [exp])) C ->
       (match C with
          Kbranchif lbl :: C' ->
            compexp exp (Kbranchifnot lbl :: C')
        | Kbranchifnot lbl :: C' ->
            compexp exp (Kbranchif lbl :: C')
        | _ ->
            compexp exp (Kprim Pnot :: C))
  | (Lprim((Ptest tst as p), explist)) C ->
       (match C with
          Kbranchif lbl :: C' ->
            compexplist explist (Ktest(tst,lbl) :: C')
        | Kbranchifnot lbl :: C' ->
            compexplist explist (Ktest(invert_bool_test tst,lbl) :: C')
        | _ ->
            compexplist explist (Kprim p :: C))
  | (Lprim(p, explist)) C ->
        compexplist explist (Kprim p :: C)
  | (Lstatichandle(body, Lstaticfail)) C ->
        compexp body C
  | (Lstatichandle(body, handler)) C ->
        let branch1, C1 = make_branch C
        and lbl2 = new_label() in
          compile_expr lbl2 body (branch1 :: Klabel lbl2 :: compexp handler C1)
  | (Lstaticfail) C ->
        Kbranch staticfail :: C
  | (Lhandle(body, handler)) C ->
        let branch1, C1 = make_branch C in
        let lbl2 = new_label() in
        let C2 = if is_return C1 then C1 else Kendlet 1 :: C1 in
          Kpushtrap lbl2 ::
            compexp body
                    (Kpoptrap :: branch1 :: Klabel lbl2 :: compexp handler C2)
  | (Lifthenelse(cond, ifso, ifnot)) C ->
        comp_test2 cond ifso ifnot C
  | (Lsequence(exp1, exp2)) C ->
        compexp	exp1 (compexp exp2 C)
  | (Lwhile(cond, body)) C ->
        let lbl1 = new_label() and lbl2 = new_label() in
          Kbranch lbl1 :: Klabel lbl2 :: Kstartfun "" ::
          compexp body (Klabel lbl1 :: compexp cond (
            Kbranchif lbl2 :: Kquote(SCblock(ConstrRegular(0,0), [])) :: C))
  | (Lfor(start, stop, up_flag, body)) C ->
        let lbl_end = new_label()
        and lbl_loop = new_label() in
          compexp start (
            Kmakeblock(ConstrRegular(0,1), 1, Unshared) :: Klet ::
            compexp stop (
              Klet :: Klabel lbl_loop :: Kstartfun "" ::
              Kaccess 1 :: Kprim(Pfield 0) :: Kpush :: Kaccess 0 ::
              Ktest(Pint_test(if up_flag then PTlt else PTgt), lbl_end) ::
              compexp body (
                Kaccess 1 :: Kprim(if up_flag then Pincr else Pdecr) ::
                Kbranch lbl_loop ::
                Klabel lbl_end :: Kendlet 2 ::
                Kquote(SCblock(ConstrRegular(0,0), [])) :: C)))
  | (Lsequand(exp1, exp2)) C ->
       (match C with
          Kbranch lbl :: _  ->
            compexp exp1 (Kstrictbranchifnot lbl :: compexp exp2 C)
        | Kbranchifnot lbl :: _ ->
            compexp exp1 (Kbranchifnot lbl :: compexp exp2 C)
        | Kbranchif lbl :: C' ->
            let lbl1, C1 = label_code C' in
              compexp exp1 (Kbranchifnot lbl1 ::
                            compexp exp2 (Kbranchif lbl :: C1))
        | _ ->
            let lbl = new_label() in
              compexp exp1 (Kstrictbranchifnot lbl ::
                            compexp exp2 (Klabel lbl :: C)))
  | (Lsequor(exp1, exp2)) C ->
       (match C with
          Kbranch lbl :: _  ->
            compexp exp1 (Kstrictbranchif lbl :: compexp exp2 C)
        | Kbranchif lbl :: _  ->
            compexp exp1 (Kbranchif lbl :: compexp exp2 C)
        | Kbranchifnot lbl :: C' ->
            let lbl1, C1 = label_code C' in
              compexp exp1 (Kbranchif lbl1 ::
                            compexp exp2 (Kbranchifnot lbl :: C1))
        | _ ->
            let lbl = new_label() in
              compexp exp1 (Kstrictbranchif lbl ::
                            compexp exp2 (Klabel lbl :: C)))
  | (Lcond(arg, casel)) C ->
        let C1 =
          match casel with
            (ACchar _,_) :: _ ->
              if cond_through_jumptable 256 casel
              then comp_switch int_of_atom 256 casel C
              else comp_tests test_for_atom casel C
          | _ ->
              comp_tests test_for_atom casel C
        in
          compexp arg C1
  | (Lswitch(1, arg, [ConstrRegular(_,_), exp])) C ->
        compexp exp C
        (* En supposant que l'argument est toujours du code pur !!!
           (vrai quand c'est le pattern-matcher qui genere le Switch).     *)
  | (Lswitch(2, arg, [ConstrRegular(0,_), exp0])) C ->
        compexp arg (Kbranchif staticfail :: compexp exp0 C)
  | (Lswitch(2, arg, [ConstrRegular(1,_), exp1])) C ->
        compexp arg (Kbranchifnot staticfail :: compexp exp1 C)
  | (Lswitch(2, arg, [ConstrRegular(0,_), exp0;
                      ConstrRegular(1,_), exp1])) C ->
        comp_test2 arg exp1 exp0 C
  | (Lswitch(2, arg, [ConstrRegular(1,_), exp1;
                      ConstrRegular(0,_), exp0])) C ->
        comp_test2 arg exp1 exp0 C
  | (Lswitch(size, arg, casel)) C ->
        let C1 = 
          if switch_through_jumptable size casel
          then Kprim Ptag_of :: comp_switch int_of_constr_tag size casel C
          else comp_tests (fun tag -> Pnoteqtag_test tag) casel C
        in
          compexp arg C1

  in compexp
;;

let rec compile_rest C =
  try
    let (name, exp, lbl) = get_to_compile() in
      compile_rest (Klabel lbl :: Kstartfun name ::
                      compile_expr Nolabel exp (Kreturn :: C))
  with failure "finished" ->
    C
;;

let compile_lambda (rec_flag : bool) expr =
  reset_to_compile(); reset_label();
  let init_code =
    compile_expr Nolabel expr [] in
  let function_code =
    compile_rest [] in
  { kph_rec = rec_flag; kph_init = init_code; kph_fcts = function_code }
;;
