#open "unix";;
#open "obj";;
#open "misc";;
#open "const";;
#open "lexer";;
#open "parser";;
#open "location";;
#open "syntax";;
#open "builtins";;
#open "hashtbl";;
#open "globals";;
#open "modules";;
#open "types";;
#open "ty_error";;
#open "typing";;
#open "ty_decl";;
#open "ty_intf";;
#open "front";;
#open "instruct";;
#open "back";;
#open "buffcode";;
#open "emitcode";;

(* Fonctions de parsing *)

let parse_phrase parsing_fun lexing_fun lexbuf =
  let rec skip () =
    try
      match lexing_fun lexbuf with
        EOF -> ()
      | SEMISEMI -> ()
      | _ -> skip()
    with lexer__Lexical_error(_,_,_) ->
      skip() in
  try
    let res = parsing_fun lexing_fun lexbuf in
      parsing__clear_parser();
      res
  with parsing__Parse_error f ->
         let pos1 = lexing__get_lexeme_start lexbuf in
         if f (obj__Repr EOF) or f (obj__Repr SEMISEMI) then () else skip();
         let pos2 = lexing__get_lexeme_end lexbuf in
         prerr_location (Loc(pos1, pos2));
         prerr_begline ">> Syntax error.";
         prerr_endline "";
         raise Toplevel
     | lexer__Lexical_error(msg, pos1, pos2) ->
         if pos1 >= 0 & pos2 >= 0 then prerr_location (Loc(pos1, pos2));
         prerr_begline ">> Lexical error: ";
         prerr_string msg;
         prerr_endline ".";
         skip();
         raise Toplevel
     | Toplevel ->
         skip ();
         raise Toplevel
;;

let parse_impl_phrase = parse_phrase Implementation Main
and parse_intf_phrase = parse_phrase Interface Main
;;

(* Execution des directives *)

let do_directive = function
    Zdir("open", name) ->
      used_modules := find_module name :: !used_modules; ()
  | Zdir("close", name) ->
      let rec remove_module = function
        [] ->
          prerr_begline "> Module \""; prerr_string name;
          prerr_endline "\" was not open.";
          []
      | md :: rest ->
          if md.mod_name = name then rest else md :: remove_module rest in
      used_modules := remove_module !used_modules; ()
  | Zdir("infix", name) ->
      add_infix name; ()
  | Zdir("uninfix", name) ->
      remove_infix name; ()
  | Zdir(d, name) ->
      prerr_begline "> Unknown directive \"";
	  prerr_string d;
      prerr_endline2 "\", ignored."
;;

(* Compilation d'une interface *)

let compile_intf_phrase (Intf(desc,loc)) =
  begin match desc with
    Zvaluedecl decl ->
      type_valuedecl loc decl
  | Ztypedecl decl ->
      type_typedecl loc decl
  | Zexcdecl decl ->
      type_excdecl loc decl
  | Zintfdirective dir ->
      do_directive dir
  end;
  flush std_out
;;

let compile_intf name filename =
  let intf_name = name ^ ".zi" in
  let ic = open_in filename
  and oc = open_out intf_name in
    new_globalenv name;
    type_stamp := 268435456;
(* Hack pour eviter de donner le meme stamp a un type defini ou declare dans
   l'interface et a un type defini dans l'implementation *)
    let lexbuf = lexing__create_lexer_channel ic in
    input_name := filename;
    input_chan := ic;
    input_lexbuf := lexbuf;
    try
      while true do
        compile_intf_phrase(parse_intf_phrase lexbuf)
      done
    with End_of_file ->
      close_in ic;
      write_compiled_interface oc;
      close_out oc
    | x ->
      close_in ic;
      close_out oc;
      unix__unlink intf_name;
      raise x
;;


(* Compilation d'une implementation *)

let compile_impl_phrase outstream (Impl(desc,loc)) =
  reset_type_expression_vars [];
  begin match desc with
    Zexpr expr ->
      type_expression loc expr;
      emit_phrase outstream (compile_lambda false (translate_expression expr))
  | Zletdef(rec_flag, pat_expr_list) ->
      type_letdef loc rec_flag pat_expr_list;
      emit_phrase outstream
         (if rec_flag
          then compile_lambda true  (translate_letdef_rec loc pat_expr_list)
          else compile_lambda false (translate_letdef loc pat_expr_list))
  | Ztypedef decl ->
      type_typedecl loc decl
  | Zexcdef decl ->
      type_excdecl loc decl
  | Zimpldirective dir ->
      do_directive dir
  end;
  flush std_out
;;

let compile_impl name filename =
  let impl_name = name ^ ".zo" in
  let ic = open_in filename
  and oc = open_out impl_name in
  let lexbuf = lexing__create_lexer_channel ic in
    input_name := filename;
    input_chan := ic;
    input_lexbuf := lexbuf;
    output_int oc 0;
    abs_out_pos := 0;
    reloc__reset();
    try
      while true do
        compile_impl_phrase oc (parse_impl_phrase lexbuf)
      done
    with End_of_file ->
      output_value oc (reloc__get_info ());
      seek_out oc 0;
      output_int oc !abs_out_pos;
      close_in ic;
      close_out oc
    | x ->
      close_in ic;
      close_out oc;
      unix__unlink impl_name;
      raise x
;;

let compile_implementation name filename =
  new_globalenv name;
  if file_exists (name ^ ".mli") then
    begin
      let intf = load_module name in
        enter_interface_definitions intf;
        compile_impl name filename;
        check_interface intf
    end
  else
    begin
      let intf_name = name ^ ".zi" in
      let oc = open_out intf_name in
        try
          compile_impl name filename;
          write_compiled_interface oc;
          close_out oc
        with x ->
          close_out oc;
          unix__unlink intf_name;
          raise x
    end
;;
