#open "misc";;
#open "const";;
#open "globals";;
#open "syntax";;
#open "location";;
#open "builtins";;
#open "modules";;
#open "lambda";;
#open "prim";;
#open "match";;

(*  front.ml : traduction syntaxe abstraite -> lambda-calcul etendu.
               Reconnaissance des identificateurs.
               Expansion du pattern matching en tests.
               Propagation des constantes.                              *)


(* Gestion de l'environnement de traduction *)

(* Chaque case de l'environnement correspond a un motif;
   on lui associe la liste des variables de ce motif avec pour chaque
   le chemin d'acces *)

type access_path =
    Path_root
  | Path_son of int * access_path
;;

type transl_env =
    Tnullenv
  | Treserved of transl_env
  | Tenv of (string * access_path) list * transl_env
;; 

let rec paths_of_pat path (Pat(desc,loc)) =
  match desc with
    Zvarpat s ->
      [s, path]
  | Zaliaspat(pat,s) ->
      (s, path) :: paths_of_pat path pat
  | Zconstructpat(cstr, patlist) ->
      let rec paths_of_patlist i = function
        [] -> []
      | p::pl ->
          paths_of_pat (Path_son(i,path)) p @ paths_of_patlist (i+1) pl in
      paths_of_patlist 0 patlist
  | Zconstraintpat(pat,_) ->
      paths_of_pat path pat
  | Zrecordpat lbl_pat_list ->
      let rec paths_of_lbl_pat_list = function
        [] -> []
      | (lbl,p)::pl ->
          paths_of_pat (Path_son(lbl.info.lbl_pos,path)) p @
          paths_of_lbl_pat_list pl in
      paths_of_lbl_pat_list lbl_pat_list
  | _ -> []
;;

let add_pat_to_env env pat =
  Tenv(paths_of_pat Path_root pat, env)
;;
let add_pat_list_to_env =
  it_list add_pat_to_env
;;
let add_for_parameter_to_env env id =
  Treserved(Tenv([id, Path_son(0, Path_root)], env))
;;
(* Le parametre d'une boucle "for" est stocke comme une reference, avec
   l'indice de Bruijn 1. L'indice 0 correspond a la valeur de fin de boucle.
*)

let search_env s =
  search_rec 0
  where rec search_rec i = function
    Tnullenv      -> failwith "global"
  | Treserved env -> search_rec (i+1) env
  | Tenv(L,env)   -> try (i, assoc s L)
                     with failure "find" -> search_rec (i+1) env
;;

let translate_path root = transl where rec transl = function
    Path_root          -> root
  | Path_son(n, p)     -> Lprim(Pfield n, [transl p])
;;

let translate_access s env =
  let (i,path) = search_env s env in
    translate_path (Lvar i) path
;;

let translate_update s env newval =
  match search_env s env with
    (i, Path_root) ->
      fatal_error "translate_update"
  | (i, Path_son(start,rest)) ->
      Lprim(Psetfield start, [translate_path (Lvar i) rest; newval])
;;

(* Le traducteur *)

let extract_constant = function
    Lconst cst -> cst
  |       _    -> failwith "not constant"
;;

let partial_fun (Loc(start,stop) as loc) tsb =
  let handler =
    Lprim(Praise,
         [Lconst(SCblock(match_failure_tag,
                         [SCatom(ACstring !input_name);
                          SCatom(ACint start);
                          SCatom(ACint stop)]))]) in
  match tsb with
    True ->
      prerr_location loc;
      prerr_begline "> Warning: pattern matching is not exhaustive";
      prerr_endline2 "";
      handler
  | _ ->
      handler
;;

let partial_try (tsb : tristate_logic) =
  Lprim(Praise, [Lvar 0])
;;

let rec translate_expr env =
  let rec transl (Expr(desc, loc)) =
  match desc with
    Zident(ref(Zlocal s)) ->
      translate_access s env
  | Zident(ref(Zglobal g)) ->
      (match g.info.val_prim with
        ValueNotPrim ->
          Lprim(Pget_global g.qualid, [])
      | ValuePrim(arity, p) ->
          let rec make_fct args n =
            if n >= arity
            then Lprim(p, args)
            else Lfunction(g.qualid.id, make_fct (Lvar n :: args) (n+1))
          in
            make_fct [] 0)
  | Zconstant cst ->
      Lconst cst
  | Zconstruct(c,args) ->
      let tr_args =
        map transl args in
      if c.info.cs_pure then
        try
          Lconst(SCblock(c.info.cs_tag, map extract_constant tr_args))
        with failure "not constant" ->
          Lprim(Pmakeblock(c.info.cs_tag, Unshared), tr_args)
      else
        Lprim(Pmakeblock(c.info.cs_tag, Shared), tr_args)
  | Zapply(Expr(Zfunction ((patl,_)::_ as case_list), _) as funct, args) ->
      if list_length patl == list_length args then
        Llet(translate_let env args,
             translate_match env (partial_fun loc) case_list)
      else
      Lapply(transl funct, map transl args)
  | Zapply((Expr(Zident(ref (Zglobal g)), _) as fct), args) ->
     (match g.info.val_prim with
        ValueNotPrim ->
          Lapply(transl fct, map transl args)
      | ValuePrim(arity, p) ->
          if arity == list_length args
          then Lprim(p, map transl args)
          else Lapply(transl fct, map transl args))
  | Zapply(funct, args) ->
      Lapply(transl funct, map transl args)
  | Zlet(rec_flag, pat_expr_list, body) ->
      let cas = map (fun (x, _, _) -> x) pat_expr_list in
      let tr_body = translate_match env (partial_fun loc) [cas, body] in
        if rec_flag then
          Lletrec(translate_rec_bind
                     (add_pat_list_to_env env cas) pat_expr_list, tr_body)
        else
          Llet(translate_bind env pat_expr_list, tr_body)
  | Zfunction(case_list) ->
      translate_fun env "" loc case_list
  | Ztrywith(body, pat_expr_list) ->
      Lhandle(transl body,
              translate_simple_match env partial_try pat_expr_list)
  | Zsequence(E1, E2) ->
      Lsequence(transl E1, transl E2)
  | Zcondition(Eif, Ethen, Eelse) ->
      Lifthenelse(transl Eif, transl Ethen, transl Eelse)
  | Zwhile(Econd, Ebody) ->
      Lwhile(transl Econd, transl Ebody)
  | Zfor(id, Estart, Estop, up_flag, Ebody) ->
      Lfor(transl Estart,
           translate_expr (Treserved env) Estop,
           up_flag,
           translate_expr (add_for_parameter_to_env env id) Ebody)
  | Zsequand(E1, E2) ->
      Lsequand(transl E1, transl E2)
  | Zsequor(E1, E2) ->
      Lsequor(transl E1, transl E2)
  | Zconstraint(E, _) ->
      transl E
  | Zvector [] ->
      Lconst(SCblock(ConstrRegular(0,0), []))
  | Zvector args ->
      let tr_args = map transl args in
        Lprim(Pmakeblock (ConstrRegular(0,0), Shared), tr_args)
  | Zassign(id, E) ->
      translate_update id env (transl E)
  | Zrecord lbl_expr_list ->
      let v = make_vect (list_length lbl_expr_list) Lstaticfail in
        do_list
          (fun (lbl, e) -> v.(lbl.info.lbl_pos) <- transl e)
          lbl_expr_list;
        if for_all
             (fun (lbl, e) -> lbl.info.lbl_mut == Notmutable)
             lbl_expr_list
        then                          (* aucun champ mutable *)
          begin try
            Lconst(SCblock(ConstrRegular(0,0),
                           map_vect_list extract_constant v))
          with failure "not constant" ->
            Lprim(Pmakeblock(ConstrRegular(0,0), Unshared), list_of_vect v)
          end
        else
          Lprim(Pmakeblock(ConstrRegular(0,0), Shared), list_of_vect v)
  | Zrecord_access (e, lbl) ->
      Lprim(Pfield lbl.info.lbl_pos, [transl e])
  | Zrecord_update (e1, lbl, e2) ->
      Lprim(Psetfield lbl.info.lbl_pos, [transl e1; transl e2])

  in transl

and translate_match env failure_code casel =
  let transl_action (patlist, expr) =
    patlist, (translate_expr (add_pat_list_to_env env patlist) expr, expr) in
  translate_matching failure_code (map transl_action casel)

and translate_simple_match env failure_code pat_expr_list =
  let transl_action (pat, expr) =
    [pat], (translate_expr (add_pat_to_env env pat) expr, expr) in
  translate_matching failure_code (map transl_action pat_expr_list)

and translate_fun env name loc = function
    [] -> fatal_error "translate_fun"
  | (patl,_)::_ as case_list ->
      let rec transl_fun = function
         []  -> translate_match env (partial_fun loc) case_list
      | a::L -> Lfunction(name, transl_fun L)
      in
        transl_fun patl

and translate_let env = function
     [] ->  []
  | a::L -> translate_expr env a :: translate_let (Treserved env) L

and translate_bind env = function
    [] -> []
  | (Pat(Zvarpat name, _), Expr(Zfunction case_list, loc), _) :: rest ->
      translate_fun env name loc case_list
        :: translate_bind (Treserved env) rest
  | (pat, expr, _) :: rest ->
      translate_expr env expr :: translate_bind (Treserved env) rest

and translate_rec_bind env = function
    [] -> []
  | (Pat(Zvarpat name, _), Expr(Zfunction case_list, loc), sz) :: rest ->
      (translate_fun env name loc case_list, !sz)
      :: translate_rec_bind env rest
  | (pat, expr, sz) :: rest ->
      (translate_expr env expr, !sz) :: translate_rec_bind env rest
;;

let translate_expression = translate_expr Tnullenv
;;

let translate_letdef loc pat_expr_list =
  let translate_bind = function
    (Pat(Zvarpat i, _), expr, _) ->       (* Le cas le plus frequent *)
      Lprim(Pset_global {qual=(!defined_module).mod_name; id=i},
            [translate_expression expr])
  | (pat, expr, _) ->                      (* Le cas general *)
      let vars = free_vars_of_pat pat in
      let env = add_pat_to_env Tnullenv pat in
      let store_global var =
        Lprim(Pset_global {qual=(!defined_module).mod_name; id=var},
              [translate_access var env]) in
      let rec store_global_list = function
        [] ->
          Lconst(SCatom(ACint 0))
      | [var] ->
          store_global var
      | var::rest ->
          Lsequence(store_global var, store_global_list rest)
      in
        Llet([translate_expression expr],
             translate_matching (partial_fun loc)
               [[pat], (store_global_list vars,
                        Expr(Zconstant(SCatom(ACint 0)), loc))]) in
  let rec translate_binds = function
    [] ->
      Lconst(SCatom(ACint 0))
  | [bind] ->
      translate_bind bind
  | bind::rest ->
      Lsequence(translate_bind bind, translate_binds rest)
  in
    translate_binds pat_expr_list
;;

let translate_letdef_rec loc pat_expr_list =
  let mn = (!defined_module).mod_name

  in let rec translate_bind = function
      (Pat (Zvarpat i, _), expr, _) -> Lprim (Pupdate_global {qual=mn;id=i},
                                                [translate_expression expr])
    | (Pat (Zconstraintpat (pat, _), _), e, sz) -> translate_bind (pat, e, sz)
    | (Pat (Zaliaspat (pat, _), _), e, sz) -> translate_bind (pat, e, sz)
    | _ -> fatal_error "translate_letdef_rec:translate_bind"

  in let rec translate_binds = function
      [] -> Lconst (SCatom (ACint 0))
    | [bind] -> translate_bind bind
    | bind::rest -> Lsequence (translate_bind bind, translate_binds rest)

  in let rec make_dummy = function
      (Pat (Zvarpat i, _), _, sz)
        -> Lprim (Pdummy_global ({qual=mn; id=i}, !sz), [])
    | (Pat (Zconstraintpat (pat, _), _), e, sz)
        -> make_dummy (pat, e, sz)
    | (Pat (Zaliaspat (pat, v), _), e, sz)
        -> Lsequence (Lprim (Pdummy_global ({qual=mn; id=v}, !sz), []),
                      copy_dummy v pat)
    | _ -> fatal_error "translate_letdef_rec:make_dummy"

  and copy_dummy v = function
      Pat (Zvarpat i, _)
        -> Lprim (Pset_global {qual=mn; id=i},
                  [Lprim (Pget_global {qual=mn; id=v}, [])])
    | Pat (Zconstraintpat (pat, _), _) -> copy_dummy v pat
    | Pat (Zaliaspat (pat, i), _)
        -> Lsequence (Lprim (Pset_global {qual=mn; id=i},
                             [Lprim (Pget_global {qual=mn; id=v}, [])]),
                      copy_dummy v pat)
    | _ -> fatal_error "translate_letdef_rec:copy_dummy"

  in let rec make_dummies = function
      [] -> Lconst (SCatom (ACint 0))
    | [bind] -> make_dummy bind
    | bind::rest -> Lsequence (make_dummy bind, make_dummies rest)

  in
     Lsequence (make_dummies pat_expr_list, translate_binds pat_expr_list)
;;
