{
#open "misc";;
#open "parser";;
#open "hash";;

(* Pour singaler les erreurs lexicales *)

exception Lexical_error of string * int * int
;;

(* Pour les commentaires emboites *)

let comment_depth = ref 0;;

(* La table des mots-cles *)

let keyword_table = make_vect 31 ([] : (string * token) list)
and infix_table = make_vect 17  ([] : string list);;

let add_keyword (s,tok as pair) =
  let i = hash_univ_param 1 1 s mod  31 in
    keyword_table.(i) <- pair :: keyword_table.(i)
;;

do_list add_keyword [
  "and", AND;
  "as", AS;
  "begin", BEGIN;
  "do", DO;
  "done", DONE;
  "down", DOWN;
  "else", ELSE;
  "end", END;
  "exception", EXCEPTION;
  "for", FOR;
  "fun", FUN;
  "function", FUNCTION;
  "if", IF;
  "in", IN;
  "let", LET;
  "match", MATCH;
  "mutable", MUTABLE;
  "not", NOT;
  "of", OF;
  "or", OR;
  "prefix", PREFIX;
  "rec", REC;
  "then", THEN;
  "to", TO;
  "try", TRY;
  "type", TYPE;
  "value", VALUE;
  "where", WHERE;
  "while", WHILE;
  "with", WITH
];;

let find_keyword s =
  match keyword_table.(hash_univ_param 1 1 s  mod  31) with
    [] -> raise (failure "find")
  | (s1,tok1)::rest1 ->
      if s = s1 then tok1 else match rest1 with
        [] -> raise (failure "find")
      | (s2,tok2)::rest2 ->
          if s = s2 then tok2 else assoc s rest2
;;

(* Les infixes *)

let is_infix s =
  match infix_table.(hash_univ_param 1 1 s  mod  17) with
    [] -> false
  | s1::rest1 ->
      s = s1 or match rest1 with
        [] -> false
      | s2::rest2 ->
          s = s2 or mem s rest2
;;

let add_infix s =
  let i = hash_univ_param 1 1 s  mod  17 in
    infix_table.(i) <- s :: infix_table.(i); ()
;;

let remove_infix s =
  let i = hash_univ_param 1 1 s  mod  17 in
    infix_table.(i) <- except s infix_table.(i); ()
;;

do_list add_infix ["quo"; "mod"]
;;

(* Bufferisation des chaines a la lecture *)

let initial_string_buffer = create_string 256;;
let string_buff = ref initial_string_buffer;;
let string_index = ref 0;;

let reset_string_buffer () =
  string_buff := initial_string_buffer;
  string_index := 0;
  ()
;;

let store_string_char c =
  (if !string_index >= string_length (!string_buff) then
    let new_buff = create_string (string_length (!string_buff) * 2) in
      blit_string (!string_buff) 0 new_buff 0 (string_length (!string_buff));
      string_buff := new_buff;
      ());
  set_nth_char (!string_buff) (!string_index) c;
  incr string_index
;;

let get_stored_string () =
  let s = sub_string (!string_buff) 0 (!string_index) in
    string_buff := initial_string_buffer;
    s
;;

(* Les caracteres speciaux *)

let char_for_backslash = function
    `n` -> config__newline_char
  | `r` -> config__return_char
  | `b` -> `\008`
  | `t` -> `\009`
  | c   -> c
;;

let char_for_decimal_code lexbuf i =
  char_of_int(100 * (int_of_char(get_lexeme_char lexbuf i) - 48) +
               10 * (int_of_char(get_lexeme_char lexbuf (i+1)) - 48) +
                    (int_of_char(get_lexeme_char lexbuf (i+2)) - 48))
;;

let saved_string_start = ref 0;;

}

rule Main = parse
    [` ` `\010` `\013` `\009` `\026` `\012`] +
      { Main lexbuf }
  | ([`A`-`Z` `a`-`z`] | `_` [`A`-`Z` `a`-`z` `'` `0`-`9`])
    ( `_` ? [`A`-`Z` `a`-`z` `'` `0`-`9`] ) *
      { let s = get_lexeme lexbuf in
          try
            find_keyword s
          with failure "find" ->
            if is_infix s then INFIX s else IDENT s }
  | (([`0`-`9`]+ (`.` [`0`-`9`]+)? [`e` `E`] (`-`)? [`0`-`9`]+)
          |([`0`-`9`]+ `.` [`0`-`9`]+ ([`e` `E`] (`-`)? [`0`-`9`]+)?))
      { FLOAT (float_of_string(get_lexeme lexbuf)) }
  | [`0`-`9`]+
      { INT (int_of_string(get_lexeme lexbuf)) }
  | `"`
      { reset_string_buffer();
        (* Start of token is start of string. *)
        saved_string_start := lexbuf.lex_start_pos;
        String lexbuf;
        lexbuf.lex_start_pos <- !saved_string_start;
        STRING (get_stored_string()) }
  | "`"
      { CHAR (Char lexbuf) }
  | "(*"
      { comment_depth := 1; Comment lexbuf; Main lexbuf }
  | "#" { SHARP }
  | "!" { BANG }
  | "!=" { BANGEQUAL }
  | "&" { AMPERSAND }
  | "'" { QUOTE }
  | "(" { LPAREN }
  | ")" { RPAREN }
  | "*" { STAR }
  | "+" { PLUS }
  | "," { COMMA }
  | "-" { MINUS }
  | "->" { MINUSGREATER }
  | "." { DOT }
  | ".(" { DOTLPAREN }
  | "/" { SLASH }
  | ":" { COLON }
  | "::" { COLONCOLON }
  | ":=" { COLONEQUAL }
  | ";" { SEMI }
  | ";;" { SEMISEMI }
  | "<" { LESS }
  | "<-" { LESSMINUS }
  | "<=" { LESSEQUAL }
  | "<>" { LESSGREATER }
  | "=" { EQUAL }
  | "==" { EQUALEQUAL }
  | ">" { GREATER }
  | ">=" { GREATEREQUAL }
  | "@" { AT }
  | "[" { LBRACKET }
  | "[|" { LBRACKETBAR }
  | "]" { RBRACKET }
  | "^" { CARET }
  | "_" { UNDERSCORE }
  | "__" { UNDERUNDER }
  | "{" { LBRACE }
  | "|" { BAR }
  | "|]" { BARRBRACKET }
  | "}" { RBRACE }
  | eof { EOF }
  | _
      { raise (Lexical_error("illegal character",
                            get_lexeme_start lexbuf, get_lexeme_end lexbuf)) }

and Comment = parse
    "(*"
      { comment_depth := succ !comment_depth; Comment lexbuf }
  | "*)"
      { comment_depth := pred !comment_depth;
        if !comment_depth > 0 then Comment lexbuf }
  | `"`
      { reset_string_buffer();
        String lexbuf;
        reset_string_buffer();
        Comment lexbuf }
  | eof
      { raise (Lexical_error("comment not terminated", -1, -1)) }
  | _
      { Comment lexbuf }

and Char = parse
    [^ `\\` `\``] "`"
      { get_lexeme_char lexbuf 0 }
  | `\\` [`\\` `\`` `n` `t` `b` `r`] "`"
      { char_for_backslash (get_lexeme_char lexbuf 1) }
  | `\\` [`0`-`9`] [`0`-`9`] [`0`-`9`] "`"
      { char_for_decimal_code lexbuf 1 }
  | _
      { raise (Lexical_error("bad character constant",
                            get_lexeme_start lexbuf, get_lexeme_end lexbuf)) }

and String = parse
    `"`
      { () }
  | `\\` [` ` `\010` `\013` `\009` `\026` `\012`] +
      { String lexbuf }
  | `\\` [`\\` `"` `n` `t` `b` `r`]
      { store_string_char(char_for_backslash(get_lexeme_char lexbuf 1));
        String lexbuf }
  | `\\` [`0`-`9`] [`0`-`9`] [`0`-`9`]
      { store_string_char(char_for_decimal_code lexbuf 1);
         String lexbuf }
  | eof
      { raise (Lexical_error("string not terminated", -1, -1)) }
  | _
      { store_string_char(get_lexeme_char lexbuf 0);
        String lexbuf }
;;
