#open "misc";;
#open "const";;
#open "globals";;
#open "syntax";;
#open "location";;
#open "lambda";;
#open "prim";;

(*  match.ml : expansion du filtrage dans le lambda-calcul.

    Bibliographie : Peyton-Jones, The Implementation of F.P.L, chap. 5    *)


(* Representation d'un pattern matching : une disjonction de conjonctions.

      pat & pat & ... & pat  ->  action
    | pat & pat & ... & pat  ->  action
    | ...
    | pat & pat & ... & pat  ->  action

      exp   exp   ...   exp

  Un motif "pat" s'applique a (i.e. doit filtrer) l'expression qui est
  en-dessous de lui.
*)

type pattern_matching =
  Matching of (pattern list * (lambda * expression)) list * lambda list
;;

(* Quelques manipulations triviales de matchings *)

let make_path start n (path::pathl) =
  let rec make i =
    if i >= n then pathl else Lprim(Pfield i, [path]) :: make (i+1)
  in
    make start
;;

let add_to_match (Matching(casel,pathl)) cas =
  Matching(cas :: casel, pathl)

and make_constant_match (path :: pathl) cas =
  Matching([cas], pathl)

and make_construct_match cstr numargs pathl cas =
  Matching([cas], make_path 0 numargs pathl)
;;

let add_to_division make_match divlist key cas =
  try
    let matchref = assoc key divlist in
      matchref := add_to_match !matchref cas; divlist
    with failure "find" ->
      (key, ref (make_match cas)) :: divlist
;;

(* Factorisation d'un matching dans le cas ou il "commence" (en haut a gauche)
   par un pattern constant *)

let divide_constant_matching (Matching(casel, pathl)) =
  divide_rec casel where rec divide_rec = function
      (Pat(Zaliaspat(pat,v),_) :: patl, action) :: rest ->
        divide_rec ((pat::patl, action) :: rest)
    | (Pat(Zconstraintpat(pat,ty),_) :: patl, action) :: rest ->
        divide_rec ((pat::patl, action) :: rest)
    | (Pat(Zorpat(pat1, pat2),_) :: patl, action) :: rest ->
        divide_rec ((pat1::patl, action) :: (pat2::patl, action) :: rest)
    | (Pat(Zconstantpat(cst),_) :: patl, action) :: rest ->
        let (constant, others) = divide_rec rest in
          add_to_division
            (make_constant_match pathl) constant cst (patl, action),
          others
    | casel ->
        [], Matching(casel, pathl)
;;

(* Factorisation d'un matching dans le cas ou il "commence" (en haut a gauche)
   par un pattern construit *)

let divide_construct_matching (Matching(casel, pathl)) =
  divide_rec casel where rec divide_rec = function
      (Pat(Zaliaspat(pat,v),_) :: patl, action) :: rest ->
        divide_rec ((pat::patl, action) :: rest)
    | (Pat(Zconstraintpat(pat,ty),_) :: patl, action) :: rest ->
        divide_rec ((pat::patl, action) :: rest)
    | (Pat(Zorpat(pat1, pat2),_) :: patl, action) :: rest ->
        divide_rec ((pat1::patl, action) :: (pat2::patl, action) :: rest)
    | (Pat(Zconstructpat(c,args),_) :: patl, action) :: rest ->
        let (constrs, others) = divide_rec rest in
          add_to_division
            (make_construct_match c (list_length args) pathl) constrs
            c.info.cs_tag (args @ patl, action),
          others
    | casel ->
        [], Matching(casel, pathl)
;;

(* Factorisation d'un matching dans le cas ou il "commence" (en haut a gauche)
   par une variable ou par _                                                *)

let divide_var_matching (Matching(casel, (_ :: endpathl as pathl))) =
  divide_rec casel where rec divide_rec = function
      (Pat((Zwildpat | Zvarpat _),_) :: patl, action) :: rest ->
        let vars, others = divide_rec rest in
          add_to_match vars (patl, action),
          others
    | (Pat(Zaliaspat(pat,v),_) :: patl, action) :: rest ->
        divide_rec ((pat::patl, action) :: rest)
    | (Pat(Zconstraintpat(pat,ty),_) :: patl, action) :: rest ->
        divide_rec ((pat::patl, action) :: rest)
    | (Pat(Zorpat(pat1, pat2),_) :: patl, action) :: rest ->
        divide_rec ((pat1::patl, action) :: (pat2::patl, action) :: rest)
    | casel ->
        Matching([], endpathl), Matching(casel, pathl)
;;

(* Factorisation d'un matching dans le cas ou il "commence" (en haut a gauche)
   par un record pattern *)

let divide_record_matching (Matching(casel, pathl)) =
  let max_pos = ref 0 in
  let rec max_size = function
      Pat(Zaliaspat(pat,v),_) -> max_size pat
    | Pat(Zconstraintpat(pat,ty),_) -> max_size pat
    | Pat(Zorpat(pat1,pat2),_) -> max_size pat1; max_size pat2
    | Pat(Zrecordpat pat_expr_list,_) ->
        do_list
          (fun (lbl,p) ->
            if lbl.info.lbl_pos > !max_pos then
              (max_pos := lbl.info.lbl_pos; ()))
          pat_expr_list
    | _ -> () in
  do_list (fun (pat::patl, act) -> max_size pat) casel;
  let rec divide_rec = function
      (Pat(Zaliaspat(pat,v),_) :: patl, action) :: rest ->
        divide_rec ((pat::patl, action) :: rest)
    | (Pat(Zconstraintpat(pat,ty),_) :: patl, action) :: rest ->
        divide_rec ((pat::patl, action) :: rest)
    | (Pat(Zorpat(pat1, pat2),_) :: patl, action) :: rest ->
        divide_rec ((pat1::patl, action) :: (pat2::patl, action) :: rest)
    | (Pat(Zrecordpat pat_expr_list,_) :: patl, action) :: rest ->
        divide_rec_cont pat_expr_list patl action rest
    | (Pat((Zwildpat | Zvarpat _),_) :: patl, action) :: rest ->
        divide_rec_cont [] patl action rest
    | [] ->
        Matching([], make_path 0 (succ !max_pos) pathl)
    | _ ->
        fatal_error "divide_record_matching"
  and divide_rec_cont pat_expr_list patl action rest =
    let v = make_vect (succ !max_pos) (Pat(Zwildpat, Loc(0,0))) in
      do_list (fun (lbl, pat) -> v.(lbl.info.lbl_pos) <- pat) pat_expr_list;
      add_to_match (divide_rec rest) (list_of_vect v @ patl, action)
  in
    divide_rec casel
;;

(* Utilitaires quelconques. *)

let length_of_matching (Matching(casel,_)) = list_length casel
;;

let upper_left_pattern =
  let rec strip = function
      Pat(Zaliaspat(pat,_),_) -> strip pat
    | Pat(Zconstraintpat(pat,_),_) -> strip pat
    | Pat(Zorpat(pat1,pat2),_) -> strip pat1
    | pat -> pat in
  function Matching((pat::_, _) :: _, _) -> strip pat
      |                _                 -> fatal_error "upper_left_pattern"
;;

let get_span_of_matching matching =
  match upper_left_pattern matching with
      Pat(Zconstructpat(c,_), _) -> 
        (match c.info.cs_tag with
            ConstrExtensible _      -> 0       (* Valeur impossible... *)
          | ConstrRegular(tag,span) -> span)
    | _ -> fatal_error "get_span_of_matching"
;;

(* La logique trois etats. *)

let tristate_or = function
    (True, _)     -> True
  | (_, True)     -> True
  | (False,False) -> False
  |      _        -> Maybe
;;

(* matching -> lambda, drapeau "partiel", liste des cas atteints *)

let rec conquer_matching =
  let rec conquer_divided_matching = function
    [] ->
      [], False, []
  | (key, matchref) :: rest ->
      let lambda1, partial1, used1 = conquer_matching !matchref
      and list2,   partial2, used2 = conquer_divided_matching rest in
        (key, lambda1) :: list2,
        tristate_or(partial1,partial2),
        used1 @ used2
  in function
    Matching([], _) ->
      Lstaticfail, True, []
  | Matching(([], (compiled_action, Expr(_,loc_action))) :: rest, _) ->
      compiled_action, False, [loc_action]
  | Matching(_, (path :: _)) as matching ->
     (match upper_left_pattern matching with
        Pat((Zwildpat | Zvarpat _), _) ->
          let vars, rest = divide_var_matching matching in
          let lambda1, partial1, used1 = conquer_matching vars
          and lambda2, partial2, used2 = conquer_matching rest in
            if partial1 == False then
              lambda1, False, used1
            else
	      Lstatichandle(lambda1, lambda2),
              (if partial2 == False then False else Maybe),
              used1 @ used2
      | Pat(Zconstructpat(_,_),_) ->
          let constrs, vars = divide_construct_matching matching in
          let switchlst, partial1, used1 = conquer_divided_matching constrs
          and lambda,    partial2, used2 = conquer_matching vars in
          let span = get_span_of_matching matching
          and num_cstr = list_length constrs in
            if num_cstr == span & partial1 == False then
              Lswitch(span, path, switchlst), False, used1
            else
              Lstatichandle(Lswitch(span, path, switchlst), lambda),
              (if partial2 == False then False
               else if num_cstr < span then True
               else Maybe),
              used1 @ used2
      | Pat(Zconstantpat _,_) ->
          let constants, vars = divide_constant_matching matching in
            let condlist1, _, used1 = conquer_divided_matching constants
            and lambda2, partial2, used2 = conquer_matching vars in
              Lstatichandle(Lcond(path, condlist1), lambda2),
              partial2,
              used1 @ used2
      | Pat(Zrecordpat _,_) ->
          conquer_matching (divide_record_matching matching)
      | _ ->
          fatal_error "conquer_matching 2")
  | _ -> fatal_error "conquer_matching 1"
;;

let make_initial_matching = function
    [] ->
      fatal_error "make_initial_matching: empty"
  | (patl, _) :: _ as casel ->
      let rec make_path n =
        if n <= 0 then [] else Lvar(n-1) :: make_path(n-1)
      in
        Matching(casel, make_path(list_length patl))
;;

let find_unused_cases (used, casel) =
  do_list
    (function (_, (_, Expr(_,loc))) ->
      if mem loc used then () else begin
        prerr_location loc;
        prerr_begline "> Unused case in matching.";
	prerr_endline2 ""
      end)
    casel        
;;

(* The point d'entree *)

let translate_matching failure_code casel =
  let (lambda, partial, used) =
    conquer_matching (make_initial_matching casel)
  in
    find_unused_cases (used, casel);    
    match partial with
        False -> lambda
      |   _   -> Lstatichandle(lambda, failure_code partial)
;;
