#open "misc";;
#open "const";;
#open "globals";;
#open "hashtbl";;
#open "errors";;

(*  modules.ml : les modules et les tables de symboles globaux  *)

let name_of_module    md = md.mod_name
and values_of_module  md = md.mod_values
and constrs_of_module md = md.mod_constrs
and labels_of_module  md = md.mod_labels
and types_of_module   md = md.mod_types
;;

let add_to_symtable = add_to_hashtable little_name_of_global
and remove_from_symtable = remove_from_hashtable little_name_of_global
and find_in_symtable = find_in_hashtable little_name_of_global
;;

(* La table des interfaces deja chargees *)

let module_table = (new_hashtable 37 : (string * module) hashtable);;

let new_module nm =
  let md =
    { mod_name = nm;
      mod_values = new_hashtable 17;
      mod_constrs = new_hashtable 13;
      mod_labels = new_hashtable 11;
      mod_types = new_hashtable 7 }
  in
    add_to_assoctable module_table (nm,md); md
;;

let path_interfaces = ref [current_dir_name]
;;

(* Charger une interface depuis un fichier *)

let load_module name =
  let truename = find_in_path (name ^ ".zi") !path_interfaces in
  let ic = open_in truename in
  try
    let md = (input_value ic : module) in
      close_in ic;
      md
  with End_of_file ->
    close_in ic;
    prerr_begline ">> Corrupted compiled interface file ";
    prerr_endline truename;
    raise Toplevel
;;

(* Trouver une interface par son nom *)

let find_module name =
  try
    find_in_assoctable module_table name
  with failure "find" ->
    let md = load_module name in
      add_to_assoctable module_table (name, md); md
;;

(* Oublier une interface *)

let kill_module (* name *) =
  remove_from_assoctable module_table (* name *)
;;

(* L'etat courant du compilo *)

let default_used_modules = ref ([] : string list);;

let defined_module = ref (new_module "")
and used_modules = ref ([] : module list)
and type_stamp = ref 0
;;

let new_globalenv name =
  defined_module := new_module name;
  used_modules := map find_module !default_used_modules;
  type_stamp := 0;
  ()
;;

let compiled_module_name() = (!defined_module).mod_name
;;

let defined_global name desc =
  { qualid = { qual=compiled_module_name(); id=name }; info = desc }
;;

let new_type_stamp () =
  incr type_stamp
;;

(* Pour pouvoir annuler des ajouts en cas d'interruptions ou d'erreurs *)

let rollback_buffer = ref ([] : (unit -> unit) list)
;;
let reset_rollback () =
  rollback_buffer := []; ()
;;
let rec rollback () =
  match !rollback_buffer with
    [] -> ()
  | f::rest -> f (); rollback_buffer := rest; rollback()
;;

(* Ajouts dans le module en cours de compilation *)

let add_global_info sel_fct data =
  let tbl = sel_fct !defined_module in
    rollback_buffer :=
      (fun () -> remove_from_symtable tbl data) :: !rollback_buffer;
    add_to_symtable tbl data
;;

let add_value = add_global_info values_of_module
and add_constr = add_global_info constrs_of_module
and add_label = add_global_info labels_of_module
and add_type = add_global_info types_of_module
;;

(* Recherches d'une reference globale.
   Si de la forme mod__name, on cherche juste dans le module mod (s'il est
   ouvert). Sinon, on cherche dans le module courant, puis dans tous les
   modules utilises, en commencant par le plus recent *)

let find_desc sel_fct = function
    GRmodname q ->
     (try
        find_in_symtable (sel_fct (find_module q.qual)) q.id
      with failure "find" ->
        failwith "find_desc")
  | GRname s ->
      try
        find_in_symtable (sel_fct !defined_module) s
      with failure "find" ->
        let rec find_rec = function
          []       -> failwith "find_desc"
        | md::rest -> try find_in_symtable (sel_fct md) s
                      with failure _ -> find_rec rest
        in find_rec !used_modules
;;

let find_value_desc = find_desc values_of_module
and find_constr_desc = find_desc constrs_of_module
and find_label_desc = find_desc labels_of_module
and find_type_desc = find_desc types_of_module
;;

(* Sauvegarde (export) de l' interface du module en cours de definition *)

let write_compiled_interface oc =
  output_value oc !defined_module
;;
