#open "misc";;
#open "const";;
#open "globals";;
#open "syntax";;
#open "builtins";;
#open "modules";;
#open "types";;
#open "errors";;
#open "ty_error";;

(* typing.ml : l'inference de types *)

(* Des expressions de type aux types *)

let reset_type_expression_vars, type_of_type_expression =
  let vars = ref ([] : (string * typ) list) in
    (* reset_type_expression_vars *)
    (fun var_list ->
      vars := [];
      map
        (fun v ->
          if mem_assoc v !vars then
            failwith "reset_type_expression_vars"
          else
           (let t = new_type_var() in
              vars := (v, t) :: !vars; t))
        var_list),
    (* type_of_type_expression *)
    (fun strict_flag typexp ->
      type_of typexp
      where rec type_of = function
        Typexp(Ztypevar v, _) ->
          (try
            assoc v !vars
           with failure "find" ->
            if strict_flag then
              unbound_type_var_err v typexp
            else
             (let t = new_type_var() in
                vars := (v,t) :: !vars; t))
      | Typexp(Ztypearrow(arg1, arg2), loc) ->
          type_arrow(type_of arg1, type_of arg2)
      | Typexp(Ztypeconstr(cstr_name, args), loc) as texp ->
          let cstr =
            try
              find_type_desc cstr_name
            with failure "find_desc" ->
              unbound_err "Type constructor" cstr_name loc
          in
            if not same_type_constr cstr constr_type_tuple
             & list_length args != cstr.info.ty_arity
            then type_arity_err cstr args loc;
            match cstr.info.ty_desc with
              Abbrev_type(ty_params,ty_body) ->
                let (ty_params', ty_body') =
                  type_abbrev_instance (ty_params, ty_body)
                in
                  do_list2
                    (fun typexp ty -> unify (type_of typexp, ty))
                    args
                    ty_params';
                  ty_body'
            | _ ->
                Typ(Tconstr(cstr, map type_of args), notgeneric))
;;

(* Typage des constantes *)

let type_of_atomic_constant = function
    ACint _ -> type_int
  | ACfloat _ -> type_float
  | ACstring _ -> type_string
  | ACchar _ -> type_char
;;
let rec type_of_structured_constant = function
    SCatom ac ->
      type_of_atomic_constant ac
  | SCblock(cstr, args) ->
      fatal_error "type_of_structured_constant" (* a faire, un jour *)
;;

(* Typage des patterns *)

let (type_pattern, type_pattern_list, type_pattern_list2) =
  let rec tpat new_env ((Pat(desc, loc) as pat), ty, mut_flag) =
    match desc with
      Zwildpat ->
        new_env
    | Zvarpat v ->
        if mem_assoc v new_env then
          non_linear_pattern_err pat v
        else
          (v, (ty, mut_flag)) :: new_env
    | Zaliaspat(pat, v) ->
        if mem_assoc v new_env then
          non_linear_pattern_err pat v
        else
          tpat ((v, (ty, mut_flag)) :: new_env) (pat, ty, mut_flag)
    | Zconstantpat cst ->
        let ty' = type_of_atomic_constant cst in
         (try
            unify (ty, ty'); new_env
          with Unify ->
            pat_wrong_type_err pat ty' ty)
    | Zconstructpat(cstr, args) ->
        if list_length args == list_length cstr.info.cs_args then
          let (ty_res, ty_args) = type_constr_instance cstr in
            try
              unify(ty, ty_res);
              tpat_list new_env (args, ty_args)
            with Unify ->
              pat_wrong_type_err pat ty_res ty
        else
          constr_wrong_arity_err cstr (list_length args) loc
    | Zorpat(pat1, pat2) ->
       (match free_vars_of_pat pat with
          [] -> tpat (tpat new_env (pat1, ty, mut_flag)) (pat2, ty, mut_flag)
        | _  -> orpat_should_be_closed_err pat)
    | Zconstraintpat(pat, ty_expr) ->
        let ty' = type_of_type_expression false ty_expr in
        let new_env' = tpat new_env (pat, ty', mut_flag) in
         (try
            unify (ty, ty'); new_env'
          with Unify ->
            pat_wrong_type_err pat ty' ty)
    | Zrecordpat lbl_pat_list ->
        let rec tpat_lbl new_env = function
          [] -> new_env
        | (lbl,p) :: rest ->
            let (ty_res, ty_arg) = type_label_instance lbl in
              begin try
                unify (ty, ty_res)
              with Unify ->
                pat_wrong_type_err pat ty_res ty
              end;
              tpat_lbl (tpat new_env (p, ty_arg, lbl.info.lbl_mut)) rest
        in
          tpat_lbl new_env lbl_pat_list

  and tpat_list new_env = function
      [], [] -> new_env
    | pat::patl, (ty, mut_flag)::tyl ->
        tpat_list (tpat new_env (pat, ty, mut_flag)) (patl, tyl)
    | _ ->
        fatal_error "type_pattern: arity error"

  in (tpat [], tpat_list [], it_list tpat [])
;;

(* Taille maximum d'un objet de type donne. *)

let rec constr_list_max_size
  = function ([], n) -> n
           | ({info={cs_args=l; _}; _}::rest, n)
             -> let n1 = list_length l
                in constr_list_max_size (rest, if n > n1 then n else n1)
;;

let rec ty_max_size ty loc
  = match (type_repr ty)
    with Typ (Tvar _, _) -> rec_unknown_size_err ty loc
       | Typ (Tarrow (_, _), _) -> 2
       | Typ (Tconstr ({info={ty_desc=desc2; _}; _}, _), _)
         -> match desc2
            with Abstract_type -> rec_unknown_size_err ty loc
               | Variant_type l -> constr_list_max_size (l, 0)
               | Record_type l -> list_length l
               | Abbrev_type (_, _) -> fatal_error "ty_max_size"
;;

(* Typage des expressions *)

let rec type_expr env =
  let rec texp ((Expr(desc, loc) as expr), ty) =
    match desc with
      Zident r ->
        let ty' =
          match !r with
            Zglobal glob_desc ->
              type_instance glob_desc.info.val_typ
          | Zlocal s ->
              try
                let (ty_schema, mut_flag) = assoc s env in
                  type_instance ty_schema
              with failure "find" ->
                try
                  let glob_desc = find_value_desc(GRname s) in
                    r := Zglobal glob_desc;
                    type_instance glob_desc.info.val_typ
                with failure "find_desc" ->
                  unbound_err "Variable" (GRname s) loc
        in
         (try unify(ty, ty') with Unify -> expr_wrong_type_err expr ty' ty)
    | Zconstant cst ->
        let ty' = type_of_structured_constant cst in
          (try unify(ty, ty') with Unify -> expr_wrong_type_err expr ty' ty)
    | Zconstruct(cstr, args) ->
        if list_length args == list_length cstr.info.cs_args then
          let (ty_res, ty_args) = type_constr_instance cstr in
            try
              unify(ty, ty_res);
              texp_list_constr(args, ty_args)
            with Unify ->
              expr_wrong_type_err expr ty_res ty
        else
          constr_wrong_arity_err cstr (list_length args) loc
    | Zapply(fct, args) ->
        let rec make_ty_fct = function
          [] -> ([], ty)
        | arg1::argl ->
            let alpha = new_type_var()
            and (ty_args, ty_fct) = make_ty_fct argl
            in (alpha::ty_args, type_arrow(alpha, ty_fct)) in
        let (ty_args, ty_fct) = make_ty_fct args in
          texp(fct, ty_fct);
          texp_list(args, ty_args)
    | Zlet(rec_flag, pat_expr_list, body) ->
        type_expr (type_let_decl env rec_flag pat_expr_list) (body, ty)
    | Zfunction [] ->
        fatal_error "type_expr: empty matching"
    | Zfunction ((casel1,expr1)::_ as matching) ->
        let rec find_types ty = function
          [] -> ([],ty)
        | pat::rest ->
            try
              let (ty1,ty2) = filter_arrow ty in
              let (ty_args, ty_res) = find_types ty2 rest in
                ((ty1, Notmutable) :: ty_args, ty_res)
            with Unify ->
              expr_wrong_type_err expr
                 (type_arrow(new_type_var(), new_type_var())) ty in
        let (ty_args, ty_res) =
          find_types ty casel1 in
        let tcase (patl, expr) =
          if list_length patl != list_length ty_args then
            ill_shaped_match_err expr;
          type_expr (type_pattern_list (patl, ty_args) @ env) (expr, ty_res)
        in
          do_list tcase matching
    | Ztrywith (body, matching) ->
        texp (body, ty);
        do_list
          (fun (pat, expr) ->
            type_expr (type_pattern (pat, type_exn, Notmutable) @ env)
                      (expr, ty))
          matching
    | Zsequence (e1, e2) ->
        texp (e1, new_type_var()); texp(e2, ty)
    | Zcondition (cond, ifso, ifnot) ->
        texp(cond, type_bool);
        texp(ifnot, ty);
        texp(ifso, ty)
    | Zwhile (cond, body) ->
        texp(cond, type_bool);
        texp(body, new_type_var());
        (try
           unify(ty, type_unit)
         with Unify ->
           expr_wrong_type_err expr type_unit ty)
    | Zfor (id, start, stop, up_flag, body) ->
        texp(start, type_int);
        texp(stop, type_int);
        type_expr ((id,(type_int,Notmutable)) :: env) (body, new_type_var());
        (try
           unify(ty, type_unit)
         with Unify ->
           expr_wrong_type_err expr type_unit ty)
    | Zsequand (e1, e2) ->
        texp(e1, type_bool);
        texp(e2, type_bool);
        (try
           unify(ty, type_bool)
         with Unify ->
           expr_wrong_type_err expr type_bool ty)
    | Zsequor (e1, e2) ->
        texp(e1, type_bool);
        texp(e2, type_bool);
        (try
           unify(ty, type_bool)
         with Unify ->
           expr_wrong_type_err expr type_bool ty)
    | Zconstraint (e, ty_expr) ->
        let ty' = type_of_type_expression false ty_expr in
          texp(e, ty');
          (try
             unify (ty, ty')
           with Unify ->
             expr_wrong_type_err expr ty' ty)
    | Zvector elist ->
        let ty_arg = new_type_var() in
        let ty_vect = type_vect ty_arg in
        (try
           unify (ty, ty_vect);
           do_list (fun e -> texp(e, ty_arg)) elist
         with Unify ->
           expr_wrong_type_err expr ty_vect ty)
    | Zassign(id, e) ->
       (try
          match assoc id env with
            (ty_schema, Notmutable) ->
              not_mutable_err id loc
          | (ty_schema, Mutable) ->
              let ty' = type_instance ty_schema in
                try
                  unify (ty, ty');
                  texp (e, ty)
                with Unify ->
                  expr_wrong_type_err expr ty' ty
        with failure "find" ->
          unbound_err "Variable" (GRname id) loc)
    | Zrecord lbl_expr_list ->
        do_list (fun (lbl, e) ->
          let (ty_res, ty_arg) = type_label_instance lbl in
            begin try
              unify (ty, ty_res)
            with Unify ->
              expr_wrong_type_err expr ty_res ty
            end;
            texp (e, ty_arg))
          lbl_expr_list;
        let label_list = labels_of_type ty in
        let v = make_vect (list_length label_list) false in
          do_list (fun (lbl, e) ->
            let p = lbl.info.lbl_pos in
              if v.(p)
              then label_err " is multiply defined." expr lbl
              else v.(p) <- true)
            lbl_expr_list;
          let rec check_labels i = function
            [] -> ()
          | lbl::rest ->
              if v.(i)
              then check_labels (succ i) rest
              else label_err " is undefined." expr lbl
          in check_labels 0 label_list
    | Zrecord_access (e, lbl) ->
        let (ty_res, ty_arg) = type_label_instance lbl in
          begin try
            unify (ty, ty_arg)
          with Unify ->
            expr_wrong_type_err expr ty_arg ty
          end;
          texp(e, ty_res)
    | Zrecord_update (e1, lbl, e2) ->
        let (ty_res, ty_arg) = type_label_instance lbl in
          begin try
            unify (ty, ty_arg)
          with Unify ->
            expr_wrong_type_err expr ty_arg ty
          end;
          if lbl.info.lbl_mut == Notmutable then
            label_err " is not mutable." expr lbl;
          texp(e1, ty_res);
          texp(e2, ty_arg)

  and texp_list = function
    [], [] -> ()
  | e::el, ty::tyl ->
      texp(e,ty); texp_list(el,tyl)
  | _ ->
      fatal_error "texp_list"

  and texp_list_constr = function
    [], [] -> ()
  | e::el, (ty,mut_flag)::tyl ->
      texp(e,ty); texp_list_constr(el,tyl)
  | _ ->
      fatal_error "texp_list_constr"

  in texp

(* Typage des definitions "let" *)

and type_let_decl env rec_flag pat_expr_list =
  push_type_level();
  let (pat_ty_list, expr_ty_list) =
    list_it
     (fun (pat,expr,_) (pt, et) ->
        let alpha = new_type_var () in
          (pat,alpha,Notmutable)::pt, (expr,alpha)::et)
     pat_expr_list
     ([],[]) in
  let add_env =
    type_pattern_list2 pat_ty_list in
  let new_env =
    add_env @ env
  in
    do_list
      (type_expr (if rec_flag then new_env else env))
      expr_ty_list;
    if rec_flag
    then do_list2 (fun (e, ty) (p, (Expr (d, loc)), size)
                              -> size := ty_max_size ty loc)
                         expr_ty_list pat_expr_list;
    pop_type_level();
    do_list (fun (_, (ty, _)) -> generalize_type ty) add_env;
   new_env
;;
