#open "misc";;
#open "const";;
#open "globals";;
#open "builtins";;
#open "modules";;

(* types.ml : operations de base sur les types *)

(* Egalite des constructeurs de types *)

let same_type_constr cstr1 cstr2 =
  cstr1.info.ty_stamp == cstr2.info.ty_stamp &
  cstr1.qualid.qual = cstr2.qualid.qual
;;

(* Le representant canonique d'un type *)

let rec type_repr = function
    Typ(Tvar Tnolink, _) as t -> t
  | Typ(Tvar (Tlinkto t1 as r), _) ->
      let t2 = type_repr t1 in
        r <- Tlinkto t2; t2
  | t -> t
;;

(* Le niveau courant de let *)

let current_level = ref 0;;

let reset_type_var () =
  current_level := 0; ()
and push_type_level () =
  incr current_level; ()
and pop_type_level () =
  decr current_level; ()
;;

(* Produire des variables de types *)

let new_type_var () =
  Typ(Tvar Tnolink, !current_level)
;;

(* Calculer les variables libres (non generiques)
   et les variables dangereuses d'un type *)

let rec free_type_vars ty =
  match type_repr ty with
    Typ(Tvar _, level) as v ->
      if level == generic then [] else [v]
  | Typ(Tarrow(t1,t2), _) ->
      free_type_vars t1 @ free_type_vars t2
  | Typ(Tconstr(c, ty_list), _) ->
      flat_map free_type_vars ty_list
;;

let rec dangerous_vars ty =
  match type_repr ty with
    Typ(Tvar _, _) as v -> []
  | Typ(Tarrow(t1,t2), _) -> []
  | Typ(Tconstr(c, ty_list), _) as t ->
      if c.info.ty_dang
      then free_type_vars t
      else flat_map dangerous_vars ty_list
;;

(* Generaliser un type *)

type type_position = Regular | Protected | Dangerous
;;

let (generalize_type, generalize_type_constr) =
  let rec gen_type pos ty =
    match type_repr ty with
      Typ(Tvar _, level) ->
        if level > !current_level then
          level <- (if pos == Dangerous then !current_level else generic)
        else
          level
    | Typ(Tarrow(t1,t2), level) ->
        let pos' = if pos == Dangerous then Dangerous else Protected in
        let lvl1 = gen_type pos' t1
        and lvl2 = gen_type pos' t2 in
          level <- (if lvl1 < lvl2 then lvl1 else lvl2)
    | Typ(Tconstr(c, ty_list), level) ->
        level <- gen_type_list
                   (if pos != Regular then pos
                    else if c.info.ty_dang then Dangerous else Regular)
                   ty_list
  and gen_type_list pos = function
      [] ->
        notgeneric
    | ty::rest ->
        let level_ty = gen_type pos ty
        and level_rest = gen_type_list pos rest in
          if level_ty < level_rest then level_ty else level_rest
  in
    (gen_type Regular, gen_type Protected)
;;

(* Prendre une instance d'un type *)

(* La ruse: on profite du fait que une variable generique a toujours le
   champ link vide (Tnolink). On s'en sert donc pour mettre un pointeur
   vers la nouvelle variable qui est la copie de la variable. *)

let rec copy_type = function
    Typ(Tvar(Tnolink as link), level) as ty ->
      if level == generic
      then begin let v = new_type_var() in link <- Tlinkto v; v end
      else ty
  | Typ(Tvar(Tlinkto ty), level) ->
      if level == generic
      then ty
      else copy_type ty
  | Typ(Tarrow(t1,t2), level) as ty ->
      if level == generic
      then Typ(Tarrow(copy_type t1, copy_type t2), notgeneric)
      else ty
  | Typ(Tconstr(cstr, ty_list), level) as ty ->
      if level == generic
      then Typ(Tconstr(cstr, map copy_type ty_list), notgeneric)
      else ty
;;

(* Bien sur, quand on a fini de copier, il faut remettre les champs link
   des variables generiques a Tnolink. *)

let rec cleanup_type = function
    Typ(Tvar Tnolink, level) ->
      ()
  | Typ(Tvar(Tlinkto ty as link), level) ->
      if level == generic
      then begin link <- Tnolink; () end
      else cleanup_type ty
  | Typ(Tarrow(t1,t2), level) ->
      if level == generic
      then (cleanup_type t1; cleanup_type t2)
      else ()
  | Typ(Tconstr(_, ty_list), level) ->
      if level == generic
      then do_list cleanup_type ty_list
      else ()
;;

let type_instance ty =
  let ty' = copy_type ty in
    cleanup_type ty;
    ty'

and type_constr_instance cstr =
  let ty_res = cstr.info.cs_res
  and ty_args = cstr.info.cs_args in
  let ty_res' = copy_type ty_res
  and ty_args' = map (fun (ty, mut_flag) -> (copy_type ty, mut_flag)) ty_args
  in
    cleanup_type ty_res;
    do_list (fun (ty, mut_flag) -> cleanup_type ty) ty_args;
    (ty_res', ty_args')

and type_label_instance lbl =
  let ty_res = lbl.info.lbl_res
  and ty_arg = lbl.info.lbl_arg in
  let ty_res' = copy_type ty_res
  and ty_arg' = copy_type ty_arg in
    cleanup_type ty_res;
    cleanup_type ty_arg;
    (ty_res', ty_arg')

and type_abbrev_instance (ty_params, ty_body) =
  let ty_params' = map copy_type ty_params
  and ty_body'   = copy_type ty_body in
    do_list cleanup_type ty_params;
    cleanup_type ty_body;
    (ty_params', ty_body')
;;

(* L'occur check *)

exception Unify;;

let occur_check level0 v =
  occurs_rec where rec occurs_rec ty =
    match type_repr ty with
      Typ(Tvar _, level) as ty' ->
        if ty' == v then raise Unify
        else if level > level0 then (level <- level0; ())
        else ()
    | Typ(Tarrow(t1,t2), _) ->
        occurs_rec t1;
        occurs_rec t2
    | Typ(Tconstr(_, ty_list), _) ->
        do_list occurs_rec ty_list
;;

(* L'unification *)

let rec unify (ty1, ty2) =
  if ty1 == ty2 then () else begin
    let ty1' = type_repr ty1
    and ty2' = type_repr ty2 in
      if ty1' == ty2' then () else begin
        match (ty1', ty2') with
          Typ(Tvar link1, level1), Typ(Tvar link2, level2) ->
            if level1 < level2
            then (level2 <- level1; link2 <- Tlinkto ty1'; ())
            else (level1 <- level2; link1 <- Tlinkto ty2'; ())
        | Typ(Tvar link1, level1), _ ->
            occur_check level1 ty1' ty2';
            link1 <- Tlinkto ty2';
            ()
        | _, Typ(Tvar link2, level2) ->
            occur_check level2 ty2' ty1';
            link2 <- Tlinkto ty1';
            ()
        | Typ(Tarrow(t1, t1'), _), Typ(Tarrow(t2, t2'), _) ->
            unify (t1, t2);
            unify (t1', t2')
        | Typ(Tconstr(cstr1, tyl1), _), Typ(Tconstr(cstr2, tyl2), _) ->
            if cstr1.info.ty_stamp == cstr2.info.ty_stamp &
               cstr1.qualid.qual = cstr2.qualid.qual
            then unify_list (tyl1, tyl2)
            else raise Unify
        | _ ->
            raise Unify
      end
  end

and unify_list = function
    [], [] -> ()
  | ty1::rest1, ty2::rest2 ->
      unify(ty1,ty2); unify_list(rest1,rest2)
  | _ ->
      raise Unify
;;

(* Un cas particulier d'unification *)

let filter_arrow ty =
  match type_repr ty with
    Typ(Tvar link, level) ->
      let ty1 = Typ(Tvar Tnolink, level)
      and ty2 = Typ(Tvar Tnolink, level) in
        link <- Tlinkto(Typ(Tarrow(ty1, ty2), notgeneric));
        (ty1, ty2)
  | Typ(Tarrow(ty1, ty2), _) ->
      (ty1, ty2)
  | _ ->
      raise Unify
;;

(* Le filtrage. Instancie ty1 pour qu'il soit egal a ty2.
   Ne modifie pas ty2. Ne pas generaliser ty1 par la suite! *)

let rec filter (ty1, ty2) =
  if ty1 == ty2 then () else begin
    let ty1' = type_repr ty1
    and ty2' = type_repr ty2 in
      if ty1' == ty2' then () else begin
        match (ty1', ty2') with
          Typ(Tvar link1, level1), Typ(Tvar link2, level2) ->
            link1 <- Tlinkto ty2';
            ()
        | Typ(Tvar link1, level1), _ ->
            occur_check level1 ty1' ty2';
            link1 <- Tlinkto ty2';
            ()
        | Typ(Tarrow(t1, t1'), _), Typ(Tarrow(t2, t2'), _) ->
            filter (t1, t2);
            filter (t1', t2')
        | Typ(Tconstr(cstr1, ty_list1), _), Typ(Tconstr(cstr2, ty_list2), _) ->
            if cstr1.qualid = cstr2.qualid
            then filter_list (ty_list1, ty_list2)
            else raise Unify
        | _ ->
            raise Unify
      end
  end

and filter_list = function
    [], [] -> ()
  | ty1::rest1, ty2::rest2 ->
      filter(ty1,ty2); filter_list(rest1,rest2)
  | _ ->
      raise Unify
;;

(* Liste des labels d'un type produit *)

let labels_of_type ty =
  match type_repr ty with
    Typ(Tconstr(cstr, _), _) ->
      begin match cstr.info.ty_desc with
        Record_type lbl_list -> lbl_list
      | _ -> fatal_error "labels_of_type"
      end
  | _ -> fatal_error "labels_of_type"
;;

