#open "prelude";;
#open "terms";;
#open "equation";;

(****************** Critical pairs *********************)

(* All (u,sig) such that N/u (&var) unifies with M,
   with principal unifier sig *)
(* super : term -> term -> (num list & subst) list *)
let super M = suprec where rec suprec = function
    Term(_,sons) as N ->
      let collate (pairs,n) son =
       (pairs @ map (fun (u,sig) -> (n::u,sig)) (suprec son), n+1) in
      let insides = fst (it_list collate ([],1) sons) in
       (try
          ([], unify(M,N)) :: insides
        with failure _ -> insides)
  | _ -> []
;;

(* Ex :
let (M,_) = <<F(A,B)>> 
and (N,_) = <<H(F(A,x),F(x,y))>> in super M N;;
==> [[1],[2,Term ("B",[])];                      x <- B
     [2],[2,Term ("A",[]); 1,Term ("B",[])]]     x <- A  y <- B
*)

(* All (u,sig), u&[], such that N/u unifies with M *)
(* super_strict : term -> term -> (num list & subst) list *)
let super_strict M = function
      Term(_,sons) ->
        let collate (pairs,n) son =
          (pairs @ map (fun (u,sig) -> (n::u,sig)) (super M son), n+1) in
        fst (it_list collate ([],1) sons)
    | _ -> []
;;

(* Critical pairs of L1=R1 with L2=R2 *)
(* critical_pairs : term_pair -> term_pair -> term_pair list *)
let critical_pairs (L1,R1) (L2,R2) =
  let mk_pair (u,sig) =
     substitute sig (replace L2 u R1), substitute sig R2 in
  map mk_pair (super L1 L2);;

(* Strict critical pairs of L1=R1 with L2=R2 *)
(* strict_critical_pairs : term_pair -> term_pair -> term_pair list *)
let strict_critical_pairs (L1,R1) (L2,R2) =
  let mk_pair (u,sig) =
    substitute sig (replace L2 u R1), substitute sig R2 in
  map mk_pair (super_strict L1 L2)
;;

(* All critical pairs of eq1 with eq2 *)
let mutual_critical_pairs eq1 eq2 =
  (strict_critical_pairs eq1 eq2) @ (critical_pairs eq2 eq1);;

(* Renaming of variables *)

let rename n (t1,t2) =
  let rec ren_rec = function
    Var k -> Var(k+n)
  | Term(op,sons) -> Term(op, map ren_rec sons) in
  (ren_rec t1, ren_rec t2)
;;

(************************ Completion ******************************)

let deletion_message k =
  print_string "Rule ";print_int k; message " deleted"
;;

(* Generate failure message *)
let non_orientable (M,N) =
    pretty_term M; print_string " = "; pretty_term N; print_newline()
;;

let rule_set = let x = (0, ([] : (int * (int * (term * term))) list))
               in ref x
;;
let confirmed_rules =
    (msgbox__new () : (int * (int * (int * (term * term)))) msgbox__t)
;;
let new_rules = (msgbox__new () : (int * int * term * term) msgbox__t);;
let reduced_eqs = (msgbox__new () : (int * int * term * term) msgbox__t);;
let numbered_eqs = (msgbox__new () : (int * int * term * term) msgbox__t);;
let live_eqs = ref ([] : (int * bool ref * bool ref) list);;
let cp = lock__new (queue__new () : (term * term) queue__t);;
let deleted_rules = lock__new (queue__new () : (term * term) queue__t) ;;
let failures = lock__new (queue__new () : (int * term * term) queue__t);;
let stopper_box = flagbox__new ();;
let comparator_fbs = map flagbox__new [(); (); ()];;

let kill_equation_1 nkill =
  let rec k = function
      [] -> failwith "kill_equation_1"
    | (n, b1, b2) :: t -> 
                          if n = nkill then (b1 := true; ())
                          else k t
  in
     k (!live_eqs);
     flagbox__signal stopper_box
;;
let kill_equation_2 nkill =
  let rec k = function
      [] -> failwith "kill_equation_2"
    | (n, b1, b2) :: t -> if n = nkill then (b2 := true; ())
                          else k t
  in k (!live_eqs);
     flagbox__signal stopper_box
;;
let kill_equation nkill = kill_equation_1 nkill; kill_equation_2 nkill;;

let put_failure (vers, M, N) =
   lock__f failures (fun dq -> queue__put dq (vers, M, N));
   flagbox__signal stopper_box
;;

let put_cp (M, N) =
   lock__f cp (fun dq -> queue__put dq (M, N));
   flagbox__signal stopper_box
;;

let put_deleted (M, N) =
   lock__f deleted_rules (fun dq -> queue__put dq (M, N));
   flagbox__signal stopper_box
;;

let num_rules () = match !rule_set with (x, _) -> x;;

let rec orienter greater =
  let (vers, num, M, N) = msgbox__get reduced_eqs
  in
     if M = N then kill_equation num
     else if greater (M, N) then msgbox__put new_rules (vers, num, M, N)
     else if greater (N, M) then msgbox__put new_rules (vers, num, N, M)
     else (put_failure (vers, M, N); kill_equation num);
     orienter greater
;;

let rec reduce_aux rules M n = function
   [] -> M
 | (num, (v, (left, right))) :: t
    -> if num < n then M
       else if reducible left M then mrewrite_all rules M
       else reduce_aux rules M n t
;;

let rec reducer () =
  let (vers, num, M, N) = msgbox__get numbered_eqs
  in
     let (n, rules) = !rule_set
  in let M1 = reduce_aux rules M vers rules
     and N1 = reduce_aux rules N vers rules
  in msgbox__put reduced_eqs (n, num, M1, N1);
     reducer ()
;;

exception Nothing;;
let eq_count = ref 0;;

let rec purge = function
    [] -> []
  | (_, ref true, ref true) :: l -> purge l
  | x :: l -> x :: purge l
;;

(* Do NOT lauch several stoppers in parallel ! *)
let rec stopper () =
  flagbox__wait stopper_box;
  begin try while true
      do let (vers, (M, N))
               = try lock__f cp (fun dq -> (-1, queue__get dq))
                 with queue__Empty ->
                 try lock__f deleted_rules (fun dq -> (-1, queue__get dq))
                 with queue__Empty ->
                 lock__f failures (fun dq ->
                       let (vers, M, N) = queue__get dq
                       in if vers >= num_rules ()
                          then raise queue__Empty
                          else (vers, (M, N)))
         in eq_count := !eq_count + 1;
            live_eqs := (!eq_count, ref false, ref false) :: !live_eqs;
            msgbox__put numbered_eqs (vers, !eq_count, M, N)
      done
  with queue__Empty -> ()
  end;
  live_eqs := purge (!live_eqs);
  if length (!live_eqs) = 0
  then ()
  else stopper ()
;;

let rec comparator rules_fb =
  let (num, (vers, (v, el))) = msgbox__get confirmed_rules
  in
      let rec mutual = function (n, (_, ek)) :: l ->
                if n <= vers
                then do_list put_cp (mutual_critical_pairs el (rename v ek));
                mutual l
             | [] -> ()
  in let rec loop () =
             let (n, rules) = !rule_set
             in if n < vers
                then (flagbox__wait rules_fb; loop ())
                else (mutual (rev rules);
                      do_list put_cp (strict_critical_pairs el (rename v el));
                      kill_equation_2 num)
  in loop ();
     comparator rules_fb
;;

let rec reducible_new M N vers = function
    [] -> false
  | (n, (v, (left, right))) :: t
       -> if vers > n then reducible_new M N vers t
          else reducible left M or reducible left N or reducible_new M N vers t
;;

let rec red M rules = function
    [] -> []
  | ((n, (v, (left, right))) as r) :: t
      -> if reducible M left
         then (put_deleted (left, right);
               deletion_message n;
               red M rules t)
         else (n, mk_rule left (mrewrite_all rules right)) :: red M rules t
;;

(* Do NOT lauch several mixers in parallel ! *)
let rec mixer () =
  let (vers, num, M, N) = msgbox__get new_rules
  in
     let (n, rules) = !rule_set
  in if vers < n & reducible_new M N vers rules
     then (put_cp (M, N);
           kill_equation num)
     else (let r = (n, mk_rule M N)
           in msgbox__put confirmed_rules (num, r);
              pretty_rule r;
              rule_set := (n+1, r :: red M (r :: rules) rules);
              kill_equation_1 num;
              do_list flagbox__signal comparator_fbs;
              flagbox__signal stopper_box);
     mixer ()
;;

let inited = ref false;;

let kb_complete greater [] equations =
  let equations = map (fun (_, (_, (tl, tr))) -> (tl, tr)) equations
  in
  if not !inited then begin
   thread__fork mixer;
   do_list (fun fb -> thread__fork (fun () -> comparator fb)) comparator_fbs;
   for i = 0 to 3 do thread__fork (fun () -> orienter greater) done;
   for i = 0 to 5 do thread__fork reducer done;
   inited := true; ()
  end;
  rule_set := (0, []);
  do_list put_cp equations;
  stopper ();
  lock__f failures (fun dq ->
        try queue__get dq;
            message "Non-orientable."
        with queue__Empty
             -> message "Canonical set found :";
                let (_, rules) = !rule_set
                in pretty_rules (rev rules); ())
;;
